@@ -12,15 +12,24 @@
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
     GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+    You should have received a copy of the GNU General Public License along
+    with this program; if not, write to the Free Software Foundation, Inc.,
+    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
     As a special exception, permission is given to link this program
     with any edition of Qt, and distribute the resulting executable,
     without including the source code for Qt in the source distribution.
 */
 
+#include "imagewidget.h"
+
+#include <QtGui/QGroupBox>
+#include <QtGui/QImage>
+#include <QtGui/QMenu>
+#include <QtGui/QMouseEvent>
+#include <QtGui/QPixmap>
+#include <QtGui/QVBoxLayout>
+
 #include <kabc/picture.h>
 #include <kdebug.h>
 #include <kdialog.h>
@@ -31,34 +40,14 @@
 #include <kio/netaccess.h>
 #include <klocale.h>
 #include <kmessagebox.h>
-#include <kurldrag.h>
-#include <libkdepim/kpixmapregionselectordialog.h>
+#include <kpixmapregionselectordialog.h>
 
-#include <librss/loader.h>
-#include <librss/document.h>
-#include <librss/image.h>
-
-#include <qapplication.h>
-#include <qdragobject.h>
-#include <qeventloop.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qpixmap.h>
-#include <qpopupmenu.h>
-
-#include <unistd.h>
-
-#include "imagewidget.h"
-
-ImageLoader::ImageLoader()
-  : QObject( 0, "ImageLoader" ),
-    mIsLoadingBlog( false )
+ImageLoader::ImageLoader( QWidget *parent )
+  : QObject( 0 ), mParent( parent )
 {
 }
 
-
-KABC::Picture ImageLoader::loadPicture( const KURL &url, bool *ok )
+KABC::Picture ImageLoader::loadPicture( const KUrl &url, bool *ok )
 {
   KABC::Picture picture;
   QString tempFile;
@@ -70,36 +59,37 @@ KABC::Picture ImageLoader::loadPicture( const KURL &url, bool *ok )
 
   QImage image;
   if ( url.isLocalFile() ) {
-    image.load( url.path() );
-    picture.setData( image );
-    (*ok) = true;
-  } else if ( KIO::NetAccess::download( url, tempFile, 0 ) ) {
-    image.load( tempFile );
-    picture.setData( image );
-    (*ok) = true;
+    if ( image.load( url.toLocalFile() ) ) {
+      picture.setData( image );
+      (*ok) = true;
+    }
+  } else if ( KIO::NetAccess::download( url, tempFile, mParent ) ) {
+    if ( image.load( tempFile ) ) {
+      picture.setData( image );
+      (*ok) = true;
+    }
     KIO::NetAccess::removeTempFile( tempFile );
   }
 
   if ( !(*ok) ) {
     // image does not exist (any more)
-    KMessageBox::sorry( 0, i18n( "This contact's image cannot be found." ) );
+    KMessageBox::sorry( mParent, i18n( "This contact's image cannot be found." ) );
     return picture;
   }
 
-  QPixmap pixmap = picture.data();
+  QPixmap pixmap = QPixmap::fromImage( picture.data() );
 
-  QPixmap selectedPixmap = KPIM::KPixmapRegionSelectorDialog::getSelectedImage( pixmap, 100, 140, 0 );
-  if ( selectedPixmap.isNull() ) {
+  image = KPixmapRegionSelectorDialog::getSelectedImage( pixmap, 100, 140, mParent );
+  if ( image.isNull() ) {
     (*ok) = false;
     return picture;
   }
 
-  image = selectedPixmap;
   if ( image.height() != 140 || image.width() != 100 ) {
     if ( image.height() > image.width() )
-      image = image.scaleHeight( 140 );
+      image = image.scaledToHeight( 140 );
     else
-      image = image.scaleWidth( 100 );
+      image = image.scaledToWidth( 100 );
   }
 
   picture.setData( image );
@@ -108,56 +98,12 @@ KABC::Picture ImageLoader::loadPicture( const KURL &url, bool *ok )
   return picture;
 }
 
-KABC::Picture ImageLoader::loadBlog( const KURL &url, bool *ok )
-{
-  RSS::Loader *loader = RSS::Loader::create();
-  connect( loader, SIGNAL( loadingComplete( Loader*, Document, Status ) ),
-           this, SLOT( loadingComplete( Loader*, Document, Status ) ) );
-  loader->loadFrom( url, new RSS::FileRetriever );
-
-  mIsLoadingBlog = true;
-  while ( mIsLoadingBlog ) {
-    qApp->eventLoop()->processEvents( QEventLoop::ExcludeUserInput );
-    usleep( 500 );
-  }
-
-  if ( mPicture.data().isNull() ) {
-    KMessageBox::sorry( 0,
-        i18n( "Blog feed at '%1' does not contain an image." ).arg( url.url() ) );
-    (*ok) = false;
-  }
-  (*ok) = true;
-
-  return mPicture;
-}
-
-void ImageLoader::loadingComplete( RSS::Loader*,
-                                   RSS::Document doc,
-                                   RSS::Status status )
-{
-  mIsLoadingBlog = false;
-
-  if ( status != RSS::Success ) {
-    mPicture = KABC::Picture();
-    return;
-  }
-
-  if ( !doc.image() ) {
-    mPicture = KABC::Picture();
-    return;
-  }
-
-  bool ok = false;
-  KABC::Picture pic = loadPicture( doc.image()->url().url(), &ok );
-  if ( ok )
-    mPicture = pic;
-}
-
-ImageButton::ImageButton( const QString &title, QWidget *parent )
-  : QPushButton( title, parent ),
+ImageButton::ImageButton( QWidget *parent )
+  : QPushButton( parent ),
     mReadOnly( false ), mImageLoader( 0 )
 {
   setAcceptDrops( true );
+  setIconSize( QSize( 100, 140 ) );
 
   connect( this, SIGNAL( clicked() ), SLOT( load() ) );
 }
@@ -170,7 +116,7 @@ void ImageButton::setReadOnly( bool readOnly )
 void ImageButton::setPicture( const KABC::Picture &picture )
 {
   mPicture = picture;
-  updateGUI();
+  updateGui();
 }
 
 KABC::Picture ImageButton::picture() const
@@ -183,38 +129,29 @@ void ImageButton::setImageLoader( ImageLoader *loader )
   mImageLoader = loader;
 }
 
-void ImageButton::setBlogFeed( const KURL &url )
-{
-  mBlogFeed = url;
-}
-
 void ImageButton::startDrag()
 {
   if ( !mPicture.data().isNull() ) {
-    QImageDrag *drag = new QImageDrag( mPicture.data(), this );
-    drag->dragCopy();
+    QDrag *drag = new QDrag( this );
+    drag->setMimeData( new QMimeData() );
+    drag->mimeData()->setImageData( mPicture.data() );
+    drag->start();
   }
 }
 
-void ImageButton::updateGUI()
+void ImageButton::updateGui()
 {
-  if ( mPicture.data().isNull() )
-    setPixmap( KGlobal::iconLoader()->iconPath( "personal", KIcon::Desktop ) );
-  else
-    setPixmap( mPicture.data() );
+  if ( mPicture.data().isNull() ) {
+    setIcon( KIcon( "user-identity" ) );
+  } else {
+    setIcon( QPixmap::fromImage( mPicture.data() ) );
+  }
 }
 
 void ImageButton::dragEnterEvent( QDragEnterEvent *event )
 {
-  bool accepted = false;
-
-  if ( QImageDrag::canDecode( event ) )
-    accepted = true;
-
-  if ( QUriDrag::canDecode( event ) )
-    accepted = true;
-
-  event->accept( accepted );
+  const QMimeData *md = event->mimeData();
+  event->setAccepted( md->hasImage() || md->hasUrls());
 }
 
 void ImageButton::dropEvent( QDropEvent *event )
@@ -222,31 +159,24 @@ void ImageButton::dropEvent( QDropEvent *event )
   if ( mReadOnly )
     return;
 
-  if ( QImageDrag::canDecode( event ) ) {
-    QPixmap pm;
-
-    if ( QImageDrag::decode( event, pm ) ) {
-      mPicture.setData( pm.convertToImage() );
-      updateGUI();
-      emit changed();
-    }
+  const QMimeData *md = event->mimeData();
+  if ( md->hasImage() ) {
+    QImage image = qvariant_cast<QImage>(md->imageData());
+    mPicture.setData( image );
+    updateGui();
+    emit changed();
   }
 
-  if ( QUriDrag::canDecode( event ) ) {
-    KURL::List urls;
-    if ( KURLDrag::decode( event, urls ) ) {
-      if ( urls.isEmpty() ) { // oops, no data
-        event->accept( false );
-        return;
-      }
-    }
-
+  KUrl::List urls = KUrl::List::fromMimeData( md );
+  if ( urls.isEmpty() ) { // oops, no data
+    event->setAccepted( false );
+  } else {
     if ( mImageLoader ) {
       bool ok = false;
-      KABC::Picture pic = mImageLoader->loadPicture( urls[ 0 ], &ok );
+      KABC::Picture pic = mImageLoader->loadPicture( urls.first(), &ok );
       if ( ok ) {
         mPicture = pic;
-        updateGUI();
+        updateGui();
         emit changed();
       }
     }
@@ -261,7 +191,7 @@ void ImageButton::mousePressEvent( QMouseEvent *event )
 
 void ImageButton::mouseMoveEvent( QMouseEvent *event )
 {
-  if ( (event->state() & LeftButton) &&
+  if ( (event->buttons() & Qt::LeftButton) &&
        (event->pos() - mDragStartPos).manhattanLength() >
        KGlobalSettings::dndEventDelay() ) {
     startDrag();
@@ -270,63 +200,54 @@ void ImageButton::mouseMoveEvent( QMouseEvent *event )
 
 void ImageButton::contextMenuEvent( QContextMenuEvent *event )
 {
-  QPopupMenu menu( this );
-  menu.insertItem( i18n( "Reset" ), this, SLOT( clear() ) );
-  if ( mBlogFeed.isValid() )
-    menu.insertItem( i18n("Get From Blog"), this, SLOT( loadBlog() ) );
-
+  QMenu menu;
+  menu.addAction( i18n( "Reset" ), this, SLOT( clear() ) );
   menu.exec( event->globalPos() );
 }
 
 void ImageButton::load()
 {
-  KURL url = KFileDialog::getOpenURL( QString(), KImageIO::pattern(), this );
+  if ( mReadOnly )
+    return;
+  KUrl url = KFileDialog::getOpenUrl( QString(), KImageIO::pattern(), this );
   if ( url.isValid() ) {
     if ( mImageLoader ) {
       bool ok = false;
       KABC::Picture pic = mImageLoader->loadPicture( url, &ok );
       if ( ok ) {
         mPicture = pic;
-        updateGUI();
+        updateGui();
         emit changed();
       }
     }
   }
 }
 
-void ImageButton::loadBlog()
-{
-  bool ok = false;
-  KABC::Picture pic = mImageLoader->loadBlog( mBlogFeed, &ok );
-  if ( ok ) {
-    mPicture = pic;
-    updateGUI();
-    emit changed();
-  }
-}
-
 void ImageButton::clear()
 {
   mPicture = KABC::Picture();
-  updateGUI();
+  updateGui();
 
   emit changed();
 }
 
-ImageBaseWidget::ImageBaseWidget( const QString &title,
-                                  QWidget *parent, const char *name )
-  : QWidget( parent, name ), mReadOnly( false )
+ImageBaseWidget::ImageBaseWidget( const QString &title, QWidget *parent )
+  : QWidget( parent ), mReadOnly( false )
 {
-  mImageLoader = new ImageLoader();
+  mImageLoader = new ImageLoader( this );
 
-  QVBoxLayout *topLayout = new QVBoxLayout( this, KDialog::marginHint(),
-                                            KDialog::spacingHint() );
-  QGroupBox *box = new QGroupBox( 0, Qt::Vertical, title, this );
-  QVBoxLayout *layout = new QVBoxLayout( box->layout(), KDialog::spacingHint() );
+  QVBoxLayout *topLayout = new QVBoxLayout( this );
+  topLayout->setMargin( KDialog::marginHint() );
+  topLayout->setSpacing( KDialog::spacingHint() );
 
-  mImageButton = new ImageButton( i18n( "Picture" ), box );
-  mImageButton->setFixedSize( 100, 140 );
+  QGroupBox *box = new QGroupBox( title, this );
+  QVBoxLayout *layout = new QVBoxLayout( box );
+  layout->setSpacing( KDialog::spacingHint() );
+
+  mImageButton = new ImageButton( box );
+  mImageButton->setFixedSize( 120, 160 );
   mImageButton->setImageLoader( mImageLoader );
+
   layout->addWidget( mImageButton );
 
   topLayout->addWidget( box );
@@ -346,11 +267,6 @@ void ImageBaseWidget::setReadOnly( bool readOnly )
   mImageButton->setReadOnly( mReadOnly );
 }
 
-void ImageBaseWidget::setBlogFeed( const QString &feed )
-{
-  mImageButton->setBlogFeed( feed );
-}
-
 void ImageBaseWidget::setImage( const KABC::Picture &photo )
 {
   mImageButton->setPicture( photo );
@@ -362,11 +278,12 @@ KABC::Picture ImageBaseWidget::image() const
 }
 
 
-ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent, const char *name )
-  : KAB::ContactEditorWidget( ab, parent, name )
+ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent )
+  : KAB::ContactEditorWidget( ab, parent )
 {
-  QHBoxLayout *layout = new QHBoxLayout( this, KDialog::marginHint(),
-                                         KDialog::spacingHint() );
+  QHBoxLayout *layout = new QHBoxLayout( this );
+  layout->setSpacing( KDialog::spacingHint() );
+  layout->setMargin( 0 );
 
   mPhotoWidget = new ImageBaseWidget( KABC::Addressee::photoLabel(), this );
   layout->addWidget( mPhotoWidget );
@@ -381,7 +298,6 @@ ImageWidget::ImageWidget( KABC::AddressBook *ab, QWidget *parent, const char *na
 void ImageWidget::loadContact( KABC::Addressee *addr )
 {
   mPhotoWidget->setImage( addr->photo() );
-  mPhotoWidget->setBlogFeed( addr->custom( "KADDRESSBOOK", "BlogFeed" ) );
   mLogoWidget->setImage( addr->logo() );
 }
 
