@@ -24,14 +24,18 @@
 #ifndef KADDRESSBOOKSERVICE_H
 #define KADDRESSBOOKSERVICE_H
 
-#include <libkdepim/interfaces/AddressBookServiceIface.h>
+#include <QtCore/QObject>
+
+class KUrl;
 
 namespace KAB {
 class Core;
 }
 
-class KAddressBookService : virtual public KPIM::AddressBookServiceIface
+class KAddressBookService : public QObject
 {
+  Q_OBJECT
+
   public:
     KAddressBookService( KAB::Core *core );
 
@@ -40,14 +44,14 @@ class KAddressBookService : virtual public KPIM::AddressBookServiceIface
 
       @param vCard The vCard in string representation.
      */
-    void importVCard( const QString& vCard );
+    void importVCardFromData( const QString& vCard );
 
     /**
       This method will add a vcard to the address book.
 
       @param url The url where the vcard is located.
      */
-    void importVCard( const KURL& url );
+    void importVCard( const KUrl& url );
 
   private:
     KAB::Core *mCore;
