@@ -22,27 +22,29 @@
     without including the source code for Qt in the source distribution.
 */
 
-#include <qbuttongroup.h>
-#include <qlabel.h>
-#include <qlayout.h>
-#include <qpixmap.h>
-#include <qradiobutton.h>
-
-#include <kcombobox.h>
-#include <kdialog.h>
-#include <klocale.h>
-
 #include "stylepage.h"
 
+#include <QtGui/QGridLayout>
+#include <QtGui/QGroupBox>
+#include <QtGui/QLabel>
+#include <QtGui/QPixmap>
+#include <QtGui/QVBoxLayout>
+
+#include <kabc/addressbook.h>
+#include <KComboBox>
+#include <KDialog>
+#include <KLocale>
+
 StylePage::StylePage( KABC::AddressBook *ab, QWidget* parent,  const char* name )
-  : QWidget( parent, name ), mAddressBook( ab )
+  : QWidget( parent ), mAddressBook( ab )
 {
+  setObjectName( name );
   initGUI();
 
   initFieldCombo();
 
-  mSortTypeCombo->insertItem( i18n( "Ascending" ) );
-  mSortTypeCombo->insertItem( i18n( "Descending" ) );
+  mSortTypeCombo->addItem( i18n( "Ascending" ) );
+  mSortTypeCombo->addItem( i18n( "Descending" ) );
 
   connect( mStyleCombo, SIGNAL( activated( int ) ), SIGNAL( styleChanged( int ) ) );
 }
@@ -61,7 +63,7 @@ void StylePage::setPreview( const QPixmap &pixmap )
 
 void StylePage::addStyleName( const QString &name )
 {
-  mStyleCombo->insertItem( name );
+  mStyleCombo->addItem( name );
 }
 
 void StylePage::clearStyleNames()
@@ -71,28 +73,28 @@ void StylePage::clearStyleNames()
 
 void StylePage::setSortField( KABC::Field *field )
 {
-  mFieldCombo->setCurrentText( field->label() );
+  mFieldCombo->setItemText( mFieldCombo->currentIndex(), field->label() );
 }
 
 void StylePage::setSortAscending( bool value )
 {
   if ( value )
-    mSortTypeCombo->setCurrentItem( 0 );
+    mSortTypeCombo->setCurrentIndex( 0 );
   else
-    mSortTypeCombo->setCurrentItem( 1 );
+    mSortTypeCombo->setCurrentIndex( 1 );
 }
 
 KABC::Field* StylePage::sortField()
 {
-  if ( mFieldCombo->currentItem() == -1 )
+  if ( mFieldCombo->currentIndex() == -1 )
     return mFields[ 0 ];
 
-  return mFields[ mFieldCombo->currentItem() ];
+  return mFields[ mFieldCombo->currentIndex() ];
 }
 
 bool StylePage::sortAscending()
 {
-  return ( mSortTypeCombo->currentItem() == 0 );
+  return ( mSortTypeCombo->currentIndex() == 0 );
 }
 
 void StylePage::initFieldCombo()
@@ -104,26 +106,28 @@ void StylePage::initFieldCombo()
 
   mFields = mAddressBook->fields( KABC::Field::All );
   KABC::Field::List::ConstIterator it;
-  for ( it = mFields.begin(); it != mFields.end(); ++it )
-    mFieldCombo->insertItem( (*it)->label() );
+  for ( it = mFields.constBegin(); it != mFields.constEnd(); ++it )
+    mFieldCombo->addItem( (*it)->label() );
 }
 
 void StylePage::initGUI()
 {
-  setCaption( i18n( "Choose Printing Style" ) );
+  setWindowTitle( i18n( "Choose Printing Style" ) );
 
-  QGridLayout *topLayout = new QGridLayout( this, 2, 2, KDialog::marginHint(),
-                                            KDialog::spacingHint() );
+  QGridLayout *topLayout = new QGridLayout( this );
+  topLayout->setSpacing( KDialog::spacingHint() );
+  topLayout->setMargin( KDialog::marginHint() );
 
   QLabel *label = new QLabel( i18n( "What should the print look like?\n"
                                     "KAddressBook has several printing styles, designed for different purposes.\n"
                                     "Choose the style that suits your needs below." ), this );
-  topLayout->addMultiCellWidget( label, 0, 0, 0, 1 );
+  topLayout->addWidget( label, 0, 0, 1, 2 );
 
-  QButtonGroup *group = new QButtonGroup( i18n( "Sorting" ), this );
-  group->setColumnLayout( 0, Qt::Vertical );
-  QGridLayout *sortLayout = new QGridLayout( group->layout(), 2, 2,
-                                             KDialog::spacingHint() );
+  QGroupBox *group = new QGroupBox( i18n( "Sorting" ), this );
+  QGridLayout *sortLayout = new QGridLayout();
+  group->setLayout( sortLayout );
+  sortLayout->setSpacing( KDialog::spacingHint() );
+  sortLayout->setMargin( KDialog::marginHint() );
   sortLayout->setAlignment( Qt::AlignTop );
 
   label = new QLabel( i18n( "Criterion:" ), group );
@@ -140,10 +144,11 @@ void StylePage::initGUI()
 
   topLayout->addWidget( group, 1, 0 );
 
-  group = new QButtonGroup( i18n( "Print Style" ), this );
-  group->setColumnLayout( 0, Qt::Vertical );
-  QVBoxLayout *styleLayout = new QVBoxLayout( group->layout(),
-                                              KDialog::spacingHint() );
+  group = new QGroupBox( i18n( "Print Style" ), this );
+  QVBoxLayout *styleLayout = new QVBoxLayout();
+  group->setLayout( styleLayout );
+  styleLayout->setSpacing( KDialog::spacingHint() );
+  styleLayout->setMargin( KDialog::marginHint() );
 
   mStyleCombo = new KComboBox( false, group );
   styleLayout->addWidget( mStyleCombo );
@@ -153,7 +158,8 @@ void StylePage::initGUI()
   font.setPointSize( 20 );
   mPreview->setFont( font );
   mPreview->setScaledContents( true );
-  mPreview->setAlignment( int( QLabel::WordBreak | QLabel::AlignCenter ) );
+  mPreview->setAlignment( Qt::AlignCenter );
+  mPreview->setWordWrap( true );
   styleLayout->addWidget( mPreview );
 
   topLayout->addWidget( group, 1, 1 );
