@@ -1,22 +1,22 @@
 /*
-*  editdlgtypes.cpp  -  dialogs to create or edit alarm or alarm template types
-*  Program:  kalarm
-*  Copyright © 2001-2008 by David Jarvie <djarvie@kde.org>
-*
-*  This program is free software; you can redistribute it and/or modify
-*  it under the terms of the GNU General Public License as published by
-*  the Free Software Foundation; either version 2 of the License, or
-*  (at your option) any later version.
-*
-*  This program is distributed in the hope that it will be useful,
-*  but WITHOUT ANY WARRANTY; without even the implied warranty of
-*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-*  GNU General Public License for more details.
-*
-*  You should have received a copy of the GNU General Public License along
-*  with this program; if not, write to the Free Software Foundation, Inc.,
-*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
-*/
+ *  editdlgtypes.cpp  -  dialogs to create or edit alarm or alarm template types
+ *  Program:  kalarm
+ *  Copyright © 2001-2009 by David Jarvie <djarvie@kde.org>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ */
 
 #include "kalarm.h"
 #include "editdlgtypes.moc"
@@ -28,6 +28,7 @@
 #include "emailidcombo.h"
 #include "fontcolourbutton.h"
 #include "functions.h"
+#include "identities.h"
 #include "kalarmapp.h"
 #include "kamail.h"
 #include "latecancel.h"
@@ -152,6 +153,7 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	mTypeCombo->setFixedSize(mTypeCombo->sizeHint());
 	mTypeCombo->setCurrentIndex(-1);    // ensure slotAlarmTypeChanged() is called when index is set
 	connect(mTypeCombo, SIGNAL(currentIndexChanged(int)), SLOT(slotAlarmTypeChanged(int)));
+	connect(mTypeCombo, SIGNAL(currentIndexChanged(int)), SLOT(contentsChanged()));
 	label->setBuddy(mTypeCombo);
 	box->setWhatsThis(i18nc("@info:whatsthis", "<para>Select what the alarm should display:"
 	      "<list><item><interface>%1</interface>: the alarm will display the text message you type in.</item>"
@@ -165,6 +167,7 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	mTextMessageEdit = new TextEdit(parent);
 	mTextMessageEdit->setLineWrapMode(KTextEdit::NoWrap);
 	mTextMessageEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the text of the alarm message. It may be multi-line."));
+	connect(mTextMessageEdit, SIGNAL(textChanged()), SLOT(contentsChanged()));
 	frameLayout->addWidget(mTextMessageEdit);
 
 	// File name edit box
@@ -174,6 +177,7 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	mFileMessageEdit = new LineEdit(LineEdit::Url, mFileBox);
 	mFileMessageEdit->setAcceptDrops(true);
 	mFileMessageEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the name or URL of a text or image file to display."));
+	connect(mFileMessageEdit, SIGNAL(textChanged(const QString&)), SLOT(contentsChanged()));
 
 	// File browse button
 	mFileBrowseButton = new QPushButton(mFileBox);
@@ -186,50 +190,35 @@ void EditDisplayAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
 	// Command type checkbox and edit box
 	mCmdEdit = new CommandEdit(parent);
-	frameLayout->addWidget(mCmdEdit);
 	connect(mCmdEdit, SIGNAL(scriptToggled(bool)), SLOT(slotCmdScriptToggled(bool)));
+	connect(mCmdEdit, SIGNAL(changed()), SLOT(contentsChanged()));
+	frameLayout->addWidget(mCmdEdit);
 
-	// Background colour for file display alarms
+	// Sound checkbox and file selector
 	QHBoxLayout* hlayout = new QHBoxLayout();
 	hlayout->setMargin(0);
 	frameLayout->addLayout(hlayout);
-	mBgColourBox = new KHBox(parent);
-	mBgColourBox->setMargin(0);
-	mBgColourBox->setSpacing(KDialog::spacingHint());
-	hlayout->addWidget(mBgColourBox);
-	hlayout->addStretch();
-	label = new QLabel(i18nc("@label:listbox", "Background color:"), mBgColourBox);
-	mBgColourButton = new ColourButton(mBgColourBox);
-	label->setBuddy(mBgColourButton);
-	mBgColourBox->setWhatsThis(i18nc("@info:whatsthis", "Select the alarm message background color"));
-
-	// Sound checkbox and file selector
-	mSpecialActionsLayout1 = new QHBoxLayout();
-	mSpecialActionsLayout1->setMargin(0);
-	frameLayout->addLayout(mSpecialActionsLayout1);
 	mSoundPicker = new SoundPicker(parent);
 	mSoundPicker->setFixedSize(mSoundPicker->sizeHint());
-	mSpecialActionsLayout1->addWidget(mSoundPicker);
-	mSpecialActionsLayout1->addSpacing(2*spacingHint());
-	mSpecialActionsLayout1->addStretch();
+	connect(mSoundPicker, SIGNAL(changed()), SLOT(contentsChanged()));
+	hlayout->addWidget(mSoundPicker);
+	hlayout->addSpacing(2*spacingHint());
+	hlayout->addStretch();
 
 	// Font and colour choice button and sample text
 	mFontColourButton = new FontColourButton(parent);
 	mFontColourButton->setMaximumHeight(mFontColourButton->sizeHint().height() * 3/2);
-	mSpecialActionsLayout1->addWidget(mFontColourButton);
+	hlayout->addWidget(mFontColourButton);
 	connect(mFontColourButton, SIGNAL(selected(const QColor&, const QColor&)), SLOT(setColours(const QColor&, const QColor&)));
+	connect(mFontColourButton, SIGNAL(selected(const QColor&, const QColor&)), SLOT(contentsChanged()));
 
 	if (ShellProcess::authorised())    // don't display if shell commands not allowed (e.g. kiosk mode)
 	{
 		// Special actions button
-		mSpecialActionsLayout2 = new QHBoxLayout();
-		mSpecialActionsLayout2->setMargin(0);
-		frameLayout->addLayout(mSpecialActionsLayout2);
-		mSpecialActionsLayout2->addStretch();
 		mSpecialActionsButton = new SpecialActionsButton(false, parent);
 		mSpecialActionsButton->setFixedSize(mSpecialActionsButton->sizeHint());
-		mSpecialActionsLayout2->addWidget(mSpecialActionsButton);
-		mSpecialActionsInLayout1 = false;
+		connect(mSpecialActionsButton, SIGNAL(selected()), SLOT(contentsChanged()));
+		frameLayout->addWidget(mSpecialActionsButton, 0, Qt::AlignRight);
 	}
 
 	// Top-adjust the controls
@@ -279,7 +268,6 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
 			mFontColourButton->setFont(event->font());
 		mFontColourButton->setBgColour(event->bgColour());
 		mFontColourButton->setFgColour(event->fgColour());
-		mBgColourButton->setColour(event->bgColour());
 		setColours(event->fgColour(), event->bgColour());
 		mConfirmAck->setChecked(event->confirmAck());
 		bool recurs = event->recurs();
@@ -320,7 +308,6 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
 		mFontColourButton->setDefaultFont();
 		mFontColourButton->setBgColour(Preferences::defaultBgColour());
 		mFontColourButton->setFgColour(Preferences::defaultFgColour());
-		mBgColourButton->setColour(Preferences::defaultBgColour());
 		setColours(Preferences::defaultFgColour(), Preferences::defaultBgColour());
 		mConfirmAck->setChecked(Preferences::defaultConfirmAck());
 		reminder()->setMinutes(0, false);
@@ -332,6 +319,31 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
 	}
 }
 
+/******************************************************************************
+* Called when the More/Less Options button is clicked.
+* Show/hide the optional options.
+*/
+void EditDisplayAlarmDlg::type_showOptions(bool more)
+{
+	if (mSpecialActionsButton)
+	{
+		if (more)
+		{
+			switch (mTypeCombo->currentIndex())
+			{
+				case tFILE:
+				case tTEXT:
+					mSpecialActionsButton->show();
+					break;
+				default:
+					break;
+			}
+		}
+		else
+			mSpecialActionsButton->hide();
+	}
+}
+
 /******************************************************************************
 * Called when the font/color button has been clicked.
 * Set the colors in the message text entry control.
@@ -380,6 +392,41 @@ void EditDisplayAlarmDlg::setAction(KAEvent::Action action, const AlarmText& ala
 	}
 }
 
+/******************************************************************************
+* Initialise various values in the New Alarm dialogue.
+*/
+void EditDisplayAlarmDlg::setBgColour(const QColor& colour)
+{
+	mFontColourButton->setBgColour(colour);
+	setColours(mFontColourButton->fgColour(), colour);
+}
+void EditDisplayAlarmDlg::setFgColour(const QColor& colour)
+{
+	mFontColourButton->setFgColour(colour);
+	setColours(colour, mFontColourButton->bgColour());
+}
+void EditDisplayAlarmDlg::setConfirmAck(bool confirm)
+{
+	mConfirmAck->setChecked(confirm);
+}
+void EditDisplayAlarmDlg::setAutoClose(bool close)
+{
+	lateCancel()->setAutoClose(close);
+}
+void EditDisplayAlarmDlg::setAudio(Preferences::SoundType type, const QString& file, float volume, bool repeat)
+{
+	mSoundPicker->set(type, file, volume, -1, 0, repeat);
+}
+void EditDisplayAlarmDlg::setReminder(int minutes)
+{
+	bool onceOnly = (minutes < 0);
+	if (onceOnly)
+		minutes = -minutes;
+	reminder()->setMinutes(minutes, dateOnly());
+	reminder()->setOnceOnly(onceOnly);
+	reminder()->enableOnceOnly(isTimedRecurrence());
+}
+
 /******************************************************************************
 * Set the read-only status of all non-template controls.
 */
@@ -390,7 +437,6 @@ void EditDisplayAlarmDlg::setReadOnly(bool readOnly)
 	mFileMessageEdit->setReadOnly(readOnly);
 	mCmdEdit->setReadOnly(readOnly);
 	mFontColourButton->setReadOnly(readOnly);
-	mBgColourButton->setReadOnly(readOnly);
 	mSoundPicker->setReadOnly(readOnly);
 	mConfirmAck->setReadOnly(readOnly);
 	reminder()->setReadOnly(readOnly);
@@ -418,7 +464,7 @@ void EditDisplayAlarmDlg::saveState(const KAEvent* event)
 	mSavedConfirmAck  = mConfirmAck->isChecked();
 	mSavedFont        = mFontColourButton->font();
 	mSavedFgColour    = mFontColourButton->fgColour();
-	mSavedBgColour    = (mTypeCombo->currentIndex() == tFILE) ? mBgColourButton->colour() : mFontColourButton->bgColour();
+	mSavedBgColour    = mFontColourButton->bgColour();
 	mSavedReminder    = reminder()->minutes();
 	mSavedOnceOnly    = reminder()->isOnceOnly();
 	mSavedAutoClose   = lateCancel()->isAutoClose();
@@ -444,7 +490,7 @@ bool EditDisplayAlarmDlg::type_stateChanged() const
 	||  mSavedConfirmAck != mConfirmAck->isChecked()
 	||  mSavedFont       != mFontColourButton->font()
 	||  mSavedFgColour   != mFontColourButton->fgColour()
-	||  mSavedBgColour   != (mTypeCombo->currentIndex() == tFILE ? mBgColourButton->colour() : mFontColourButton->bgColour())
+	||  mSavedBgColour   != mFontColourButton->bgColour()
 	||  mSavedReminder   != reminder()->minutes()
 	||  mSavedOnceOnly   != reminder()->isOnceOnly()
 	||  mSavedAutoClose  != lateCancel()->isAutoClose())
@@ -488,8 +534,7 @@ void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, con
 		default:
 		case tTEXT:     type = KAEvent::MESSAGE; break;
 	}
-	event.set(dt, text, (mTypeCombo->currentIndex() == tFILE ? mBgColourButton->colour() : mFontColourButton->bgColour()),
-	          mFontColourButton->fgColour(), mFontColourButton->font(),
+	event.set(dt, text, mFontColourButton->bgColour(), mFontColourButton->fgColour(), mFontColourButton->font(),
 	          type, lateCancel, getAlarmFlags());
 	if (type == KAEvent::MESSAGE)
 	{
@@ -537,19 +582,9 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 			mFilePadding->hide();
 			mCmdEdit->hide();
 			mTextMessageEdit->show();
-			mFontColourButton->show();
-			mBgColourBox->hide();
 			mSoundPicker->showSpeak(true);
-			if (mSpecialActionsButton)
-			{
-				if (mSpecialActionsInLayout1)
-				{
-					mSpecialActionsLayout1->removeWidget(mSpecialActionsButton);
-					mSpecialActionsLayout2->addWidget(mSpecialActionsButton);
-					mSpecialActionsInLayout1 = false;
-				}
+			if (mSpecialActionsButton  &&  showingMore())
 				mSpecialActionsButton->show();
-			}
 			setButtonWhatsThis(Try, i18nc("@info:whatsthis", "Display the alarm message now"));
 			focus = mTextMessageEdit;
 			break;
@@ -558,19 +593,9 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 			mFileBox->show();
 			mFilePadding->show();
 			mCmdEdit->hide();
-			mFontColourButton->hide();
-			mBgColourBox->show();
 			mSoundPicker->showSpeak(false);
-			if (mSpecialActionsButton)
-			{
-				if (!mSpecialActionsInLayout1)
-				{
-					mSpecialActionsLayout2->removeWidget(mSpecialActionsButton);
-					mSpecialActionsLayout1->addWidget(mSpecialActionsButton);
-					mSpecialActionsInLayout1 = true;
-				}
+			if (mSpecialActionsButton  &&  showingMore())
 				mSpecialActionsButton->show();
-			}
 			setButtonWhatsThis(Try, i18nc("@info:whatsthis", "Display the file now"));
 			mFileMessageEdit->setNoSelect();
 			focus = mFileMessageEdit;
@@ -580,8 +605,6 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 			mFileBox->hide();
 			slotCmdScriptToggled(mCmdEdit->isScript());  // show/hide mFilePadding
 			mCmdEdit->show();
-			mFontColourButton->show();
-			mBgColourBox->hide();
 			mSoundPicker->showSpeak(true);
 			if (mSpecialActionsButton)
 				mSpecialActionsButton->hide();
@@ -601,7 +624,11 @@ void EditDisplayAlarmDlg::slotPickFile()
 	static QString defaultDir;   // default directory for file browse button
 	QString file = KAlarm::browseFile(i18nc("@title:window", "Choose Text or Image File to Display"),
 	                                  defaultDir, mFileMessageEdit->text(), QString(), KFile::ExistingOnly, this);
-	mFileMessageEdit->setText(file);
+	if (!file.isEmpty())
+	{
+		mFileMessageEdit->setText(file);
+		contentsChanged();
+	}
 }
 
 /******************************************************************************
@@ -764,31 +791,34 @@ void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 
 	mCmdEdit = new CommandEdit(parent);
 	connect(mCmdEdit, SIGNAL(scriptToggled(bool)), SLOT(slotCmdScriptToggled(bool)));
+	connect(mCmdEdit, SIGNAL(changed()), SLOT(contentsChanged()));
 	frameLayout->addWidget(mCmdEdit);
 
 	// What to do with command output
 
-	QGroupBox* cmdOutputBox = new QGroupBox(i18nc("@title:group", "Command Output"), parent);
-	frameLayout->addWidget(cmdOutputBox);
-	QVBoxLayout* vlayout = new QVBoxLayout(cmdOutputBox);
+	mCmdOutputBox = new QGroupBox(i18nc("@title:group", "Command Output"), parent);
+	frameLayout->addWidget(mCmdOutputBox);
+	QVBoxLayout* vlayout = new QVBoxLayout(mCmdOutputBox);
 	vlayout->setMargin(marginHint());
 	vlayout->setSpacing(spacingHint());
-	mCmdOutputGroup = new ButtonGroup(cmdOutputBox);
+	mCmdOutputGroup = new ButtonGroup(mCmdOutputBox);
+	connect(mCmdOutputGroup, SIGNAL(buttonSet(QAbstractButton*)), SLOT(contentsChanged()));
 
 	// Execute in terminal window
-	mCmdExecInTerm = new RadioButton(i18n_radio_ExecInTermWindow(), cmdOutputBox);
+	mCmdExecInTerm = new RadioButton(i18n_radio_ExecInTermWindow(), mCmdOutputBox);
 	mCmdExecInTerm->setFixedSize(mCmdExecInTerm->sizeHint());
 	mCmdExecInTerm->setWhatsThis(i18nc("@info:whatsthis", "Check to execute the command in a terminal window"));
 	mCmdOutputGroup->addButton(mCmdExecInTerm, Preferences::Log_Terminal);
 	vlayout->addWidget(mCmdExecInTerm, 0, Qt::AlignLeft);
 
 	// Log file name edit box
-	KHBox* box = new KHBox(cmdOutputBox);
+	KHBox* box = new KHBox(mCmdOutputBox);
 	box->setMargin(0);
 	(new QWidget(box))->setFixedWidth(mCmdExecInTerm->style()->pixelMetric(QStyle::PM_ExclusiveIndicatorWidth));   // indent the edit box
 	mCmdLogFileEdit = new LineEdit(LineEdit::Url, box);
 	mCmdLogFileEdit->setAcceptDrops(true);
 	mCmdLogFileEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the name or path of the log file."));
+	connect(mCmdLogFileEdit, SIGNAL(textChanged(const QString&)), SLOT(contentsChanged()));
 
 	// Log file browse button.
 	// The file browser dialog is activated by the PickLogFileRadio class.
@@ -800,15 +830,16 @@ void EditCommandAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	browseButton->setWhatsThis(i18nc("@info:whatsthis", "Select a log file."));
 
 	// Log output to file
-	mCmdLogToFile = new PickLogFileRadio(browseButton, mCmdLogFileEdit, i18nc("@option:radio", "Log to file"), mCmdOutputGroup, cmdOutputBox);
+	mCmdLogToFile = new PickLogFileRadio(browseButton, mCmdLogFileEdit, i18nc("@option:radio", "Log to file"), mCmdOutputGroup, mCmdOutputBox);
 	mCmdLogToFile->setFixedSize(mCmdLogToFile->sizeHint());
 	mCmdLogToFile->setWhatsThis(i18nc("@info:whatsthis", "Check to log the command output to a local file. The output will be appended to any existing contents of the file."));
+	connect(mCmdLogToFile, SIGNAL(fileChanged()), SLOT(contentsChanged()));
 	mCmdOutputGroup->addButton(mCmdLogToFile, Preferences::Log_File);
 	vlayout->addWidget(mCmdLogToFile, 0, Qt::AlignLeft);
 	vlayout->addWidget(box);
 
 	// Discard output
-	mCmdDiscardOutput = new RadioButton(i18nc("@option:radio", "Discard"), cmdOutputBox);
+	mCmdDiscardOutput = new RadioButton(i18nc("@option:radio", "Discard"), mCmdOutputBox);
 	mCmdDiscardOutput->setFixedSize(mCmdDiscardOutput->sizeHint());
 	mCmdDiscardOutput->setWhatsThis(i18nc("@info:whatsthis", "Check to discard command output."));
 	mCmdOutputGroup->addButton(mCmdDiscardOutput, Preferences::Log_Discard);
@@ -846,6 +877,18 @@ void EditCommandAlarmDlg::type_initValues(const KAEvent* event)
 	slotCmdScriptToggled(mCmdEdit->isScript());
 }
 
+/******************************************************************************
+* Called when the More/Less Options button is clicked.
+* Show/hide the optional options.
+*/
+void EditCommandAlarmDlg::type_showOptions(bool more)
+{
+	if (more)
+		mCmdOutputBox->show();
+	else
+		mCmdOutputBox->hide();
+}
+
 /******************************************************************************
 * Set the dialog's action and the action's text.
 */
@@ -1056,10 +1099,11 @@ void EditEmailAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 		label->setFixedSize(label->sizeHint());
 		grid->addWidget(label, 0, 0);
 
-		mEmailFromList = new EmailIdCombo(KAMail::identityManager(), parent);
+		mEmailFromList = new EmailIdCombo(Identities::identityManager(), parent);
 		mEmailFromList->setMinimumSize(mEmailFromList->sizeHint());
 		label->setBuddy(mEmailFromList);
 		mEmailFromList->setWhatsThis(i18nc("@info:whatsthis", "Your email identity, used to identify you as the sender when sending email alarms."));
+		connect(mEmailFromList, SIGNAL(identityChanged(uint)), SLOT(contentsChanged()));
 		grid->addWidget(mEmailFromList, 0, 1, 1, 2);
 	}
 
@@ -1072,6 +1116,7 @@ void EditEmailAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	mEmailToEdit->setMinimumSize(mEmailToEdit->sizeHint());
 	mEmailToEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the addresses of the email recipients. Separate multiple addresses by "
 	                                "commas or semicolons."));
+	connect(mEmailToEdit, SIGNAL(textChanged(const QString&)), SLOT(contentsChanged()));
 	grid->addWidget(mEmailToEdit, 1, 1);
 
 	mEmailAddressButton = new QPushButton(parent);
@@ -1092,11 +1137,13 @@ void EditEmailAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	mEmailSubjectEdit->setMinimumSize(mEmailSubjectEdit->sizeHint());
 	label->setBuddy(mEmailSubjectEdit);
 	mEmailSubjectEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the email subject."));
+	connect(mEmailSubjectEdit, SIGNAL(textChanged(const QString&)), SLOT(contentsChanged()));
 	grid->addWidget(mEmailSubjectEdit, 2, 1, 1, 2);
 
 	// Email body
 	mEmailMessageEdit = new TextEdit(parent);
 	mEmailMessageEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the email message."));
+	connect(mEmailMessageEdit, SIGNAL(textChanged()), SLOT(contentsChanged()));
 	frameLayout->addWidget(mEmailMessageEdit);
 
 	// Email attachments
@@ -1134,6 +1181,7 @@ void EditEmailAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
 	mEmailBcc = new CheckBox(i18n_chk_CopyEmailToSelf(), parent);
 	mEmailBcc->setFixedSize(mEmailBcc->sizeHint());
 	mEmailBcc->setWhatsThis(i18nc("@info:whatsthis", "If checked, the email will be blind copied to you."));
+	connect(mEmailBcc, SIGNAL(toggled(bool)), SLOT(contentsChanged()));
 	grid->addWidget(mEmailBcc, 1, 0, 1, 2, Qt::AlignLeft);
 }
 
@@ -1157,7 +1205,15 @@ void EditEmailAlarmDlg::type_initValues(const KAEvent* event)
 		// Set the values to their defaults
 		mEmailBcc->setChecked(Preferences::defaultEmailBcc());
 	}
-	bool enable = !!mEmailAttachList->count();
+	attachmentEnable();
+}
+
+/******************************************************************************
+* Enable/disable controls depending on whether any attachments are entered.
+*/
+void EditEmailAlarmDlg::attachmentEnable()
+{
+	bool enable = mEmailAttachList->count();
 	mEmailAttachList->setEnabled(enable);
 	if (mEmailRemoveButton)
 		mEmailRemoveButton->setEnabled(enable);
@@ -1179,6 +1235,29 @@ void EditEmailAlarmDlg::setAction(KAEvent::Action action, const AlarmText& alarm
 		mEmailMessageEdit->setPlainText(alarmText.displayText());
 }
 
+/******************************************************************************
+* Initialise various values in the New Alarm dialogue.
+*/
+void EditEmailAlarmDlg::setEmailFields(uint fromID, const EmailAddressList& addresses,
+                                       const QString& subject, const QStringList& attachments)
+{
+	if (fromID)
+		mEmailFromList->setCurrentIdentity(fromID);
+	if (!addresses.isEmpty())
+		mEmailToEdit->setText(addresses.join(", "));
+	if (!subject.isEmpty())
+		mEmailSubjectEdit->setText(subject);
+	if (!attachments.isEmpty())
+	{
+		mEmailAttachList->addItems(attachments);
+		attachmentEnable();
+	}
+}
+void EditEmailAlarmDlg::setBcc(bool bcc)
+{
+	mEmailBcc->setChecked(bcc);
+}
+
 /******************************************************************************
 * Set the read-only status of all non-template controls.
 */
@@ -1317,11 +1396,14 @@ bool EditEmailAlarmDlg::type_validate(bool trial)
 void EditEmailAlarmDlg::type_trySuccessMessage(ShellProcess*, const QString&)
 {
 	QString msg;
+	QString to = mEmailAddresses.join("<nl/>");
+	to.replace('<', "&lt;");
+	to.replace('>', "&gt;");
 	if (mEmailBcc->isChecked())
-		msg = i18nc("@info", "Email sent to:<nl/>%1<nl/>Bcc: <email>%2</email>",
-		            mEmailAddresses.join("<nl/>"), Preferences::emailBccAddress());
+		msg = "<qt>" + i18nc("@info", "Email sent to:<nl/>%1<nl/>Bcc: <email>%2</email>",
+		            to, Preferences::emailBccAddress()) + "</qt>";
 	else
-		msg = i18nc("@info", "Email sent to:<nl/>%1", mEmailAddresses.join("<nl/>"));
+		msg = "<qt>" + i18nc("@info", "Email sent to:<nl/>%1", to) + "</qt>";
 	KMessageBox::information(this, msg);
 }
 
@@ -1354,6 +1436,7 @@ void EditEmailAlarmDlg::slotAddAttachment()
 		mEmailAttachList->setCurrentIndex(mEmailAttachList->count() - 1);   // select the new item
 		mEmailRemoveButton->setEnabled(true);
 		mEmailAttachList->setEnabled(true);
+		contentsChanged();
 	}
 }
 
@@ -1372,6 +1455,7 @@ void EditEmailAlarmDlg::slotRemoveAttachment()
 		mEmailRemoveButton->setEnabled(false);
 		mEmailAttachList->setEnabled(false);
 	}
+	contentsChanged();
 }
 
 /******************************************************************************
@@ -1397,16 +1481,19 @@ CommandEdit::CommandEdit(QWidget* parent)
 	vlayout->setSpacing(KDialog::spacingHint());
 	mTypeScript = new CheckBox(EditCommandAlarmDlg::i18n_chk_EnterScript(), this);
 	mTypeScript->setFixedSize(mTypeScript->sizeHint());
-	connect(mTypeScript, SIGNAL(toggled(bool)), SLOT(slotCmdScriptToggled(bool)));
 	mTypeScript->setWhatsThis(i18nc("@info:whatsthis", "Check to enter the contents of a script instead of a shell command line"));
+	connect(mTypeScript, SIGNAL(toggled(bool)), SLOT(slotCmdScriptToggled(bool)));
+	connect(mTypeScript, SIGNAL(toggled(bool)), SIGNAL(changed()));
 	vlayout->addWidget(mTypeScript, 0, Qt::AlignLeft);
 
 	mCommandEdit = new LineEdit(LineEdit::Url, this);
 	mCommandEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter a shell command to execute."));
+	connect(mCommandEdit, SIGNAL(textChanged(const QString&)), SIGNAL(changed()));
 	vlayout->addWidget(mCommandEdit);
 
 	mScriptEdit = new TextEdit(this);
 	mScriptEdit->setWhatsThis(i18nc("@info:whatsthis", "Enter the contents of a script to execute"));
+	connect(mScriptEdit, SIGNAL(textChanged()), SIGNAL(changed()));
 	vlayout->addWidget(mScriptEdit);
 
 	slotCmdScriptToggled(mTypeScript->isChecked());
