@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  *  In addition, as a special exception, the copyright holders give
  *  permission to link the code of this program with any edition of
@@ -30,10 +30,6 @@
  *  your version.
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "cachedimapjob.h"
 #include "imapaccountbase.h"
 
@@ -44,47 +40,50 @@
 #include "kmacctcachedimap.h"
 #include "kmmsgdict.h"
 #include "maildirjob.h"
+#include "util.h"
+#include "scalix.h"
 
 #include <kio/scheduler.h>
 #include <kio/job.h>
-
-#include <kmessagebox.h>
 #include <klocale.h>
 #include <kdebug.h>
 
+#include <QList>
+#include <QByteArray>
+
 
 namespace KMail {
 
 // Get messages
-CachedImapJob::CachedImapJob( const QValueList<MsgForDownload>& msgs,
+CachedImapJob::CachedImapJob( const QList<MsgForDownload>& msgs,
                               JobType type, KMFolderCachedImap* folder )
   : FolderJob( type ), mFolder( folder ), mMsgsForDownload( msgs ),
     mTotalBytes(0), mMsg(0), mParentFolder( 0 )
 {
-  QValueList<MsgForDownload>::ConstIterator it = msgs.begin();
+  QList<MsgForDownload>::ConstIterator it = msgs.begin();
   for ( ; it != msgs.end() ; ++it )
     mTotalBytes += (*it).size;
 }
 
 // Put messages
-CachedImapJob::CachedImapJob( const QPtrList<KMMessage>& msgs, JobType type,
+CachedImapJob::CachedImapJob( const QList<KMMessage*>& msgs, JobType type,
                               KMFolderCachedImap* folder )
-  : FolderJob( msgs, QString::null, type, folder?folder->folder():0 ), mFolder( folder ),
+  : FolderJob( msgs, QString(), type, folder?folder->folder():0 ), mFolder( folder ),
     mTotalBytes( msgs.count() ), // we abuse it as "total number of messages"
     mMsg( 0 ), mParentFolder( 0 )
 {
 }
 
-CachedImapJob::CachedImapJob( const QValueList<unsigned long>& msgs,
-			      JobType type, KMFolderCachedImap* folder )
-  : FolderJob( QPtrList<KMMessage>(), QString::null, type, folder?folder->folder():0 ),
+CachedImapJob::CachedImapJob( const QList<unsigned long>& msgs,
+                              JobType type, KMFolderCachedImap* folder )
+  : FolderJob( QList<KMMessage*>(), QString(), type, folder?folder->folder():0 ),
     mFolder( folder ), mSerNumMsgList( msgs ), mTotalBytes( msgs.count() ), mMsg( 0 ),
     mParentFolder ( 0 )
 {
 }
 
 // Add sub folders
-CachedImapJob::CachedImapJob( const QValueList<KMFolderCachedImap*>& fList,
+CachedImapJob::CachedImapJob( const QList<KMFolderCachedImap*>& fList,
                               JobType type, KMFolderCachedImap* folder )
   : FolderJob( type ), mFolder( folder ), mFolderList( fList ), mMsg( 0 ),
     mParentFolder ( 0 )
@@ -119,7 +118,7 @@ CachedImapJob::CachedImapJob( JobType type, KMFolderCachedImap* folder )
 
 CachedImapJob::~CachedImapJob()
 {
-  mAccount->mJobList.remove(this);
+  mAccount->mJobList.removeAll(this);
 }
 
 void CachedImapJob::execute()
@@ -136,7 +135,7 @@ void CachedImapJob::execute()
   assert( mAccount != 0 );
   if( mAccount->makeConnection() != ImapAccountBase::Connected ) {
     // No connection to the IMAP server
-    kdDebug(5006) << "mAccount->makeConnection() failed" << endl;
+    kDebug() <<"mAccount->makeConnection() failed";
     mPassiveDestructor = true;
     delete this;
     return;
@@ -146,6 +145,34 @@ void CachedImapJob::execute()
   // All necessary conditions have been met. Register this job
   mAccount->mJobList.append(this);
 
+  /**
+   * The Scalix server requires to send him a custom X-SCALIX-ID command
+   * to switch it into a special mode.
+   *
+   * This should be done once after the login and before the first command.
+   */
+  if ( mAccount->groupwareType() == KMAcctCachedImap::GroupwareScalix ) {
+    if ( !mAccount->sentCustomLoginCommand() ) {
+      QByteArray packedArgs;
+      QDataStream stream( &packedArgs, QIODevice::WriteOnly );
+
+      const QString command = QString( "X-SCALIX-ID " );
+      const QString argument = QString( "(\"name\" \"Evolution\" \"version\" \"2.10.0\")" );
+
+      stream << (int) 'X' << 'N' << command << argument;
+
+      const KUrl url = mAccount->getUrl();
+
+      ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+      jd.items << mFolder->label(); // for the err msg
+      KIO::SimpleJob *simpleJob = KIO::special( url.url(), packedArgs, false );
+      KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
+      mAccount->insertJob(simpleJob, jd);
+
+      mAccount->setSentCustomLoginCommand( true );
+    }
+  }
+
   switch( mType ) {
   case tGetMessage:       slotGetNextMessage();     break;
   case tPutMessage:       slotPutNextMessage();     break;
@@ -163,32 +190,32 @@ void CachedImapJob::execute()
 
 void CachedImapJob::listMessages()
 {
-  KURL url = mAccount->getUrl();
+  KUrl url = mAccount->getUrl();
   url.setPath( mFolder->imapPath() + ";UID=1:*;SECTION=FLAGS RFC822.SIZE");
 
-  KIO::SimpleJob *job = KIO::get(url, false, false);
+  KIO::SimpleJob *job = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   jd.cancellable = true;
   mAccount->insertJob( job, jd );
-  connect( job, SIGNAL( result(KIO::Job *) ),
-           this, SLOT( slotListMessagesResult( KIO::Job* ) ) );
+  connect( job, SIGNAL( result(KJob *) ),
+           this, SLOT( slotListMessagesResult( KJob* ) ) );
   // send the data directly for KMFolderCachedImap
   connect( job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
-           mFolder, SLOT( slotGetMessagesData( KIO::Job* , const QByteArray& ) ) );
+           mFolder, SLOT( slotGetMessagesData( KIO::Job*, const QByteArray& ) ) );
 }
 
-void CachedImapJob::slotDeleteNextMessages( KIO::Job* job )
+void CachedImapJob::slotDeleteNextMessages( KJob* job )
 {
   if (job) {
-    KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+    KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
     if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
       delete this;
       return;
     }
 
     if( job->error() ) {
-      mAccount->handleJobError( job, i18n( "Error while deleting messages on the server: " ) + '\n' );
+      mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while deleting messages on the server: " ) + '\n' );
       delete this;
       return;
     }
@@ -203,35 +230,35 @@ void CachedImapJob::slotDeleteNextMessages( KIO::Job* job )
 
   QString uids = mFoldersOrMessages.front(); mFoldersOrMessages.pop_front();
 
-  KURL url = mAccount->getUrl();
+  KUrl url = mAccount->getUrl();
   url.setPath( mFolder->imapPath() +
                QString::fromLatin1(";UID=%1").arg(uids) );
 
-  KIO::SimpleJob *simpleJob = KIO::file_delete( url, false );
+  KIO::SimpleJob *simpleJob = KIO::file_delete( url, KIO::HideProgressInfo );
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), simpleJob );
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   mAccount->insertJob( simpleJob, jd );
-  connect( simpleJob, SIGNAL( result(KIO::Job *) ),
-           this, SLOT( slotDeleteNextMessages(KIO::Job *) ) );
+  connect( simpleJob, SIGNAL( result(KJob *) ),
+           this, SLOT( slotDeleteNextMessages(KJob *) ) );
 }
 
 void CachedImapJob::expungeFolder()
 {
-  KURL url = mAccount->getUrl();
+  KUrl url = mAccount->getUrl();
   // Special URL that means EXPUNGE
   url.setPath( mFolder->imapPath() + QString::fromLatin1(";UID=*") );
 
-  KIO::SimpleJob *job = KIO::file_delete( url, false );
+  KIO::SimpleJob *job = KIO::file_delete( url, KIO::HideProgressInfo );
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   mAccount->insertJob( job, jd );
-  connect( job, SIGNAL( result(KIO::Job *) ),
-           this, SLOT( slotExpungeResult(KIO::Job *) ) );
+  connect( job, SIGNAL( result(KJob *) ),
+           this, SLOT( slotExpungeResult(KJob *) ) );
 }
 
-void CachedImapJob::slotExpungeResult( KIO::Job * job )
+void CachedImapJob::slotExpungeResult( KJob * job )
 {
-  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
@@ -239,7 +266,7 @@ void CachedImapJob::slotExpungeResult( KIO::Job * job )
 
   if (job->error()) {
     mErrorCode = job->error();
-    mAccount->handleJobError( job, i18n( "Error while deleting messages on the server: " ) + '\n' );
+    mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while deleting messages on the server: " ) + '\n' );
   }
   else
     mAccount->removeJob(it);
@@ -247,10 +274,10 @@ void CachedImapJob::slotExpungeResult( KIO::Job * job )
   delete this;
 }
 
-void CachedImapJob::slotGetNextMessage(KIO::Job * job)
+void CachedImapJob::slotGetNextMessage(KJob * job)
 {
   if (job) {
-    KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+    KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
     if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
       delete this;
       return;
@@ -258,7 +285,7 @@ void CachedImapJob::slotGetNextMessage(KIO::Job * job)
 
     if (job->error()) {
       mErrorCode = job->error();
-      mAccount->handleJobError( job, i18n( "Error while retrieving message on the server: " ) + '\n' );
+      mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while retrieving message on the server: " ) + '\n' );
       delete this;
       return;
     }
@@ -270,20 +297,22 @@ void CachedImapJob::slotGetNextMessage(KIO::Job * job)
 
       // Convert CR/LF to LF.
       size_t dataSize = (*it).data.size();
-      dataSize = FolderStorage::crlf2lf( (*it).data.data(), dataSize ); // always <=
+      dataSize = Util::crlf2lf( (*it).data.data(), dataSize ); // always <=
       (*it).data.resize( dataSize );
 
       mMsg->setComplete( true );
-      mMsg->fromByteArray( (*it).data );
+      mMsg->fromString( (*it).data );
       mMsg->setUID(uid);
       mMsg->setMsgSizeServer(size);
       mMsg->setTransferInProgress( false );
       int index = 0;
+      mFolder->open( "KMFolderCachedImap::slotGetNextMessage" );
       mFolder->addMsgInternal( mMsg, true, &index );
 
       if ( kmkernel->iCalIface().isResourceFolder( mFolder->folder() ) ) {
-        mFolder->setStatus( index, KMMsgStatusRead, false );
+        mFolder->setStatus( index, MessageStatus::statusRead(), false );
       }
+      mFolder->close( "KMFolderCachedImap::slotGetNextMessage" );
 
       emit messageRetrieved( mMsg );
       if ( index > 0 ) mFolder->unGetMsg( index );
@@ -295,9 +324,11 @@ void CachedImapJob::slotGetNextMessage(KIO::Job * job)
     mSentBytes += size;
     emit progress( mSentBytes, mTotalBytes );
     mAccount->removeJob(it);
-  }
+  } else
+    mFolder->quiet( true );
 
   if( mMsgsForDownload.isEmpty() ) {
+    mFolder->quiet( false );
     delete this;
     return;
   }
@@ -308,25 +339,25 @@ void CachedImapJob::slotGetNextMessage(KIO::Job * job)
   mMsg->setUID(mfd.uid);
   mMsg->setMsgSizeServer(mfd.size);
   if( mfd.flags > 0 )
-    KMFolderImap::flagsToStatus(mMsg, mfd.flags);
-  KURL url = mAccount->getUrl();
+    KMFolderImap::flagsToStatus(mMsg, mfd.flags, true, GlobalSettings::allowLocalFlags() ? mFolder->permanentFlags() : INT_MAX);
+  KUrl url = mAccount->getUrl();
   url.setPath(mFolder->imapPath() + QString(";UID=%1;SECTION=BODY.PEEK[]").arg(mfd.uid));
 
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   jd.cancellable = true;
   mMsg->setTransferInProgress(true);
-  KIO::SimpleJob *simpleJob = KIO::get(url, false, false);
+  KIO::SimpleJob *simpleJob = KIO::get(url, KIO::NoReload, KIO::HideProgressInfo);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
   mAccount->insertJob(simpleJob, jd);
-  connect(simpleJob, SIGNAL(processedSize(KIO::Job *, KIO::filesize_t)),
-          this, SLOT(slotProcessedSize(KIO::Job *, KIO::filesize_t)));
-  connect(simpleJob, SIGNAL(result(KIO::Job *)),
-          this, SLOT(slotGetNextMessage(KIO::Job *)));
+  connect(simpleJob, SIGNAL(processedSize(KJob *, qulonglong)),
+          this, SLOT(slotProcessedSize(KJob *, qulonglong)));
+  connect(simpleJob, SIGNAL(result(KJob *)),
+          this, SLOT(slotGetNextMessage(KJob *)));
   connect(simpleJob, SIGNAL(data(KIO::Job *, const QByteArray &)),
           mFolder, SLOT(slotSimpleData(KIO::Job *, const QByteArray &)));
 }
 
-void CachedImapJob::slotProcessedSize(KIO::Job *, KIO::filesize_t processed)
+void CachedImapJob::slotProcessedSize(KJob *, qulonglong processed)
 {
   emit progress( mSentBytes + processed, mTotalBytes );
 }
@@ -349,7 +380,7 @@ void CachedImapJob::slotPutNextMessage()
     // Find the message with this serial number
     int i = 0;
     KMFolder* aFolder = 0;
-    kmkernel->msgDict()->getLocation( serNum, &aFolder, &i );
+    KMMsgDict::instance()->getLocation( serNum, &aFolder, &i );
     if( mFolder->folder() != aFolder )
       // This message was moved or something
       continue;
@@ -362,41 +393,42 @@ void CachedImapJob::slotPutNextMessage()
     return;
   }
 
-  KURL url = mAccount->getUrl();
-  QString flags = KMFolderImap::statusToFlags( mMsg->status() );
+  KUrl url = mAccount->getUrl();
+  QString flags = KMFolderImap::statusToFlags( mMsg->status(), mFolder->permanentFlags() );
   url.setPath( mFolder->imapPath() + ";SECTION=" + flags );
 
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
 
   mMsg->setUID( 0 ); // for the index
-  QCString cstr(mMsg->asString());
-  int a = cstr.find("\nX-UID: ");
-  int b = cstr.find('\n', a);
-  if (a != -1 && b != -1 && cstr.find("\n\n") > a) cstr.remove(a, b-a);
-  QCString mData(cstr.length() + cstr.contains('\n'));
+  QByteArray cstr(mMsg->asString());
+  int a = cstr.indexOf("\nX-UID: ");
+  int b = cstr.indexOf('\n', a);
+  if (a != -1 && b != -1 && cstr.indexOf("\n\n") > a) cstr.remove(a, b-a);
+  QByteArray mData;
+  mData.resize( cstr.length() + cstr.count('\n') );
   unsigned int i = 0;
   for( char *ch = cstr.data(); *ch; ch++ ) {
     if ( *ch == '\n' ) {
-      mData.at(i) = '\r';
+      mData[i] = '\r';
       i++;
     }
-    mData.at(i) = *ch; i++;
+    mData[i] = *ch; i++;
   }
   jd.data = mData;
   jd.msgList.append( mMsg );
 
   mMsg->setTransferInProgress(true);
-  KIO::SimpleJob *simpleJob = KIO::put(url, 0, false, false, false);
+  KIO::SimpleJob *simpleJob = KIO::put(url, 0, KIO::HideProgressInfo);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
   mAccount->insertJob(simpleJob, jd);
-  connect( simpleJob, SIGNAL( result(KIO::Job *) ),
-           SLOT( slotPutMessageResult(KIO::Job *) ) );
+  connect( simpleJob, SIGNAL( result(KJob *) ),
+           SLOT( slotPutMessageResult(KJob *) ) );
   connect( simpleJob, SIGNAL( dataReq(KIO::Job *, QByteArray &) ),
            SLOT( slotPutMessageDataReq(KIO::Job *, QByteArray &) ) );
   connect( simpleJob, SIGNAL( data(KIO::Job *, const QByteArray &) ),
            mFolder, SLOT( slotSimpleData(KIO::Job *, const QByteArray &) ) );
-  connect( simpleJob, SIGNAL(infoMessage(KIO::Job *, const QString &)),
-             SLOT(slotPutMessageInfoData(KIO::Job *, const QString &)) );
+  connect( simpleJob, SIGNAL(infoMessage(KJob *, const QString &, const QString &)),
+             SLOT(slotPutMessageInfoData(KJob *, const QString &, const QString &)) );
 
 }
 
@@ -404,16 +436,16 @@ void CachedImapJob::slotPutNextMessage()
 // TODO: port to KIO::StoredTransferJob once it's ok to require kdelibs-3.3
 void CachedImapJob::slotPutMessageDataReq(KIO::Job *job, QByteArray &data)
 {
-  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
   }
   if ((*it).data.size() - (*it).offset > 0x8000) {
-    data.duplicate((*it).data.data() + (*it).offset, 0x8000);
+    data = QByteArray((*it).data.data() + (*it).offset, 0x8000);
     (*it).offset += 0x8000;
   } else if ((*it).data.size() - (*it).offset > 0) {
-    data.duplicate((*it).data.data() + (*it).offset,
+    data = QByteArray((*it).data.data() + (*it).offset,
                    (*it).data.size() - (*it).offset);
     (*it).offset = (*it).data.size();
   } else
@@ -421,33 +453,34 @@ void CachedImapJob::slotPutMessageDataReq(KIO::Job *job, QByteArray &data)
 }
 
 //----------------------------------------------------------------------------
-void CachedImapJob::slotPutMessageInfoData(KIO::Job *job, const QString &data)
+void CachedImapJob::slotPutMessageInfoData(KJob *job, const QString &data, const QString &)
 {
   KMFolderCachedImap * imapFolder = static_cast<KMFolderCachedImap*>(mDestFolder->storage());
   KMAcctCachedImap *account = imapFolder->account();
-  ImapAccountBase::JobIterator it = account->findJob( job );
+  ImapAccountBase::JobIterator it = account->findJob( static_cast<KIO::Job*>(job) );
   if ( it == account->jobsEnd() ) return;
 
-  if ( data.find("UID") != -1 && mMsg )
+  if ( data.contains("UID") && mMsg )
   {
-    int uid = (data.right(data.length()-4)).toInt();
-    kdDebug( 5006 ) << k_funcinfo << "Server told us uid is: " << uid << endl;
+    ulong uid = (data.right(data.length()-4)).toLong();
+    kDebug() << "Server told us uid is:" << uid;
     mMsg->setUID( uid );
+    Q_ASSERT( mMsg->UID() == uid );
   }
 }
 
 
 //-----------------------------------------------------------------------------
-void CachedImapJob::slotPutMessageResult(KIO::Job *job)
+void CachedImapJob::slotPutMessageResult(KJob *job)
 {
-  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
   }
 
   if ( job->error() ) {
-    bool cont = mAccount->handlePutError( job, *it, mFolder->folder() );
+    bool cont = mAccount->handlePutError( static_cast<KIO::Job*>(job), *it, mFolder->folder() );
     if ( !cont ) {
       delete this;
     } else {
@@ -466,7 +499,7 @@ void CachedImapJob::slotPutMessageResult(KIO::Job *job)
   int i;
   if( ( i = mFolder->find(mMsg) ) != -1 ) {
      /*
-      * If we have aquired a uid during upload the server supports the uidnext
+      * If we have acquired a uid during upload the server supports the uidnext
       * extension and there is no need to redownload this mail, we already have
       * it. Otherwise remove it, it will be redownloaded.
       */
@@ -477,7 +510,7 @@ void CachedImapJob::slotPutMessageResult(KIO::Job *job)
         bool b = kmkernel->iCalIface().isResourceQuiet();
         kmkernel->iCalIface().setResourceQuiet( true );
 
-        mFolder->take( i );
+        mFolder->takeTemporarily( i );
         mFolder->addMsgKeepUID( mMsg );
         mMsg->setTransferInProgress( false );
 
@@ -490,10 +523,10 @@ void CachedImapJob::slotPutMessageResult(KIO::Job *job)
 }
 
 
-void CachedImapJob::slotAddNextSubfolder( KIO::Job * job )
+void CachedImapJob::slotAddNextSubfolder( KJob * job )
 {
   if (job) {
-    KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+    KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
     if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
       delete this;
       return;
@@ -505,14 +538,26 @@ void CachedImapJob::slotAddNextSubfolder( KIO::Job * job )
 
     if ( job->error() && !silentUpload ) {
       QString myError = "<p><b>" + i18n("Error while uploading folder")
-        + "</b></p><p>" + i18n("Could not make the folder <b>%1</b> on the server.").arg((*it).items[0])
+        + "</b></p><p>" + i18n("Could not make the folder <b>%1</b> on the server.", (*it).items[0])
         + "</p><p>" + i18n("This could be because you do not have permission to do this, or because the folder is already present on the server; the error message from the server communication is here:") + "</p>";
-      mAccount->handleJobError( job, myError );
+      mAccount->handleJobError( static_cast<KIO::Job*>(job), myError );
     }
 
     if( job->error() ) {
       delete this;
       return;
+    } else {
+      KMFolderCachedImap* storage = static_cast<KMFolderCachedImap*>( (*it).current->storage() );
+      KMFolderCachedImap* parentStorage = static_cast<KMFolderCachedImap*>( (*it).parent->storage() );
+      Q_ASSERT( storage );
+      Q_ASSERT( parentStorage );
+      if ( storage->imapPath().isEmpty() ) {
+        QString path = mAccount->createImapPath( parentStorage->imapPath(), storage->folder()->name() );
+        if ( !storage->imapPathForCreation().isEmpty() )
+          path = storage->imapPathForCreation();
+        storage->setImapPath( path );
+        storage->writeConfig();
+      }
     }
     mAccount->removeJob( it );
   }
@@ -525,26 +570,53 @@ void CachedImapJob::slotAddNextSubfolder( KIO::Job * job )
 
   KMFolderCachedImap *folder = mFolderList.front();
   mFolderList.pop_front();
-  KURL url = mAccount->getUrl();
-  url.setPath(mFolder->imapPath() + folder->folder()->name());
+  KUrl url = mAccount->getUrl();
+  QString path = mAccount->createImapPath( mFolder->imapPath(),
+      folder->folder()->name() );
+  if ( !folder->imapPathForCreation().isEmpty() ) {
+    // the folder knows it's namespace
+    path = folder->imapPathForCreation();
+  }
+  url.setPath( path );
 
-  // Associate the jobData with the parent folder, not with the child
-  // This is necessary in case of an error while creating the subfolder,
-  // so that folderComplete is called on the parent (and the sync resetted).
-  ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
-  jd.items << folder->label(); // for the err msg
-  KIO::SimpleJob *simpleJob = KIO::mkdir(url);
-  KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
-  mAccount->insertJob(simpleJob, jd);
-  connect( simpleJob, SIGNAL(result(KIO::Job *)),
-           this, SLOT(slotAddNextSubfolder(KIO::Job *)) );
+  if ( mAccount->groupwareType() != KMAcctCachedImap::GroupwareScalix ) {
+    // Associate the jobData with the parent folder, not with the child
+    // This is necessary in case of an error while creating the subfolder,
+    // so that folderComplete is called on the parent (and the sync is reset).
+    ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+    jd.items << folder->label(); // for the err msg
+    jd.current = folder->folder();
+    KIO::SimpleJob *simpleJob = KIO::mkdir(url);
+    KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
+    mAccount->insertJob(simpleJob, jd);
+    connect( simpleJob, SIGNAL(result(KJob *)),
+             this, SLOT(slotAddNextSubfolder(KJob *)) );
+  } else {
+    QByteArray packedArgs;
+    QDataStream stream( &packedArgs, QIODevice::WriteOnly );
+
+    const QString command = QString( "X-CREATE-SPECIAL" );
+    const QString argument = QString( "%1 %2" ).arg( Scalix::Utils::contentsTypeToScalixId( folder->contentsType() ) )
+                                               .arg( path );
+
+    stream << (int) 'X' << 'N' << command << argument;
+
+    ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
+    jd.items << folder->label(); // for the err msg
+    jd.current = folder->folder();
+    KIO::SimpleJob *simpleJob = KIO::special( url.url(), packedArgs, false );
+    KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
+    mAccount->insertJob(simpleJob, jd);
+    connect( simpleJob, SIGNAL(result(KJob *)),
+             this, SLOT(slotAddNextSubfolder(KJob *)) );
+  }
 }
 
 
-void CachedImapJob::slotDeleteNextFolder( KIO::Job *job )
+void CachedImapJob::slotDeleteNextFolder( KJob *job )
 {
   if (job) {
-    KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+    KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
     if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
       delete this;
       return;
@@ -553,7 +625,7 @@ void CachedImapJob::slotDeleteNextFolder( KIO::Job *job )
     mAccount->removeDeletedFolder( (*it).path );
 
     if( job->error() ) {
-      mAccount->handleJobError( job, i18n( "Error while deleting folder %1 on the server: " ).arg( (*it).path ) + '\n' );
+      mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while deleting folder %1 on the server: ", (*it).path ) + '\n' );
       delete this;
       return;
     }
@@ -566,38 +638,39 @@ void CachedImapJob::slotDeleteNextFolder( KIO::Job *job )
     return;
   }
 
-  QString folderPath = mFoldersOrMessages.front(); mFoldersOrMessages.pop_front();
-  KURL url = mAccount->getUrl();
+  QString folderPath = mFoldersOrMessages.front();
+  mFoldersOrMessages.pop_front();
+  KUrl url = mAccount->getUrl();
   url.setPath(folderPath);
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   jd.path = url.path();
-  KIO::SimpleJob *simpleJob = KIO::file_delete(url, false);
+  KIO::SimpleJob *simpleJob = KIO::file_delete(url, KIO::HideProgressInfo);
   KIO::Scheduler::assignJobToSlave(mAccount->slave(), simpleJob);
   mAccount->insertJob(simpleJob, jd);
-  connect( simpleJob, SIGNAL( result(KIO::Job *) ),
-           SLOT( slotDeleteNextFolder(KIO::Job *) ) );
+  connect( simpleJob, SIGNAL( result(KJob *) ),
+           SLOT( slotDeleteNextFolder(KJob *) ) );
 }
 
 void CachedImapJob::checkUidValidity()
 {
-  KURL url = mAccount->getUrl();
+  KUrl url = mAccount->getUrl();
   url.setPath( mFolder->imapPath() + ";UID=0:0" );
 
   ImapAccountBase::jobData jd( url.url(), mFolder->folder() );
   jd.cancellable = true;
 
-  KIO::SimpleJob *job = KIO::get( url, false, false );
+  KIO::SimpleJob *job = KIO::get( url, KIO::NoReload, KIO::HideProgressInfo );
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), job );
   mAccount->insertJob( job, jd );
-  connect( job, SIGNAL(result(KIO::Job *)),
-           SLOT(slotCheckUidValidityResult(KIO::Job *)) );
+  connect( job, SIGNAL(result(KJob *)),
+           SLOT(slotCheckUidValidityResult(KJob *)) );
   connect( job, SIGNAL(data(KIO::Job *, const QByteArray &)),
            mFolder, SLOT(slotSimpleData(KIO::Job *, const QByteArray &)));
 }
 
-void CachedImapJob::slotCheckUidValidityResult(KIO::Job * job)
+void CachedImapJob::slotCheckUidValidityResult(KJob * job)
 {
-  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
@@ -605,36 +678,50 @@ void CachedImapJob::slotCheckUidValidityResult(KIO::Job * job)
 
   if( job->error() ) {
     mErrorCode = job->error();
-    mAccount->handleJobError( job, i18n( "Error while reading folder %1 on the server: " ).arg( (*it).parent->label() ) + '\n' );
+    mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while reading folder %1 on the server: ", (*it).parent->label() ) + '\n' );
     delete this;
     return;
   }
 
   // Check the uidValidity
-  QCString cstr((*it).data.data(), (*it).data.size() + 1);
-  int a = cstr.find("X-uidValidity: ");
+  QByteArray cstr((*it).data.data(), (*it).data.size());
+  int a = cstr.indexOf("X-uidValidity: ");
   if (a < 0) {
     // Something is seriously rotten here!
     // TODO: Tell the user that he has a problem
-    kdDebug(5006) << "No uidvalidity available for folder "
-                  << mFolder->name() << endl;
+    kDebug() <<"No uidvalidity available for folder"
+                  << mFolder->objectName();
   }
   else {
-    int b = cstr.find("\r\n", a);
+    int b = cstr.indexOf("\r\n", a);
     if ( (b - a - 15) >= 0 ) {
       QString uidv = cstr.mid(a + 15, b - a - 15);
-      // kdDebug(5006) << "New uidv = " << uidv << ", old uidv = "
-      //               << mFolder->uidValidity() << endl;
+      // kDebug() <<"New uidv =" << uidv <<", old uidv ="
+      //               << mFolder->uidValidity();
       if( !mFolder->uidValidity().isEmpty() && mFolder->uidValidity() != uidv ) {
-        // kdDebug(5006) << "Expunging the mailbox " << mFolder->name()
-        //               << "!" << endl;
+        // kDebug() <<"Expunging the mailbox" << mFolder->name()
+        //               << "!";
         mFolder->expunge();
+        mFolder->open( "cachedimap" ); // reopen after the forced close by expunge() for KMFolderCachedImap
         mFolder->setLastUid( 0 );
         mFolder->clearUidMap();
       }
     } else
-      kdDebug(5006) << "No uidvalidity available for folder "
-                    << mFolder->name() << endl;
+      kDebug() <<"No uidvalidity available for folder"
+                    << mFolder->objectName();
+  }
+
+  a = cstr.indexOf( "X-PermanentFlags: " );
+  if ( a < 0 ) {
+    kDebug() << "no PERMANENTFLAGS response? assumming custom flags are not available";
+  } else {
+    int b = cstr.indexOf( "\r\n", a );
+    if ( (b - a - 18) >= 0 ) {
+      int flags = cstr.mid( a + 18, b - a - 18 ).toInt();
+      emit permanentFlags( flags );
+    } else {
+      kDebug() << "PERMANENTFLAGS response broken, assumming custom flags are not available";
+    }
   }
 
   mAccount->removeJob(it);
@@ -645,11 +732,11 @@ void CachedImapJob::slotCheckUidValidityResult(KIO::Job * job)
 void CachedImapJob::renameFolder( const QString &newName )
 {
   // Set the source URL
-  KURL urlSrc = mAccount->getUrl();
+  KUrl urlSrc = mAccount->getUrl();
   urlSrc.setPath( mFolder->imapPath() );
 
   // Set the destination URL - this is a bit trickier
-  KURL urlDst = mAccount->getUrl();
+  KUrl urlDst = mAccount->getUrl();
   QString imapPath( mFolder->imapPath() );
   // Destination url = old imappath - oldname + new name
   imapPath.truncate( imapPath.length() - mFolder->folder()->name().length() - 1);
@@ -659,25 +746,26 @@ void CachedImapJob::renameFolder( const QString &newName )
   ImapAccountBase::jobData jd( newName, mFolder->folder() );
   jd.path = imapPath;
 
-  KIO::SimpleJob *simpleJob = KIO::rename( urlSrc, urlDst, false );
+  KIO::SimpleJob *simpleJob = KIO::rename( urlSrc, urlDst, KIO::HideProgressInfo );
   KIO::Scheduler::assignJobToSlave( mAccount->slave(), simpleJob );
   mAccount->insertJob( simpleJob, jd );
-  connect( simpleJob, SIGNAL(result(KIO::Job *)),
-           SLOT(slotRenameFolderResult(KIO::Job *)) );
+  connect( simpleJob, SIGNAL(result(KJob *)),
+           SLOT(slotRenameFolderResult(KJob *)) );
 }
 
 static void renameChildFolders( KMFolderDir* dir, const QString& oldPath,
                                 const QString& newPath )
 {
   if( dir ) {
-    KMFolderNode *node = dir->first();
-    while( node ) {
+    QList<KMFolderNode*>::const_iterator it;
+    for ( it = dir->constBegin(); it != dir->constEnd(); ++it ) {
+      KMFolderNode *node = *it;
       if( !node->isDir() ) {
         KMFolderCachedImap* imapFolder =
           static_cast<KMFolderCachedImap*>(static_cast<KMFolder*>(node)->storage());
         if ( !imapFolder->imapPath().isEmpty() )
           // Only rename folders that have been accepted by the server
-          if( imapFolder->imapPath().find( oldPath ) == 0 ) {
+          if( imapFolder->imapPath().startsWith( oldPath ) ) {
             QString p = imapFolder->imapPath();
             p = p.mid( oldPath.length() );
             p.prepend( newPath );
@@ -685,14 +773,13 @@ static void renameChildFolders( KMFolderDir* dir, const QString& oldPath,
             renameChildFolders( imapFolder->folder()->child(), oldPath, newPath );
           }
       }
-      node = dir->next();
     }
   }
 }
 
-void CachedImapJob::slotRenameFolderResult( KIO::Job *job )
+void CachedImapJob::slotRenameFolderResult( KJob *job )
 {
-  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
@@ -707,19 +794,19 @@ void CachedImapJob::slotRenameFolderResult( KIO::Job *job )
       mFolder->folder()->setLabel( (*renit).mOldLabel );
       mAccount->removeRenamedFolder( mFolder->imapPath() );
     }
-    mAccount->handleJobError( job, i18n( "Error while trying to rename folder %1" ).arg( mFolder->label() ) + '\n' );
+    mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while trying to rename folder %1", mFolder->label() ) + '\n' );
   } else {
     // Okay, the folder seems to be renamed on the server,
     // now rename it on disk
-    QString oldName = mFolder->name();
+    QString oldName = mFolder->objectName();
     QString oldPath = mFolder->imapPath();
     mAccount->removeRenamedFolder( oldPath );
     mFolder->setImapPath( (*it).path );
     mFolder->FolderStorage::rename( (*it).url );
 
-    if( oldPath.endsWith( "/" ) ) oldPath.truncate( oldPath.length() -1 );
+    if( oldPath.endsWith( '/' ) ) oldPath.truncate( oldPath.length() -1 );
     QString newPath = mFolder->imapPath();
-    if( newPath.endsWith( "/" ) ) newPath.truncate( newPath.length() -1 );
+    if( newPath.endsWith( '/' ) ) newPath.truncate( newPath.length() -1 );
     renameChildFolders( mFolder->folder()->child(), oldPath, newPath );
     kmkernel->dimapFolderMgr()->contentsChanged();
 
@@ -728,9 +815,9 @@ void CachedImapJob::slotRenameFolderResult( KIO::Job *job )
   delete this;
 }
 
-void CachedImapJob::slotListMessagesResult( KIO::Job * job )
+void CachedImapJob::slotListMessagesResult( KJob * job )
 {
-  KMAcctCachedImap::JobIterator it = mAccount->findJob(job);
+  KMAcctCachedImap::JobIterator it = mAccount->findJob(static_cast<KIO::Job*>(job));
   if ( it == mAccount->jobsEnd() ) { // Shouldn't happen
     delete this;
     return;
@@ -738,7 +825,7 @@ void CachedImapJob::slotListMessagesResult( KIO::Job * job )
 
   if (job->error()) {
     mErrorCode = job->error();
-    mAccount->handleJobError( job, i18n( "Error while deleting messages on the server: " ) + '\n' );
+    mAccount->handleJobError( static_cast<KIO::Job*>(job), i18n( "Error while deleting messages on the server: " ) + '\n' );
   }
   else
     mAccount->removeJob(it);
