@@ -5,13 +5,12 @@
 #ifndef _CONFIGURE_DIALOG_PRIVATE_H_
 #define _CONFIGURE_DIALOG_PRIVATE_H_
 
+#include <config-enterprise.h>
+
 #include "kmail_export.h"
 
-#include <klineedit.h>
-#include <QComboBox>
 #include <QPointer>
 #include <QString>
-
 #include <QStringList>
 #include <QLabel>
 #include <QList>
@@ -21,29 +20,36 @@
 #include <QTreeWidget>
 #include <QHash>
 
+#include <klineedit.h>
+#include <kcombobox.h>
 #include <kdialog.h>
 #include <kcmodule.h>
 #include <klocale.h>
+
 #include "ui_composercryptoconfiguration.h"
 #include "ui_warningconfiguration.h"
 #include "ui_smimeconfiguration.h"
 #include "ui_customtemplates_base.h"
+#include "ui_miscpagemaintab.h"
+#include "ui_miscpagegroupwaretab.h"
+#include "ui_securitypagegeneraltab.h"
+#include "ui_identitypage.h"
 
 class QPushButton;
 class QLabel;
 class QCheckBox;
 class QFont;
-class QTabWidget;
 class QRegExpValidator;
 class QPoint;
 class QGroupBox;
-class QLineEdit;
 class QSpinBox;
 class QListWidget;
 
+class KLineEdit;
 class KButtonGroup;
 class KUrlRequester;
 class KFontChooser;
+class KTabWidget;
 class KVBox;
 class KMAccount;
 class ListView;
@@ -59,6 +65,7 @@ class KColorCombo;
 class KFontRequester;
 class KIconButton;
 class KKeySequenceWidget;
+class KComboBox;
 
 namespace KMail {
   class IdentityDialog;
@@ -120,57 +127,11 @@ protected slots:
   virtual void slotEnableOK( const QString & );
 
 private:
-  QLineEdit  *mLineEdit;
-  QComboBox  *mComboBox;
+  KLineEdit  *mLineEdit;
+  KComboBox  *mComboBox;
   QButtonGroup *mButtonGroup;
 };
 
-
-//
-//
-// Language item handling
-//
-//
-
-struct LanguageItem
-{
-  LanguageItem() {}
-  LanguageItem( const QString & language, const QString & reply=QString(),
-                const QString & replyAll=QString(),
-                const QString & forward=QString(),
-                const QString & indentPrefix=QString() ) :
-    mLanguage( language ), mReply( reply ), mReplyAll( replyAll ),
-    mForward( forward ), mIndentPrefix( indentPrefix ) {}
-
-  QString mLanguage, mReply, mReplyAll, mForward, mIndentPrefix;
-};
-
-typedef QList<LanguageItem> LanguageItemList;
-
-class NewLanguageDialog : public KDialog
-{
-  Q_OBJECT
-
-  public:
-    explicit NewLanguageDialog( LanguageItemList & suppressedLangs, QWidget *parent=0 );
-    QString language() const;
-
-  private:
-    QComboBox *mComboBox;
-};
-
-
-class LanguageComboBox : public QComboBox
-{
-  Q_OBJECT
-
-  public:
-    LanguageComboBox( QWidget *parent=0 );
-    int insertLanguage( const QString & language );
-    QString language() const;
-    void setLanguage( const QString & language );
-};
-
 //
 //
 // Profile dialog
@@ -275,7 +236,7 @@ protected:
   void addTab( ConfigModuleTab* tab, const QString & title );
 
 private:
-  QTabWidget *mTabWidget;
+  KTabWidget *mTabWidget;
 
 };
 
@@ -318,11 +279,7 @@ private: // data members
   KMail::IdentityDialog   *mIdentityDialog;
   int                      mOldNumberOfIdentities;
 
-  KMail::IdentityListView *mIdentityList;
-  QPushButton             *mModifyButton;
-  QPushButton             *mRenameButton;
-  QPushButton             *mRemoveButton;
-  QPushButton             *mSetAsDefaultButton;
+  Ui_IdentityPage mIPage;
 };
 
 
@@ -348,10 +305,10 @@ private:
 
 private:
   QCheckBox   *mConfirmSendCheck;
-  QComboBox   *mSendOnCheckCombo;
-  QComboBox   *mSendMethodCombo;
-  QComboBox   *mMessagePropertyCombo;
-  QLineEdit   *mDefaultDomainEdit;
+  KComboBox   *mSendOnCheckCombo;
+  KComboBox   *mSendMethodCombo;
+  KComboBox   *mMessagePropertyCombo;
+  KLineEdit   *mDefaultDomainEdit;
 };
 
 
@@ -444,11 +401,11 @@ private:
 
 private:
   QCheckBox    *mCustomFontCheck;
-  QComboBox    *mFontLocationCombo;
+  KComboBox    *mFontLocationCombo;
   KFontChooser *mFontChooser;
 
   int          mActiveFontIndex;
-  QFont        mFont[14];
+  QFont        mFont[13];
 };
 
 class AppearancePageColorsTab : public ConfigModuleTab {
@@ -516,14 +473,14 @@ private: // methods
   void setDateDisplay( int id, const QString & format );
 
 private: // data
-  QCheckBox    *mShowQuickSearch;
-  QCheckBox    *mMessageSizeCheck;
-  QCheckBox    *mAttachmentCheck;
-  QCheckBox    *mNestedMessagesCheck;
-  QCheckBox    *mCryptoIconsCheck;
-  KButtonGroup *mNestingPolicy;
+  QCheckBox    *mDisplayMessageToolTips;
+  QCheckBox    *mHideTabBarWithSingleTab;
   KButtonGroup *mDateDisplay;
-  QLineEdit    *mCustomDateFormatEdit;
+  KLineEdit    *mCustomDateFormatEdit;
+  QString       mCustomDateWhatsThis;
+
+private slots:
+  void slotLinkClicked( const QString & link );
 };
 
 class AppearancePageReaderTab : public ConfigModuleTab {
@@ -550,8 +507,8 @@ private: // data
   QCheckBox *mShowExpandQuotesMark;
   KIntSpinBox  *mCollapseQuoteLevelSpin;
   QCheckBox *mShrinkQuotesCheck;
-  QComboBox *mCharsetCombo;
-  QComboBox *mOverrideCharsetCombo;
+  KComboBox *mCharsetCombo;
+  KComboBox *mOverrideCharsetCombo;
 };
 
 
@@ -629,7 +586,7 @@ private:
   void swapTagsInListBox( const int first, const int second );
 
 private: // data
-  QLineEdit *mTagNameLineEdit, *mTagAddLineEdit;
+  KLineEdit *mTagNameLineEdit, *mTagAddLineEdit;
   QPushButton *mTagAddButton, *mTagRemoveButton,
               *mTagUpButton, *mTagDownButton;
 
@@ -715,37 +672,11 @@ private:
   KIntSpinBox   *mAutoSave;
   QCheckBox     *mExternalEditorCheck;
   KUrlRequester *mEditorRequester;
-};
-
-class ComposerPagePhrasesTab : public ConfigModuleTab {
-  Q_OBJECT
-public:
-  ComposerPagePhrasesTab( QWidget * parent=0 );
-  QString helpAnchor() const;
-
-  void save();
-
-private slots:
-  void slotNewLanguage();
-  void slotRemoveLanguage();
-  void slotLanguageChanged( const QString& );
-  void slotAddNewLanguage( const QString& );
-
-private:
-  virtual void doLoadFromGlobalSettings();
-  void setLanguageItemInformation( int index );
-  void saveActiveLanguageItem();
-
-private:
-  LanguageComboBox *mPhraseLanguageCombo;
-  QPushButton      *mRemoveButton;
-  QLineEdit        *mPhraseReplyEdit;
-  QLineEdit        *mPhraseReplyAllEdit;
-  QLineEdit        *mPhraseForwardEdit;
-  QLineEdit        *mPhraseIndentPrefixEdit;
-
-  int              mActiveLanguageItem;
-  LanguageItemList mLanguageList;
+#ifdef KDEPIM_ENTERPRISE_BUILD
+  KComboBox     *mForwardTypeCombo;
+  QCheckBox     *mRecipientCheck;
+  KIntSpinBox   *mRecipientSpin;
+#endif
 };
 
 class ComposerPageTemplatesTab : public ConfigModuleTab {
@@ -843,12 +774,12 @@ private:
 
 private:
   QCheckBox   *mCreateOwnMessageIdCheck;
-  QLineEdit   *mMessageIdSuffixEdit;
+  KLineEdit   *mMessageIdSuffixEdit;
   QRegExpValidator *mMessageIdSuffixValidator;
   ListView    *mTagList;
   QPushButton *mRemoveHeaderButton;
-  QLineEdit   *mTagNameEdit;
-  QLineEdit   *mTagValueEdit;
+  KLineEdit   *mTagNameEdit;
+  KLineEdit   *mTagValueEdit;
   QLabel      *mTagNameLabel;
   QLabel      *mTagValueLabel;
 };
@@ -883,7 +814,6 @@ public:
 
   // hrmpf. moc doesn't like nested classes with slots/signals...:
   typedef ComposerPageGeneralTab GeneralTab;
-  typedef ComposerPagePhrasesTab PhrasesTab;
   typedef ComposerPageTemplatesTab TemplatesTab;
   typedef ComposerPageCustomTemplatesTab CustomTemplatesTab;
   typedef ComposerPageSubjectTab SubjectTab;
@@ -893,7 +823,6 @@ public:
 
 private:
   GeneralTab  *mGeneralTab;
-  PhrasesTab  *mPhrasesTab;
   TemplatesTab  *mTemplatesTab;
   CustomTemplatesTab  *mCustomTemplatesTab;
   SubjectTab  *mSubjectTab;
@@ -923,16 +852,9 @@ private:
   //FIXME virtual void doResetToDefaultsOther();
 
 private:
-  QCheckBox    *mExternalReferences;
-  QCheckBox    *mHtmlMailCheck;
-  QCheckBox    *mNoMDNsWhenEncryptedCheck;
   QButtonGroup *mMDNGroup;
   QButtonGroup *mOrigQuoteGroup;
-  QCheckBox    *mAutomaticallyImportAttachedKeysCheck;
-  QCheckBox    *mAlwaysDecrypt;
-  QString       mHtmlWhatsThis;
-  QString       mExternalWhatsThis;
-  QString       mReceiptWhatsThis;
+  Ui_SecurityPageGeneralTab mSGTab;
 
 private slots:
     void slotLinkClicked( const QString & link );
@@ -970,6 +892,7 @@ public:
 
 private slots:
   void slotReenableAllWarningsClicked();
+  void updateSpinBoxSuffix();
 
 private:
   //virtual void doLoadFromGlobalSettings();
@@ -1068,17 +991,7 @@ private:
   //FIXME virtual void doResetToDefaultsOther();
 
 private:
-  QCheckBox    *mEmptyFolderConfirmCheck;
-  QCheckBox    *mExcludeImportantFromExpiry;
-  QComboBox    *mLoopOnGotoUnread;
-  QComboBox    *mMailboxPrefCombo;
-  QComboBox    *mActionEnterFolder;
-  QCheckBox    *mEmptyTrashCheck;
-  QCheckBox    *mDelayedMarkAsRead;
-  KIntSpinBox  *mDelayedMarkTime;
-  QCheckBox    *mShowPopupAfterDnD;
-  KMail::FolderRequester *mOnStartupOpenFolder;
-  QComboBox    *mQuotaCmbBox;
+  Ui_MiscMainTab mMMTab;
 };
 
 class MiscPageGroupwareTab : public ConfigModuleTab  {
@@ -1096,29 +1009,7 @@ private:
   virtual void doLoadFromGlobalSettings();
 
 private:
-  QCheckBox* mEnableGwCB;
-  QCheckBox* mEnableImapResCB;
-
-  QWidget* mBox;
-  KVBox* gBox;
-
-  QComboBox* mStorageFormatCombo;
-  QComboBox* mLanguageCombo;
-
-  QLabel* mFolderComboLabel;
-  QStackedWidget* mFolderComboStack;
-  KMail::FolderRequester* mFolderCombo; // in the widgetstack
-  KMail::AccountComboBox* mAccountCombo; // in the widgetstack
-
-  QCheckBox* mHideGroupwareFolders;
-  QCheckBox* mOnlyShowGroupwareFolders;
-  QCheckBox* mSyncImmediately;
-  QCheckBox* mDeleteInvitations;
-
-  QCheckBox* mLegacyMangleFromTo;
-  QCheckBox* mLegacyBodyInvites;
-  QCheckBox* mExchangeCompatibleInvitations;
-  QCheckBox* mAutomaticSending;
+  Ui_MiscGroupTab mMGTab;
 };
 
 class KMAIL_EXPORT MiscPage : public ConfigModuleWithTabs {
