@@ -13,7 +13,7 @@
 
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     In addition, as a special exception, the copyright holders give
     permission to link the code of this program with any edition of
@@ -27,12 +27,14 @@
     your version.
 */
 
+
 #include "filterlog.h"
 
 #include <kdebug.h>
 
-#include <qdatetime.h>
-#include <qfile.h>
+#include <QDateTime>
+#include <QFile>
+#include <QByteArray>
 
 #include <sys/stat.h>
 
@@ -44,7 +46,7 @@ FilterLog * FilterLog::mSelf = NULL;
 
 
 FilterLog::FilterLog()
-{ 
+{
   mSelf = this;
   // start with logging disabled by default
   mLogging = false;
@@ -52,7 +54,7 @@ FilterLog::FilterLog()
   // when the log i sgoing to become very long
   mMaxLogSize = 512 * 1024;
   mCurrentLogSize = 0;
-  mAllowedTypes =  meta | patternDesc | ruleResult | 
+  mAllowedTypes =  meta | patternDesc | ruleResult |
                    patternResult | appliedAction;
 }
 
@@ -68,11 +70,11 @@ FilterLog * FilterLog::instance()
 }
 
 
-void FilterLog::add( QString logEntry, ContentType contentType )
+void FilterLog::add( const QString& logEntry, ContentType contentType )
 {
   if ( isLogging() && ( mAllowedTypes & contentType ) )
   {
-    QString timedLog = "[" + QTime::currentTime().toString() + "] ";
+    QString timedLog = '[' + QTime::currentTime().toString() + "] ";
     if ( contentType & ~meta )
       timedLog += logEntry;
     else
@@ -85,29 +87,29 @@ void FilterLog::add( QString logEntry, ContentType contentType )
 }
 
 
-void FilterLog::setMaxLogSize( long size ) 
+void FilterLog::setMaxLogSize( long size )
 {
   if ( size < -1)
     size = -1;
   // do not allow less than 1 KByte except unlimited (-1)
   if ( size >= 0 && size < 1024 )
-    size = 1024; 
-  mMaxLogSize = size; 
+    size = 1024;
+  mMaxLogSize = size;
   emit logStateChanged();
-  checkLogSize(); 
+  checkLogSize();
 }
 
-      
+
 void FilterLog::dump()
 {
 #ifndef NDEBUG
-  kdDebug(5006) << "----- starting filter log -----" << endl;
-  for ( QStringList::Iterator it = mLogEntries.begin();
-        it != mLogEntries.end(); ++it )
+  kDebug() <<"----- starting filter log -----";
+  for ( QStringList::ConstIterator it = mLogEntries.constBegin();
+        it != mLogEntries.constEnd(); ++it )
   {
-    kdDebug(5006) << *it << endl;
+    kDebug() << *it;
   }
-  kdDebug(5006) << "------ end of filter log ------" << endl;
+  kDebug() <<"------ end of filter log ------";
 #endif
 }
 
@@ -116,22 +118,22 @@ void FilterLog::checkLogSize()
 {
   if ( mCurrentLogSize > mMaxLogSize && mMaxLogSize > -1 )
   {
-    kdDebug(5006) << "Filter log: memory limit reached, starting to discard old items, size = "
-                  << QString::number( mCurrentLogSize ) << endl;
+    kDebug() <<"Filter log: memory limit reached, starting to discard old items, size ="
+                  << QString::number( mCurrentLogSize );
     // avoid some kind of hysteresis, shrink the log to 90% of its maximum
     while ( mCurrentLogSize > ( mMaxLogSize * 0.9 ) )
     {
-      QValueListIterator<QString> it = mLogEntries.begin();
+      QStringList::Iterator it = mLogEntries.begin();
       if ( it != mLogEntries.end())
       {
         mCurrentLogSize -= (*it).length();
-        mLogEntries.remove( it );
-        kdDebug(5006) << "Filter log: new size = " 
-                      << QString::number( mCurrentLogSize ) << endl;
+        mLogEntries.erase( it );
+        kDebug() <<"Filter log: new size ="
+                      << QString::number( mCurrentLogSize );
       }
       else
       {
-        kdDebug(5006) << "Filter log: size reduction disaster!" << endl;
+        kDebug() <<"Filter log: size reduction disaster!";
         clear();
       }
     }
@@ -140,23 +142,23 @@ void FilterLog::checkLogSize()
 }
 
 
-bool FilterLog::saveToFile( QString fileName )
+bool FilterLog::saveToFile( const QString &fileName )
 {
     QFile file( fileName );
-    if( file.open( IO_WriteOnly ) ) {
+    if( file.open( QIODevice::WriteOnly ) ) {
       fchmod( file.handle(), S_IRUSR | S_IWUSR );
       {
         QDataStream ds( &file );
-        for ( QStringList::Iterator it = mLogEntries.begin(); 
-              it != mLogEntries.end(); ++it ) 
+        for ( QStringList::Iterator it = mLogEntries.begin();
+              it != mLogEntries.end(); ++it )
         {
           QString tmpString = *it + '\n';
-          QCString cstr( tmpString.local8Bit() );
-          ds.writeRawBytes( cstr, cstr.size() );
+          QByteArray cstr( tmpString.toLocal8Bit() );
+          ds.writeRawData( cstr.data(), cstr.size() );
         }
       }
       return true;
-    } 
+    }
     else
       return false;
 }
