@@ -16,9 +16,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  */
 
 #ifndef __KMAIL_IMAPACCOUNTBASE_H__
@@ -28,18 +28,22 @@
 
 #include "networkaccount.h"
 
-#include <qtimer.h>
-#include <qguardedptr.h>
 #include <kio/global.h>
 
+#include <QList>
+#include <QPointer>
+#include <QTimer>
+
+#include <QByteArray>
+
 class AccountManager;
 class KMFolder;
-class KConfig/*Base*/;
+class KConfigGroup;
 class KMMessagePart;
 class DwBodyPart;
 class DwMessage;
 class FolderStorage;
-template <typename T> class QValueVector;
+template <typename T> class QVector;
 
 namespace KIO {
   class Job;
@@ -51,8 +55,8 @@ namespace KPIM {
 
 namespace KMail {
   struct ACLListEntry;
-  struct QuotaInfo;
-  typedef QValueVector<KMail::ACLListEntry> ACLList;
+  class QuotaInfo;
+  typedef QVector<KMail::ACLListEntry> ACLList;
 
   class AttachmentStrategy;
 
@@ -98,8 +102,8 @@ namespace KMail {
     /** Configure the slave by adding to the meta data map */
     virtual KIO::MetaData slaveConfig() const;
 
-    virtual void readConfig( KConfig& config );
-    virtual void writeConfig( KConfig& config );
+    virtual void readConfig( KConfigGroup& config );
+    virtual void writeConfig( KConfigGroup& config );
 
     /**
      * The state of the kioslave connection
@@ -141,7 +145,7 @@ namespace KMail {
     struct jobData
     {
       // Needed by QMap, don't use
-      jobData() : url(QString::null), parent(0), current(0), total(1), done(0), offset(0), progressItem(0),
+      jobData() : parent(0), current(0), total(1), done(0), offset(0), progressItem(0),
                   onlySubscribed(false), quiet(false), cancellable(false) {}
       // Real constructor
       jobData( const QString& _url, KMFolder *_parent = 0,
@@ -154,26 +158,26 @@ namespace KMail {
       QString url;
       QString curNamespace;
       QByteArray data;
-      QCString cdata;
+      QByteArray cdata;
       QStringList items;
       KMFolder *parent, *current;
-      QPtrList<KMMessage> msgList;
+      QList<KMMessage*> msgList;
       int total, done, offset;
       KPIM::ProgressItem *progressItem;
-      bool onlySubscribed, quiet, cancellable;
+      bool onlySubscribed : 1, quiet : 1, cancellable : 1;
     };
 
-    typedef QMap<KIO::Job *, jobData>::Iterator JobIterator;
+    typedef QMap<KJob *, jobData>::Iterator JobIterator;
     /**
      * Call this when starting a new job
      */
-    void insertJob( KIO::Job* job, const jobData& data ) {
+    void insertJob( KJob* job, const jobData& data ) {
       mapJobData.insert( job, data );
     }
     /**
      * Look for the jobData related to a given job. Compare with end()
      */
-    JobIterator findJob( KIO::Job* job ) { return mapJobData.find( job ); }
+    JobIterator findJob( KJob* job ) { return mapJobData.find( job ); }
     JobIterator jobsEnd() { return mapJobData.end(); }
     /**
      * Call this when a job is finished.
@@ -184,7 +188,7 @@ namespace KMail {
     void removeJob( KIO::Job* job );
 
     /**
-     * Subscribe (@p subscribe = TRUE) / Unsubscribe the folder
+     * Subscribe (@p subscribe = true) / Unsubscribe the folder
      * identified by @p imapPath.
      * Emits subscriptionChanged signal on success.
      */
@@ -218,7 +222,7 @@ namespace KMail {
     void getACL( KMFolder* folder, const QString& imapPath );
 
     /**
-     * Retrieve the the quota inforamiton on the folder
+     * Retrieve the quota information on the folder
      * identified by @p imapPath.
      * Emits receivedQuotaInfo signal on success/error.
      */
@@ -228,7 +232,7 @@ namespace KMail {
      * Set the status on the server
      * Emits imapStatusChanged signal on success/error.
      */
-    void setImapStatus( KMFolder* folder, const QString& path, const QCString& flags );
+    void setImapStatus( KMFolder* folder, const QString& path, const QByteArray& flags );
 
     /**
      * Set seen status on the server.
@@ -252,9 +256,10 @@ namespace KMail {
     virtual void cancelMailCheck();
 
     /**
-     * Init a new-mail-check for a single folder
+     * Init a new-mail-check for a single folder, and optionally its subfolders.
      */
-    void processNewMailSingleFolder(KMFolder* folder);
+    enum FolderListType { Single, Recursive };
+    void processNewMailInFolder( KMFolder* folder, FolderListType type = Single );
 
     /**
      * Return true if we are processing a mailcheck for a single folder
@@ -323,7 +328,7 @@ namespace KMail {
     /**
      * Returns the root folder of this account
      */
-    virtual FolderStorage* const rootFolder() const = 0;
+    virtual FolderStorage* rootFolder() const = 0;
 
     /**
      * Progress item for listDir
@@ -421,7 +426,7 @@ namespace KMail {
      * is called when the changeSubscription has finished
      * emits subscriptionChanged
      */
-    void slotSubscriptionResult(KIO::Job * job);
+    void slotSubscriptionResult(KJob * job);
 
   protected slots:
     virtual void slotCheckQueuedFolders();
@@ -434,16 +439,16 @@ namespace KMail {
     /**
      * Only delete information about the job and ignore write errors
      */
-    void slotSetStatusResult(KIO::Job * job);
+    void slotSetStatusResult(KJob * job);
 
     /// Result of getUserRights() job
-    void slotGetUserRightsResult( KIO::Job* _job );
+    void slotGetUserRightsResult( KJob* _job );
 
     /// Result of getACL() job
-    void slotGetACLResult( KIO::Job* _job );
+    void slotGetACLResult( KJob* _job );
 
     /// Result of getStorageQuotaInfo() job
-    void slotGetStorageQuotaInfoResult( KIO::Job* _job );
+    void slotGetStorageQuotaInfoResult( KJob* _job );
 
     /**
      * Send a NOOP command regularly to keep the slave from disconnecting
@@ -462,10 +467,10 @@ namespace KMail {
     /**
      * Only delete information about the job
      */
-    void slotSimpleResult(KIO::Job * job);
+    void slotSimpleResult(KJob * job);
 
     /** Gets and parses the namespaces */
-    void slotNamespaceResult( KIO::Job*, const QString& str );
+    void slotNamespaceResult( KJob*, const QString& str,const QString & );
 
     /**
      * Saves the fetched namespaces
@@ -475,7 +480,7 @@ namespace KMail {
     /**
      * Saves the capabilities list
      */
-    void slotCapabilitiesResult( KIO::Job*, const QString& result );
+    void slotCapabilitiesResult( KJob*, const QString& result,const QString & );
 
   protected:
 
@@ -518,10 +523,10 @@ namespace KMail {
     QString prettifyQuotaError( const QString& _error, KIO::Job * job );
 
   protected:
-    QPtrList<QGuardedPtr<KMFolder> > mOpenFolders;
+    QList<QPointer<KMFolder>* > mOpenFolders;
     QStringList mSubfolderNames, mSubfolderPaths,
         mSubfolderMimeTypes, mSubfolderAttributes;
-    QMap<KIO::Job *, jobData> mapJobData;
+    QMap<KJob *, jobData> mapJobData;
     /** used to detect when the slave has not been used for a while */
     QTimer mIdleTimer;
     /** used to send a noop to the slave in regular intervals to keep it from disonnecting */
@@ -538,6 +543,7 @@ namespace KMail {
 
     bool mErrorDialogIsActive : 1;
     bool mPasswordDialogIsActive : 1;
+    bool mPasswordEnteredAndEmpty : 1;
     bool mACLSupport : 1;
     bool mAnnotationSupport : 1;
     bool mQuotaSupport : 1;
@@ -546,15 +552,15 @@ namespace KMail {
     bool mCheckingSingleFolder : 1;
 
     // folders that should be checked for new mails
-    QValueList<QGuardedPtr<KMFolder> > mMailCheckFolders;
+    QList<QPointer<KMFolder> > mMailCheckFolders;
     // folders that should be checked after the current check is done
-    QValueList<QGuardedPtr<KMFolder> > mFoldersQueuedForChecking;
+    QList<QPointer<KMFolder> > mFoldersQueuedForChecking;
     // holds messageparts from the bodystructure
-    QPtrList<KMMessagePart> mBodyPartList;
+    QList<KMMessagePart*> mBodyPartList;
     // the current message for the bodystructure
     KMMessage* mCurrentMsg;
 
-    QGuardedPtr<KPIM::ProgressItem> mListDirProgressItem;
+    QPointer<KPIM::ProgressItem> mListDirProgressItem;
 
     // our namespaces in the form section=namespaceList
     nsMap mNamespaces;
