@@ -1,9 +1,5 @@
 // kmacctlocal.cpp
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include "kmacctlocal.h"
 #include "kmfoldermbox.h"
 #include "kmacctfolder.h"
@@ -14,14 +10,13 @@ using KPIM::ProgressManager;
 
 #include "kmfoldermgr.h"
 
-#include <kapplication.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
-#include <kconfig.h>
+#include <kconfiggroup.h>
 
-#include <qfileinfo.h>
-#include <qstylesheet.h>
+#include <QFileInfo>
+#include <QTime>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -44,13 +39,6 @@ KMAcctLocal::~KMAcctLocal()
 }
 
 
-//-----------------------------------------------------------------------------
-QString KMAcctLocal::type(void) const
-{
-  return "local";
-}
-
-
 //-----------------------------------------------------------------------------
 void KMAcctLocal::init() {
   KMAccount::init();
@@ -88,7 +76,7 @@ void KMAcctLocal::processNewMail(bool)
       break;
 
     if (t.elapsed() >= 200) { //hardwired constant
-      kapp->processEvents();
+      qApp->processEvents();
       t.start();
     }
   }
@@ -125,61 +113,56 @@ bool KMAcctLocal::preProcess()
 
   //BroadcastStatus::instance()->reset();
   BroadcastStatus::instance()->setStatusMsg(
-	i18n("Preparing transmission from \"%1\"...").arg(mName));
+  i18n("Preparing transmission from \"%1\"...", mName));
 
 
   Q_ASSERT( !mMailCheckProgressItem );
-  QString escapedName = QStyleSheet::escape( mName );
   mMailCheckProgressItem = KPIM::ProgressManager::createProgressItem(
     "MailCheck" + mName,
-    escapedName,
-    i18n("Preparing transmission from \"%1\"...").arg( escapedName ),
+    mName,
+    i18n("Preparing transmission from \"%1\"...", mName),
     false, // cannot be canceled
     false ); // no tls/ssl
 
   // run the precommand
   if (!runPrecommand(precommand()))
   {
-    kdDebug(5006) << "cannot run precommand " << precommand() << endl;
+    kDebug() <<"cannot run precommand" << precommand();
     checkDone( mHasNewMail, CheckError );
     BroadcastStatus::instance()->setStatusMsg( i18n( "Running precommand failed." ));
     return false;
   }
-  
-  const int rc = mMailFolder->open("acctlocalMail");
+
+  const int rc = mMailFolder->open( "acctlocalMail" );
   if ( rc != 0 ) {
     QString aStr;
     aStr = i18n("Cannot open file:");
-    aStr += mMailFolder->path()+"/"+mMailFolder->name();
+    aStr += mMailFolder->path() + '/' + mMailFolder->name();
     KMessageBox::sorry(0, aStr);
-    kdDebug(5006) << "cannot open file " << mMailFolder->path() << "/"
-      << mMailFolder->name() << endl;
+    kDebug() <<"cannot open file" << mMailFolder->path() <<"/"
+      << mMailFolder->name();
     checkDone( mHasNewMail, CheckError );
     BroadcastStatus::instance()->setStatusMsg( i18n( "Transmission failed." ));
     return false;
   }
 
   if (!mboxStorage->isLocked()) {
-    kdDebug(5006) << "mailFolder could not be locked" << endl;
-    mMailFolder->close("acctlocalMail");
+    kDebug() <<"mailFolder could not be locked";
+    mMailFolder->close( "acctlocalMail" );
     checkDone( mHasNewMail, CheckError );
-    QString errMsg = i18n( "Transmission failed: Could not lock %1." )
-      .arg( mMailFolder->location() );
+    QString errMsg = i18n( "Transmission failed: Could not lock %1.",
+        mMailFolder->location() );
     BroadcastStatus::instance()->setStatusMsg( errMsg );
     return false;
   }
 
-  mFolder->open("acctlocalFold");
+  mFolder->open( "acctlocalFold" );
 
   mNumMsgs = mMailFolder->count();
 
   mMailCheckProgressItem->setTotalItems( mNumMsgs );
 
-  // prepare the static parts of the status message:
-  mStatusMsgStub = i18n("Moving message %3 of %2 from %1.")
-    .arg(mMailFolder->location()).arg( mNumMsgs );
-
-  //BroadcastStatus::instance()->setStatusProgressEnable( "L" + mName, true );
+  //BroadcastStatus::instance()->setStatusProgressEnable( 'L' + mName, true );
   return true;
 }
 
@@ -192,7 +175,9 @@ bool KMAcctLocal::fetchMsg()
   /* This causes mail eating
   if (kmkernel->mailCheckAborted()) break; */
 
-  const QString statusMsg = mStatusMsgStub.arg( mMsgsFetched );
+  const QString statusMsg = i18n( "Moving message %1 of %2 from %3.",
+                                  mMsgsFetched, mNumMsgs,
+                                  mMailFolder->location() );
   //BroadcastStatus::instance()->setStatusMsg( statusMsg );
   mMailCheckProgressItem->incCompletedItems();
   mMailCheckProgressItem->updateProgress();
@@ -204,20 +189,22 @@ bool KMAcctLocal::fetchMsg()
 #if 0
     // debug code, don't remove
     QFile fileD0( "testdat_xx-0-0" );
-    if( fileD0.open( IO_WriteOnly ) ) {
-      QCString s = msg->asString();
+    if( fileD0.open( QIODevice::WriteOnly ) ) {
+      QByteArray s = msg->asString();
       uint l = s.length();
       if ( l > 0 ) {
         QDataStream ds( &fileD0 );
-        ds.writeRawBytes( s.data(), l );
+        ds.writeRawData( s.data(), l );
       }
       fileD0.close();  // If data is 0 we just create a zero length file.
     }
 #endif
-    msg->setStatus(msg->headerField("Status").latin1(),
-      msg->headerField("X-Status").latin1());
-    msg->setEncryptionStateChar( msg->headerField( "X-KMail-EncryptionState" ).at(0) );
-    msg->setSignatureStateChar( msg->headerField( "X-KMail-SignatureState" ).at(0));
+    msg->setStatus( msg->headerField( "Status" ).toLatin1(),
+                    msg->headerField( "X-Status" ).toLatin1());
+    if ( !msg->headerField( "X-KMail-EncryptionState" ).isEmpty() )
+      msg->setEncryptionStateChar( msg->headerField( "X-KMail-EncryptionState" ).at(0) );
+    if ( !msg->headerField( "X-KMail-SignatureState" ).isEmpty() )
+      msg->setSignatureStateChar( msg->headerField( "X-KMail-SignatureState" ).at(0));
     msg->setComplete(true);
     msg->updateAttachmentState();
 
@@ -242,34 +229,34 @@ void KMAcctLocal::postProcess()
     if ( rc != 0 ) {
       KMessageBox::queuedMessageBox( 0, KMessageBox::Information,
                                      i18n( "<qt>Cannot remove mail from "
-                                           "mailbox <b>%1</b>:<br>%2</qt>" )
-                                     .arg( mMailFolder->location() )
-                                     .arg( strerror( rc ) ) );
+                                           "mailbox <b>%1</b>:<br />%2</qt>",
+                                       mMailFolder->location(),
+                                       strerror( rc ) ) );
     }
 
     if( mMailCheckProgressItem ) { // do this only once...
       BroadcastStatus::instance()->setStatusMsgTransmissionCompleted( mName, mNumMsgs );
       mMailCheckProgressItem->setStatus(
-        i18n( "Fetched 1 message from mailbox %1.",
-              "Fetched %n messages from mailbox %1.",
-              mNumMsgs ).arg( mMailFolder->location() ) );
+        i18np( "Fetched 1 message from mailbox %2.",
+              "Fetched %1 messages from mailbox %2.",
+              mNumMsgs, mMailFolder->location() ) );
       mMailCheckProgressItem->setComplete();
       mMailCheckProgressItem = 0;
     }
   }
   // else warning is written already
 
-  mMailFolder->close("acctlocalMail");
+  mMailFolder->close( "acctLocalMail" );
   delete mMailFolder; mMailFolder = 0;
 
-  mFolder->close("acctlocalFold");
+  mFolder->close( "acctlocalFold" );
 
   checkDone( mHasNewMail, CheckOK );
 }
 
 
 //-----------------------------------------------------------------------------
-void KMAcctLocal::readConfig(KConfig& config)
+void KMAcctLocal::readConfig(KConfigGroup& config)
 {
   KMAccount::readConfig(config);
   mLocation = config.readPathEntry("Location", mLocation);
@@ -290,7 +277,7 @@ void KMAcctLocal::readConfig(KConfig& config)
 
 
 //-----------------------------------------------------------------------------
-void KMAcctLocal::writeConfig(KConfig& config)
+void KMAcctLocal::writeConfig(KConfigGroup& config)
 {
   KMAccount::writeConfig(config);
 
