@@ -3,6 +3,8 @@
 #include "kmfilteraction.h"
 
 // other KMail headers:
+#include "customtemplates.h"
+#include "customtemplates_kfg.h"
 #include "kmcommands.h"
 #include "kmmsgpart.h"
 #include "kmfiltermgr.h"
@@ -10,11 +12,6 @@
 #include "kmfoldermgr.h"
 #include "messagesender.h"
 #include "kmmainwidget.h"
-#include <kpimidentities/identity.h>
-#include <kpimidentities/identitymanager.h>
-#include <kpimidentities/identitycombo.h>
-#include <kpimutils/kfileio.h>
-#include <mimelib/message.h>
 #include "kmfawidgets.h"
 #include "folderrequester.h"
 #include "mainfolderview.h"
@@ -28,7 +25,16 @@ using KMail::MessageProperty;
 using KMail::ActionScheduler;
 #include "regexplineedit.h"
 using KMail::RegExpLineEdit;
+#include "stringutil.h"
 
+// KDE PIM libs headers
+#include <kpimidentities/identity.h>
+#include <kpimidentities/identitymanager.h>
+#include <kpimidentities/identitycombo.h>
+#include <kpimutils/kfileio.h>
+#include <mimelib/message.h>
+
+// KDE headers
 #include <kcombobox.h>
 #include <ktemporaryfile.h>
 #include <kdebug.h>
@@ -124,7 +130,7 @@ void KMFilterAction::sendMDN( KMMessage * msg, KMime::MDN::DispositionType d,
 
   KMMessage * mdn = msg->createMDN( KMime::MDN::AutomaticAction, d, false, m );
   if ( mdn && !kmkernel->msgSender()->send( mdn, KMail::MessageSender::SendLater ) ) {
-    kDebug(5006) << "Sending failed.";
+    kDebug() << "Sending failed.";
     //delete mdn;
   }
 
@@ -294,8 +300,8 @@ KMFilterActionWithFolder::KMFilterActionWithFolder( const char* aName, const QSt
 
 QWidget* KMFilterActionWithFolder::createParamWidget( QWidget* parent ) const
 {
-  FolderRequester *req = new FolderRequester( parent,
-      kmkernel->getKMMainWidget()->mainFolderView() );
+  FolderRequester *req = new FolderRequester( parent );
+  req->setFolderTree( kmkernel->getKMMainWidget()->mainFolderView() );
   setParamWidgetValue( req );
   return req;
 }
@@ -456,7 +462,7 @@ QString KMFilterActionWithCommand::substituteCommandLineArgsFor( KMMessage *aMsg
       KTemporaryFile *tf = new KTemporaryFile();
       if ( !tf->open() ) {
         delete tf;
-        kDebug(5006) <<"KMFilterActionWithCommand: Could not create temp file!";
+        kDebug() <<"KMFilterActionWithCommand: Could not create temp file!";
         return QString();
       }
       aTempFileList.append( tf );
@@ -559,9 +565,9 @@ KMFilterAction::ReturnCode KMFilterActionWithCommand::genericProcess(KMMessage*
        unfortunate, as we need to removed the original from the folder
        using that, and look it up in the message. When the (new) message
        is uploaded, the header is stripped anyhow. */
-      QString uid = aMsg->headerField("X-UID");
+      QString uid = aMsg->headerField( "X-UID", KMMessage::NoEncoding );
       aMsg->fromString( msgText );
-      aMsg->setHeaderField("X-UID",uid);
+      aMsg->setHeaderField( "X-UID", uid, KMMessage::Unstructured, false, KMMessage::NoEncoding );
     }
     else {
       qDeleteAll( atmList );
@@ -983,7 +989,7 @@ QWidget* KMFilterActionRemoveHeader::createParamWidget( QWidget* parent ) const
 {
   KComboBox *cb = new KComboBox( parent );
   cb->setEditable( true );
-  cb->setInsertPolicy( KComboBox::AtBottom );
+  cb->setInsertPolicy( QComboBox::InsertAtBottom );
   setParamWidgetValue( cb );
   return cb;
 }
@@ -1070,7 +1076,7 @@ QWidget* KMFilterActionAddHeader::createParamWidget( QWidget* parent ) const
   KComboBox *cb = new KComboBox( w );
   cb->setObjectName( "combo" );
   cb->setEditable( true );
-  cb->setInsertPolicy( KComboBox::AtBottom );
+  cb->setInsertPolicy( QComboBox::InsertAtBottom );
   hbl->addWidget( cb, 0 /* stretch */ );
   QLabel *l = new QLabel( i18n("With value:"), w );
   l->setFixedWidth( l->sizeHint().width() );
@@ -1220,7 +1226,7 @@ QWidget* KMFilterActionRewriteHeader::createParamWidget( QWidget* parent ) const
   KComboBox *cb = new KComboBox( w );
   cb->setEditable( true );
   cb->setObjectName( "combo" );
-  cb->setInsertPolicy( KComboBox::AtBottom );
+  cb->setInsertPolicy( QComboBox::InsertAtBottom );
   hbl->addWidget( cb, 0 /* stretch */ );
 
   QLabel *l = new QLabel( i18n("Replace:"), w );
@@ -1447,14 +1453,25 @@ bool KMFilterActionCopy::requiresBody( KMMsgBase *msg ) const
 
 //=============================================================================
 // KMFilterActionForward - forward to
-// Forward message to another user
+// Forward message to another user, with a defined template
 //=============================================================================
 class KMFilterActionForward: public KMFilterActionWithAddress
 {
   public:
     KMFilterActionForward();
-    virtual ReturnCode process( KMMessage *msg ) const;
     static KMFilterAction* newAction( void );
+    virtual ReturnCode process( KMMessage *msg ) const;
+    virtual QWidget* createParamWidget( QWidget* parent ) const;
+    virtual void applyParamWidgetValue( QWidget* paramWidget );
+    virtual void setParamWidgetValue( QWidget* paramWidget ) const;
+    virtual void clearParamWidget( QWidget* paramWidget ) const;
+    virtual void argsFromString( const QString &argsStr );
+    virtual const QString argsAsString() const;
+    virtual const QString displayString() const;
+
+private:
+
+  mutable QString mTemplate;
 };
 
 KMFilterAction *KMFilterActionForward::newAction( void )
@@ -1474,15 +1491,15 @@ KMFilterAction::ReturnCode KMFilterActionForward::process( KMMessage *msg ) cons
 
   // avoid endless loops when this action is used in a filter
   // which applies to sent messages
-  if ( KMMessage::addressIsInAddressList( mParameter, QStringList( msg->to() ) ) ) {
+  if ( KMail::StringUtil::addressIsInAddressList( mParameter, QStringList( msg->to() ) ) ) {
     kWarning() << "Attempt to forward to receipient of original message, ignoring.";
     return ErrorButGoOn;
   }
 
-  KMMessage *fwdMsg = msg->createForward();
-  fwdMsg->setTo( mParameter );
+  KMMessage *fwdMsg = msg->createForward( mTemplate );
+  fwdMsg->setTo( fwdMsg->to() + ',' + mParameter );
 
-  if ( !kmkernel->msgSender()->send( fwdMsg, KMail::MessageSender::SendLater ) ) {
+  if ( !kmkernel->msgSender()->send( fwdMsg, KMail::MessageSender::SendDefault ) ) {
     kWarning() << "KMFilterAction: could not forward message (sending failed)";
     return ErrorButGoOn; // error: couldn't send
   }
@@ -1490,10 +1507,126 @@ KMFilterAction::ReturnCode KMFilterActionForward::process( KMMessage *msg ) cons
     sendMDN( msg, KMime::MDN::Dispatched );
 
   // (the msgSender takes ownership of the message, so don't delete it here)
-
   return GoOn;
 }
 
+QWidget* KMFilterActionForward::createParamWidget( QWidget* parent ) const
+{
+  QWidget *addressAndTemplate = new QWidget( parent );
+  QHBoxLayout *hBox = new QHBoxLayout( addressAndTemplate );
+  hBox->setMargin( 0 );
+  QWidget *addressEdit = KMFilterActionWithAddress::createParamWidget( addressAndTemplate );
+  addressEdit->setObjectName( "addressEdit" );
+  hBox->addWidget( addressEdit );
+
+  KLineEdit *lineEdit = addressEdit->findChild<KLineEdit*>( "addressEdit" );
+  Q_ASSERT( lineEdit );
+  lineEdit->setToolTip( i18n( "The addressee to whom the message will be forwarded." ) );
+  lineEdit->setWhatsThis( i18n( "The filter will forward the message to the addressee entered here." ) );
+
+  QComboBox *templateCombo = new QComboBox( addressAndTemplate );
+  templateCombo->setObjectName( "templateCombo" );
+  hBox->addWidget( templateCombo );
+
+  templateCombo->addItem( i18n( "Default Template" ) );
+  QStringList templateNames = GlobalSettingsBase::self()->customTemplates();
+  foreach( const QString &templateName, templateNames ) {
+    CTemplates templat( templateName );
+    if ( templat.type() == CustomTemplates::TForward ||
+         templat.type() == CustomTemplates::TUniversal )
+      templateCombo->addItem( templateName );
+  }
+  templateCombo->setEnabled( templateCombo->count() > 1 );
+  templateCombo->setToolTip( i18n( "The template used when forwarding" ) );
+  templateCombo->setWhatsThis( i18n( "Set the forwarding template that will be used with this filter." ) );
+
+  return addressAndTemplate;
+}
+
+void KMFilterActionForward::applyParamWidgetValue( QWidget* paramWidget )
+{
+  QWidget *addressEdit = paramWidget->findChild<QWidget*>( "addressEdit" );
+  Q_ASSERT( addressEdit );
+  KMFilterActionWithAddress::applyParamWidgetValue( addressEdit );
+
+  QComboBox *templateCombo = paramWidget->findChild<QComboBox*>( "templateCombo" );
+  Q_ASSERT( templateCombo );
+
+  if ( templateCombo->currentIndex() == 0 ) {
+    // Default template, so don't use a custom one
+    mTemplate.clear();
+  }
+  else {
+    mTemplate = templateCombo->currentText();
+  }
+}
+
+void KMFilterActionForward::setParamWidgetValue( QWidget* paramWidget ) const
+{
+  QWidget *addressEdit = paramWidget->findChild<QWidget*>( "addressEdit" );
+  Q_ASSERT( addressEdit );
+  KMFilterActionWithAddress::setParamWidgetValue( addressEdit );
+
+  QComboBox *templateCombo = paramWidget->findChild<QComboBox*>( "templateCombo" );
+  Q_ASSERT( templateCombo );
+
+  if ( mTemplate.isEmpty() ) {
+    templateCombo->setCurrentIndex( 0 );
+  }
+  else {
+    int templateIndex = templateCombo->findText( mTemplate );
+    if ( templateIndex != -1 ) {
+      templateCombo->setCurrentIndex( templateIndex );
+    }
+    else {
+      mTemplate.clear();
+    }
+  }
+}
+
+void KMFilterActionForward::clearParamWidget( QWidget* paramWidget ) const
+{
+  QWidget *addressEdit = paramWidget->findChild<QWidget*>( "addressEdit" );
+  Q_ASSERT( addressEdit );
+  KMFilterActionWithAddress::clearParamWidget( addressEdit );
+
+  QComboBox *templateCombo = paramWidget->findChild<QComboBox*>( "templateCombo" );
+  Q_ASSERT( templateCombo );
+
+  templateCombo->setCurrentIndex( 0 );
+}
+
+// We simply place a "@$$@" between the two parameters. The template is the last
+// parameter in the string, for compatibility reasons.
+static const QString forwardFilterArgsSeperator = "@$$@";
+
+void KMFilterActionForward::argsFromString( const QString &argsStr )
+{
+  int seperatorPos = argsStr.indexOf( forwardFilterArgsSeperator );
+
+  if ( seperatorPos == - 1 ) {
+    // Old config, assume that the whole string is the addressee
+    KMFilterActionWithAddress::argsFromString( argsStr );
+  }
+  else {
+    QString addressee = argsStr.left( seperatorPos );
+    mTemplate = argsStr.mid( seperatorPos + forwardFilterArgsSeperator.length() );
+    KMFilterActionWithAddress::argsFromString( addressee );
+  }
+}
+
+const QString KMFilterActionForward::argsAsString() const
+{
+  return KMFilterActionWithAddress::argsAsString() + forwardFilterArgsSeperator + mTemplate;
+}
+
+const QString KMFilterActionForward::displayString() const
+{
+  if ( mTemplate.isEmpty() )
+    return i18n( "Forward to %1 with default template", mParameter );
+  else
+    return i18n( "Forward to %1 with template %2", mParameter, mTemplate );
+}
 
 //=============================================================================
 // KMFilterActionRedirect - redirect to
@@ -1528,7 +1661,7 @@ KMFilterAction::ReturnCode KMFilterActionRedirect::process(KMMessage* aMsg) cons
   sendMDN( aMsg, KMime::MDN::Dispatched );
 
   if ( !kmkernel->msgSender()->send( msg, KMail::MessageSender::SendLater ) ) {
-    kDebug(5006) <<"KMFilterAction: could not redirect message (sending failed)";
+    kDebug() <<"KMFilterAction: could not redirect message (sending failed)";
     return ErrorButGoOn; // error: couldn't send
   }
   return GoOn;
@@ -1573,6 +1706,8 @@ KMFilterAction::ReturnCode KMFilterActionExec::process(KMMessage *aMsg) const
 #include <threadweaver/DebuggingAids.h>
 class PipeJob : public ThreadWeaver::Job
 {
+  Q_OBJECT
+
   public:
     PipeJob(QObject* parent = 0, KMMessage* aMsg = 0, QString cmd = 0, QString tempFileName = 0 )
       : ThreadWeaver::Job ( parent ),
@@ -1580,18 +1715,22 @@ class PipeJob : public ThreadWeaver::Job
         mCmd( cmd ),
         mMsg( aMsg )
     {
+      connect( this, SIGNAL( done(  ThreadWeaver::Job* ) ),
+               this, SLOT( slotDone( ThreadWeaver::Job* ) ) );
     }
 
     ~PipeJob() {}
 
-    virtual void done( ThreadWeaver::Job *job )
+  private slots:
+
+    void slotDone( ThreadWeaver::Job * )
     {
-      ThreadWeaver::Job::done( job );
       deleteLater( );
     }
 
   protected:
-    void run()
+
+    virtual void run()
     {
       ThreadWeaver::debug (1, "PipeJob::run: doing it .\n");
       FILE *p;
@@ -1616,16 +1755,24 @@ class PipeJob : public ThreadWeaver::Job
         KMFolder *filterFolder =  mMsg->parent();
         ActionScheduler *handler = MessageProperty::filterHandler( mMsg->getMsgSerNum() );
 
+        // If the pipe through alters the message, it could very well
+        // happen that it no longer has a X-UID header afterwards. That is
+        // unfortunate, as we need to removed the original from the folder
+        // using that, and look it up in the message. When the (new) message
+        // is uploaded, the header is stripped anyhow. */
+        const QString uid = mMsg->headerField( "X-UID", KMMessage::NoEncoding );
         mMsg->fromString( ba );
+        if ( !uid.isEmpty() )
+          mMsg->setHeaderField( "X-UID", uid, KMMessage::Unstructured, false, KMMessage::NoEncoding );
+
         if ( !origSerNum.isEmpty() )
           mMsg->setHeaderField( "X-KMail-Filtered", origSerNum );
         if ( filterFolder && handler ) {
-          bool oldStatus = handler->ignoreChanges( true );
+          handler->addMsgToIgnored( mMsg->getMsgSerNum() );
           filterFolder->take( filterFolder->find( mMsg ) );
           filterFolder->addMsg( mMsg );
-          handler->ignoreChanges( oldStatus );
         } else {
-          kDebug(5006) <<"Warning: Cannot refresh the message from the external filter.";
+          kDebug() <<"Warning: Cannot refresh the message from the external filter.";
         }
       }
 
@@ -1706,7 +1853,7 @@ void KMFilterActionExtFilter::processAsync(KMMessage* aMsg) const
   atmList.clear();
 
   PipeJob *job = new PipeJob(0, aMsg, commandLine, tempFileName);
-  QObject::connect ( job, SIGNAL( done() ),
+  QObject::connect ( job, SIGNAL( done( ThreadWeaver::Job* ) ),
                      handler, SLOT( actionMessage() ) );
   kmkernel->weaver()->enqueue(job);
 }
@@ -1823,19 +1970,20 @@ KMFilterActionWithUrl::~KMFilterActionWithUrl()
 QWidget* KMFilterActionWithUrl::createParamWidget( QWidget* parent ) const
 {
   KUrlRequester *le = new KUrlRequester(parent);
-  le->setPath( mParameter );
+  le->setUrl( KUrl( mParameter ) );
   return le;
 }
 
 
 void KMFilterActionWithUrl::applyParamWidgetValue( QWidget* paramWidget )
 {
-  mParameter = ((KUrlRequester*)paramWidget)->url().path();
+    const KUrl url = ((KUrlRequester*)paramWidget)->url();
+  mParameter = url.isLocalFile() ? url.toLocalFile() : url.path();
 }
 
 void KMFilterActionWithUrl::setParamWidgetValue( QWidget* paramWidget ) const
 {
-  ((KUrlRequester*)paramWidget)->setPath( mParameter );
+  ((KUrlRequester*)paramWidget)->setUrl( KUrl( mParameter ) );
 }
 
 void KMFilterActionWithUrl::clearParamWidget( QWidget* paramWidget ) const
@@ -1911,3 +2059,5 @@ void KMFilterActionDict::insert( KMFilterActionNewFunc aNewFunc )
   mList.append( desc );
   delete action;
 }
+
+#include "kmfilteraction.moc"
