@@ -17,7 +17,6 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
-#include <config.h>
 
 #include "kmfolder.h"
 #include "kmfolderdir.h"
@@ -27,26 +26,28 @@
 #include "kmfoldersearch.h"
 #include "kmfolderimap.h"
 #include "kmfoldermgr.h"
-#include <libkpimidentities/identitymanager.h>
-#include <libkpimidentities/identity.h>
+#include <kpimidentities/identitymanager.h>
+#include <kpimidentities/identity.h>
 #include "expirejob.h"
 #include "compactionjob.h"
-#include "kmfoldertree.h"
 #include "kmailicalifaceimpl.h"
 #include "kmaccount.h"
 
 #include <errno.h>
+#include <unistd.h> // W_OK
 
 #include <kdebug.h>
+#include <kde_file.h> // KDE_mkdir
 #include <klocale.h>
 #include <kshortcut.h>
 #include <kmessagebox.h>
-#include <qfile.h>
-#include <qfileinfo.h>
 
+#include <QFile>
+#include <QFileInfo>
+#include <QList>
 
 KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
-                             KMFolderType aFolderType, bool withIndex, bool exportedSernums )
+                    KMFolderType aFolderType, bool withIndex, bool exportedSernums )
   : KMFolderNode( aParent, aFolderName ), mStorage(0),
     mChild( 0 ),
     mIsSystemFolder( false ),
@@ -59,20 +60,20 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
     mExpireAction( ExpireDelete ),
     mUseCustomIcons( false ), mMailingListEnabled( false ),
     mAcctList( 0 ),
-    mIdentity( 0 ), // default identity
     mPutRepliesInSameFolder( false ),
     mIgnoreNewMail( false )
 {
+  mIdentity = kmkernel->identityManager()->defaultIdentity().uoid();
   if( aFolderType == KMFolderTypeCachedImap )
-    mStorage = new KMFolderCachedImap( this, aFolderName.latin1() );
+    mStorage = new KMFolderCachedImap( this, aFolderName.toLatin1() );
   else if( aFolderType == KMFolderTypeImap )
-    mStorage = new KMFolderImap( this, aFolderName.latin1() );
+    mStorage = new KMFolderImap( this, aFolderName.toLatin1() );
   else if( aFolderType == KMFolderTypeMaildir )
-    mStorage = new KMFolderMaildir( this, aFolderName.latin1() );
+    mStorage = new KMFolderMaildir( this, aFolderName.toLatin1() );
   else if( aFolderType == KMFolderTypeSearch )
-    mStorage = new KMFolderSearch( this, aFolderName.latin1() );
+    mStorage = new KMFolderSearch( this, aFolderName.toLatin1() );
   else
-    mStorage = new KMFolderMbox( this, aFolderName.latin1() );
+    mStorage = new KMFolderMbox( this, aFolderName.toLatin1() );
 
   assert( mStorage );
 
@@ -80,19 +81,19 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
   dirinfo.setFile( mStorage->location() );
   if ( !dirinfo.exists() ) {
     int rc = mStorage->create();
-    QString msg = i18n("<qt>Error while creating file <b>%1</b>:<br>%2</qt>").arg(aFolderName).arg(strerror(rc));
+    QString msg = i18n("<qt>Error while creating file <b>%1</b>:<br />%2</qt>", aFolderName, strerror(rc));
     if ( rc ) {
       KMessageBox::information(0, msg);
     }
   }
 
   if ( aParent ) {
-    connect( mStorage, SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ),
-             aParent->manager(), SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ) );
-    connect( mStorage, SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ),
-             parent()->manager(), SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ) );
-    connect( this, SIGNAL( msgChanged( KMFolder*, Q_UINT32, int ) ),
-             parent()->manager(), SIGNAL( msgChanged( KMFolder*, Q_UINT32, int ) ) );
+    connect( mStorage, SIGNAL( msgAdded( KMFolder*, quint32 ) ),
+             aParent->manager(), SIGNAL( msgAdded( KMFolder*, quint32 ) ) );
+    connect( mStorage, SIGNAL( msgRemoved( KMFolder*, quint32 ) ),
+             parent()->manager(), SIGNAL( msgRemoved( KMFolder*, quint32 ) ) );
+    connect( this, SIGNAL( msgChanged( KMFolder*, quint32, int ) ),
+             parent()->manager(), SIGNAL( msgChanged( KMFolder*, quint32, int ) ) );
     connect( this, SIGNAL( msgHeaderChanged( KMFolder*,  int ) ),
              parent()->manager(), SIGNAL( msgHeaderChanged( KMFolder*, int ) ) );
     connect( mStorage, SIGNAL( invalidated( KMFolder* ) ),
@@ -101,21 +102,22 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
 
   // Resend all mStorage signals
   connect( mStorage, SIGNAL( changed() ), SIGNAL( changed() ) );
+  connect( mStorage, SIGNAL( compacted() ), SIGNAL( compacted() ) );
   connect( mStorage, SIGNAL( cleared() ), SIGNAL( cleared() ) );
   connect( mStorage, SIGNAL( expunged( KMFolder* ) ),
            SIGNAL( expunged( KMFolder* ) ) );
   connect( mStorage, SIGNAL( nameChanged() ), SIGNAL( nameChanged() ) );
-  connect( mStorage, SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ),
-           SIGNAL( msgRemoved( KMFolder*, Q_UINT32 ) ) );
-  connect( mStorage, SIGNAL( msgRemoved( int, QString ) ),
-           SIGNAL( msgRemoved( int, QString ) ) );
+  connect( mStorage, SIGNAL( msgRemoved( KMFolder*, quint32 ) ),
+           SIGNAL( msgRemoved( KMFolder*, quint32 ) ) );
+  connect( mStorage, SIGNAL( msgRemoved( int, const QString& ) ),
+           SIGNAL( msgRemoved( int, const QString& ) ) );
   connect( mStorage, SIGNAL( msgRemoved( KMFolder* ) ),
            SIGNAL( msgRemoved( KMFolder* ) ) );
   connect( mStorage, SIGNAL( msgAdded( int ) ), SIGNAL( msgAdded( int ) ) );
-  connect( mStorage, SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ),
-           SIGNAL( msgAdded( KMFolder*, Q_UINT32 ) ) );
-  connect( mStorage, SIGNAL( msgChanged( KMFolder*, Q_UINT32 , int ) ),
-           SIGNAL( msgChanged( KMFolder*, Q_UINT32 , int ) ) );
+  connect( mStorage, SIGNAL( msgAdded( KMFolder*, quint32 ) ),
+           SIGNAL( msgAdded( KMFolder*, quint32 ) ) );
+  connect( mStorage, SIGNAL( msgChanged( KMFolder*, quint32, int ) ),
+           SIGNAL( msgChanged( KMFolder*, quint32, int ) ) );
   connect( mStorage, SIGNAL( msgHeaderChanged( KMFolder*, int ) ),
            SIGNAL( msgHeaderChanged( KMFolder*, int ) ) );
   connect( mStorage, SIGNAL( statusMsg( const QString& ) ),
@@ -124,13 +126,18 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
            SIGNAL( numUnreadMsgsChanged( KMFolder* ) ) );
   connect( mStorage, SIGNAL( removed( KMFolder*, bool ) ),
            SIGNAL( removed( KMFolder*, bool ) ) );
+  connect( mStorage, SIGNAL(noContentChanged()),
+           SIGNAL(noContentChanged()) );
 
   connect( mStorage, SIGNAL( contentsTypeChanged( KMail::FolderContentsType ) ),
                 this, SLOT( slotContentsTypeChanged( KMail::FolderContentsType ) ) );
-  
+
   connect( mStorage, SIGNAL( folderSizeChanged() ),
            this, SLOT( slotFolderSizeChanged() ) );
 
+  connect( kmkernel->identityManager(), SIGNAL( changed() ),
+           this, SLOT( slotIdentitiesChanged() ) );
+
   //FIXME: Centralize all the readConfig calls somehow - Zack
   // Meanwhile, readConfig must be done before registerWithMessageDict, since
   // that one can call writeConfig in some circumstances - David
@@ -148,81 +155,112 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
 
 KMFolder::~KMFolder()
 {
+  mStorage->close( "~KMFolder", true );
   delete mAcctList;
   if ( mHasIndex ) mStorage->deregisterFromMessageDict();
   delete mStorage;
 }
 
-void KMFolder::readConfig( KConfig* config )
+bool KMFolder::hasDescendant( KMFolder *fld ) const
 {
-  if ( !config->readEntry("SystemLabel").isEmpty() )
-    mSystemLabel = config->readEntry("SystemLabel");
-  mExpireMessages = config->readBoolEntry("ExpireMessages", false);
-  mReadExpireAge = config->readNumEntry("ReadExpireAge", 3);
-  mReadExpireUnits = (ExpireUnits)config->readNumEntry("ReadExpireUnits", expireMonths);
-  mUnreadExpireAge = config->readNumEntry("UnreadExpireAge", 12);
-  mUnreadExpireUnits = (ExpireUnits)config->readNumEntry("UnreadExpireUnits", expireNever);
-  mExpireAction = config->readEntry("ExpireAction", "Delete") == "Move" ? ExpireMove : ExpireDelete;
-  mExpireToFolderId = config->readEntry("ExpireToFolder");
+  if ( !fld )
+    return false;
+  KMFolderDir * pdir = fld->parent();
+  if ( !pdir )
+    return false;
+  fld = pdir->owner();
+  if ( fld == this )
+    return true;
+  return hasDescendant( fld );
+}
 
-  mUseCustomIcons = config->readBoolEntry("UseCustomIcons", false );
-  mNormalIconPath = config->readEntry("NormalIconPath" );
-  mUnreadIconPath = config->readEntry("UnreadIconPath" );
+KMFolder * KMFolder::ownerFolder() const
+{
+  if ( !mParent )
+    return 0;
+  return mParent->owner();
+}
 
-  mMailingListEnabled = config->readBoolEntry("MailingListEnabled");
-  mMailingList.readConfig( config );
 
-  mIdentity = config->readUnsignedNumEntry("Identity",0);
+void KMFolder::readConfig( KConfigGroup & configGroup )
+{
+  // KConfigGroup configGroup(config, "");
+  if ( !configGroup.readEntry( "SystemLabel" ).isEmpty() )
+    mSystemLabel = configGroup.readEntry( "SystemLabel" );
+  mExpireMessages = configGroup.readEntry( "ExpireMessages", false );
+  mReadExpireAge = configGroup.readEntry( "ReadExpireAge", 3 );
+  mReadExpireUnits = (ExpireUnits)configGroup.readEntry( "ReadExpireUnits", (int)expireMonths );
+  mUnreadExpireAge = configGroup.readEntry( "UnreadExpireAge", 12 );
+  mUnreadExpireUnits = (ExpireUnits)
+      configGroup.readEntry( "UnreadExpireUnits", (int)expireNever );
+  mExpireAction = configGroup.readEntry( "ExpireAction", "Delete") == "Move" ? ExpireMove : ExpireDelete;
+  mExpireToFolderId = configGroup.readEntry( "ExpireToFolder" );
 
-  setUserWhoField( config->readEntry("WhoField"), false );
-  uint savedId = config->readUnsignedNumEntry("Id", 0);
+  mUseCustomIcons = configGroup.readEntry( "UseCustomIcons", false );
+  mNormalIconPath = configGroup.readEntry( "NormalIconPath" );
+  mUnreadIconPath = configGroup.readEntry( "UnreadIconPath" );
+
+  mMailingListEnabled = configGroup.readEntry( "MailingListEnabled", false );
+  mMailingList.readConfig( configGroup );
+
+  mUseDefaultIdentity = configGroup.readEntry( "UseDefaultIdentity", true );
+  uint defaultIdentity = kmkernel->identityManager()->defaultIdentity().uoid();
+  mIdentity = configGroup.readEntry("Identity", defaultIdentity );
+  slotIdentitiesChanged();
+
+  setUserWhoField( configGroup.readEntry( "WhoField" ), false );
+  uint savedId = configGroup.readEntry( "Id", 0 );
   // make sure that we don't overwrite a valid id
   if ( savedId != 0 && mId == 0 )
     mId = savedId;
-  mPutRepliesInSameFolder = config->readBoolEntry( "PutRepliesInSameFolder", false );
-  mIgnoreNewMail = config->readBoolEntry( "IgnoreNewMail", false );
+  mPutRepliesInSameFolder = configGroup.readEntry( "PutRepliesInSameFolder", false );
+  mIgnoreNewMail = configGroup.readEntry( "IgnoreNewMail", false );
 
   if ( mUseCustomIcons )
     emit iconsChanged();
 
-  QString shortcut( config->readEntry( "Shortcut" ) );
+  QString shortcut( configGroup.readEntry( "Shortcut" ) );
   if ( !shortcut.isEmpty() ) {
     KShortcut sc( shortcut );
     setShortcut( sc );
   }
 }
 
-void KMFolder::writeConfig( KConfig* config ) const
+void KMFolder::writeConfig( KConfigGroup & configGroup ) const
 {
-  config->writeEntry("SystemLabel", mSystemLabel);
-  config->writeEntry("ExpireMessages", mExpireMessages);
-  config->writeEntry("ReadExpireAge", mReadExpireAge);
-  config->writeEntry("ReadExpireUnits", mReadExpireUnits);
-  config->writeEntry("UnreadExpireAge", mUnreadExpireAge);
-  config->writeEntry("UnreadExpireUnits", mUnreadExpireUnits);
-  config->writeEntry("ExpireAction", mExpireAction == ExpireDelete ? "Delete" : "Move");
-  config->writeEntry("ExpireToFolder", mExpireToFolderId);
+  // KConfigGroup configGroup(config, "");
+  configGroup.writeEntry("SystemLabel", mSystemLabel);
+  configGroup.writeEntry("ExpireMessages", mExpireMessages);
+  configGroup.writeEntry("ReadExpireAge", mReadExpireAge);
+  configGroup.writeEntry("ReadExpireUnits", (int)mReadExpireUnits);
+  configGroup.writeEntry("UnreadExpireAge", mUnreadExpireAge);
+  configGroup.writeEntry("UnreadExpireUnits", (int)mUnreadExpireUnits);
+  configGroup.writeEntry("ExpireAction", mExpireAction == ExpireDelete ? "Delete" : "Move");
+  configGroup.writeEntry("ExpireToFolder", mExpireToFolderId);
 
-  config->writeEntry("UseCustomIcons", mUseCustomIcons);
-  config->writeEntry("NormalIconPath", mNormalIconPath);
-  config->writeEntry("UnreadIconPath", mUnreadIconPath);
+  configGroup.writeEntry("UseCustomIcons", mUseCustomIcons);
+  configGroup.writeEntry("NormalIconPath", mNormalIconPath);
+  configGroup.writeEntry("UnreadIconPath", mUnreadIconPath);
 
-  config->writeEntry("MailingListEnabled", mMailingListEnabled);
-  mMailingList.writeConfig( config );
+  configGroup.writeEntry("MailingListEnabled", mMailingListEnabled);
+  mMailingList.writeConfig( configGroup );
 
-  if ( mIdentity != 0 && ( !mStorage || !mStorage->account() || mIdentity != mStorage->account()->identityId() ) )
-      config->writeEntry("Identity", mIdentity);
+
+  configGroup.writeEntry( "UseDefaultIdentity", mUseDefaultIdentity );
+  if ( !mUseDefaultIdentity && ( !mStorage || !mStorage->account() ||
+                           mIdentity != mStorage->account()->identityId() ) )
+      configGroup.writeEntry("Identity", mIdentity);
   else
-      config->deleteEntry("Identity");
+      configGroup.deleteEntry("Identity");
 
-  config->writeEntry("WhoField", mUserWhoField);
-  config->writeEntry("Id", mId);
-  config->writeEntry( "PutRepliesInSameFolder", mPutRepliesInSameFolder );
-  config->writeEntry( "IgnoreNewMail", mIgnoreNewMail );
-  if ( !mShortcut.isNull() )
-    config->writeEntry( "Shortcut", mShortcut.toString() );
+  configGroup.writeEntry("WhoField", mUserWhoField);
+  configGroup.writeEntry("Id", mId);
+  configGroup.writeEntry( "PutRepliesInSameFolder", mPutRepliesInSameFolder );
+  configGroup.writeEntry( "IgnoreNewMail", mIgnoreNewMail );
+  if ( !mShortcut.isEmpty() )
+    configGroup.writeEntry( "Shortcut", mShortcut.toString() );
   else
-    config->deleteEntry( "Shortcut" );
+    configGroup.deleteEntry( "Shortcut" );
 }
 
 KMFolderType KMFolder::folderType() const
@@ -232,17 +270,27 @@ KMFolderType KMFolder::folderType() const
 
 QString KMFolder::fileName() const
 {
-  return mStorage ? mStorage->fileName() : QString::null;
+  return mStorage ? mStorage->fileName() : QString();
 }
 
 QString KMFolder::location() const
 {
-  return mStorage ? mStorage->location() : QString::null;
+  return mStorage ? mStorage->location() : QString();
 }
 
 QString KMFolder::indexLocation() const
 {
-  return mStorage ? mStorage->indexLocation() : QString::null;
+  return mStorage ? mStorage->indexLocation() : QString();
+}
+
+QString KMFolder::sortedLocation() const
+{
+  return mStorage ? mStorage->sortedLocation() : QString();
+}
+
+QString KMFolder::idsLocation() const
+{
+  return mStorage ? mStorage->idsLocation() : QString();
 }
 
 QString KMFolder::subdirLocation() const
@@ -258,26 +306,28 @@ QString KMFolder::subdirLocation() const
 
 KMFolderDir* KMFolder::createChildFolder()
 {
-  if( mChild )
+  if ( mChild ) {
     return mChild;
+  }
 
-  QString childName = "." + fileName() + ".directory";
-  QString childDir = path() + "/" + childName;
-  if (access(QFile::encodeName(childDir), W_OK) != 0) // Not there or not writable
-  {
-    if (mkdir(QFile::encodeName(childDir), S_IRWXU) != 0
-      && chmod(QFile::encodeName(childDir), S_IRWXU) != 0) {
-      QString wmsg = QString(" '%1': %2").arg(childDir).arg(strerror(errno));
-      KMessageBox::information(0,i18n("Failed to create folder") + wmsg);
+  QString childName = '.' + fileName() + ".directory";
+  QString childDir = path() + '/' + childName;
+  if ( access( QFile::encodeName( childDir ), W_OK ) != 0 ) {
+    // childDir does not exist or is not writable, so create it.
+    if ( KDE_mkdir( QFile::encodeName(childDir), S_IRWXU ) != 0 &&
+         chmod( QFile::encodeName(childDir), S_IRWXU) != 0 ) {
+      QString wmsg = QString( " '%1': %2" ).arg( childDir ).arg( strerror( errno ) );
+      KMessageBox::information( 0, i18n( "Failed to create folder" ) + wmsg );
       return 0;
     }
   }
 
   KMFolderDirType newType = KMStandardDir;
-  if( folderType() == KMFolderTypeCachedImap )
+  if ( folderType() == KMFolderTypeCachedImap ) {
     newType = KMDImapDir;
-  else if( folderType() == KMFolderTypeImap )
+  } else if ( folderType() == KMFolderTypeImap ) {
     newType = KMImapDir;
+  }
 
   mChild = new KMFolderDir( this, parent(), childName, newType );
   if( !mChild )
@@ -339,13 +389,13 @@ void KMFolder::ignoreJobsForMessage( KMMessage* m )
 }
 
 FolderJob* KMFolder::createJob( KMMessage *msg, FolderJob::JobType jt,
-                                KMFolder *folder, QString partSpecifier,
+                                KMFolder *folder, const QString &partSpecifier,
                                 const AttachmentStrategy *as ) const
 {
   return mStorage->createJob( msg, jt, folder, partSpecifier, as );
 }
 
-FolderJob* KMFolder::createJob( QPtrList<KMMessage>& msgList,
+FolderJob* KMFolder::createJob( QList<KMMessage*>& msgList,
                                 const QString& sets,
                                 FolderJob::JobType jt, KMFolder *folder ) const
 {
@@ -377,9 +427,9 @@ KMMessage* KMFolder::take( int idx )
   return mStorage->take( idx );
 }
 
-void KMFolder::take( QPtrList<KMMessage> msgList ) // TODO const ref
+void KMFolder::takeMessages( const QList<KMMessage*>& msgList )
 {
-  mStorage->take( msgList );
+  mStorage->takeMessages( msgList );
 }
 
 int KMFolder::addMsg( KMMessage* msg, int* index_return )
@@ -392,9 +442,9 @@ int KMFolder::addMsgKeepUID( KMMessage* msg, int* index_return )
   return mStorage->addMsgKeepUID( msg, index_return );
 }
 
-int KMFolder::addMsg( QPtrList<KMMessage>& list, QValueList<int>& index_return )
+int KMFolder::addMessages( QList<KMMessage*>& list, QList<int>& index_return )
 {
-  return mStorage->addMsg( list, index_return );
+  return mStorage->addMessages( list, index_return );
 }
 
 void KMFolder::emitMsgAddedSignals( int idx )
@@ -407,9 +457,9 @@ void KMFolder::removeMsg( int i, bool imapQuiet )
   mStorage->removeMsg( i, imapQuiet );
 }
 
-void KMFolder::removeMsg( QPtrList<KMMessage> msgList, bool imapQuiet ) // TODO const ref
+void KMFolder::removeMessages( QList<KMMessage*> msgList, bool imapQuiet ) // TODO const ref
 {
-  mStorage->removeMsg( msgList, imapQuiet );
+  mStorage->removeMessages( msgList, imapQuiet );
 }
 
 int KMFolder::expungeOldMsg( int days )
@@ -422,19 +472,19 @@ int KMFolder::moveMsg( KMMessage* msg, int* index_return )
   return mStorage->moveMsg( msg, index_return );
 }
 
-int KMFolder::moveMsg(QPtrList<KMMessage> q, int* index_return )
+int KMFolder::moveMsg(QList<KMMessage*> q, int* index_return )
 {
   return mStorage->moveMsg( q, index_return );
 }
 
 int KMFolder::find( const KMMsgBase* msg ) const
 {
-  return mStorage ? mStorage->find( msg ) : -1;
+  return mStorage->find( msg );
 }
 
 int KMFolder::find( const KMMessage* msg ) const
 {
-  return mStorage ? mStorage->find( msg ) : -1;
+  return mStorage->find( msg );
 }
 
 int KMFolder::count( bool cache ) const
@@ -455,28 +505,34 @@ int KMFolder::countUnreadRecursive()
   if (!dir)
     return count;
 
-  QPtrListIterator<KMFolderNode> it(*dir);
-  for ( ; it.current(); ++it )
-    if (!it.current()->isDir()) {
-      folder = static_cast<KMFolder*>(it.current());
+  QList<KMFolderNode*>::const_iterator it;
+  for ( it = dir->constBegin(); it != dir->constEnd(); ++it )
+    if ( !( (*it)->isDir() ) ) {
+      folder = static_cast<KMFolder*>( (*it) );
       count += folder->countUnreadRecursive();
     }
 
   return count;
 }
 
-void KMFolder::msgStatusChanged( const KMMsgStatus oldStatus,
-                                 const KMMsgStatus newStatus, int idx )
+void KMFolder::msgStatusChanged( const MessageStatus& oldStatus,
+                                 const MessageStatus& newStatus, int idx )
 {
   mStorage->msgStatusChanged( oldStatus, newStatus, idx );
 }
 
-int KMFolder::open(const char *owner)
+void KMFolder::msgTagListChanged( int idx )
 {
-  return mStorage->open(owner);
+  emit msgHeaderChanged( this, idx );
 }
 
-int KMFolder::canAccess()
+
+int KMFolder::open( const char *owner )
+{
+  return mStorage->open( owner );
+}
+
+bool KMFolder::canAccess() const
 {
   return mStorage->canAccess();
 }
@@ -514,7 +570,7 @@ void KMFolder::remove()
      the message dict, since its message list is empty, and the .ids
      file contents are not loaded. That can lead to lookups in the
      dict returning stale pointers to the folder later. */
-  mStorage->open("kmfolder_remove");
+  mStorage->open("folderremoval");
   mStorage->remove();
 }
 
@@ -558,6 +614,11 @@ bool KMFolder::isReadOnly() const
   return mStorage->isReadOnly();
 }
 
+bool KMFolder::canDeleteMessages() const
+{
+  return mStorage->canDeleteMessages();
+}
+
 QString KMFolder::label() const
 {
   if ( !mSystemLabel.isEmpty() )
@@ -565,16 +626,16 @@ QString KMFolder::label() const
   if ( !mLabel.isEmpty() )
      return mLabel;
   if ( isSystemFolder() )
-     return i18n( name().utf8() );
+     return i18n( name().toUtf8() );
   return name();
 }
 
 //-----------------------------------------------------------------------------
-QString KMFolder::prettyURL() const
+QString KMFolder::prettyUrl() const
 {
   QString parentUrl;
   if ( parent() )
-    parentUrl = parent()->prettyURL();
+    parentUrl = parent()->prettyUrl();
   if ( !parentUrl.isEmpty() )
     return parentUrl + '/' + label();
   else
@@ -585,16 +646,16 @@ QString KMFolder::prettyURL() const
 QString KMFolder::mailingListPostAddress() const
 {
   if ( mMailingList.features() & MailingList::Post ) {
-    KURL::List::const_iterator it;
-    KURL::List post = mMailingList.postURLS();
-    for( it = post.begin(); it != post.end(); ++it ) {
+    KUrl::List::const_iterator it;
+    KUrl::List post = mMailingList.postURLS();
+    for( it = post.constBegin(); it != post.constEnd(); ++it ) {
       // We check for isEmpty because before 3.3 postAddress was just an
       // email@kde.org and that leaves protocol() field in the kurl class
       if ( (*it).protocol() == "mailto" || (*it).protocol().isEmpty() )
         return (*it).path();
     }
   }
-  return QString::null;
+  return QString();
 }
 
 void KMFolder::setMailingListEnabled( bool enabled )
@@ -609,6 +670,14 @@ void KMFolder::setMailingList( const MailingList& mlist )
   mStorage->writeConfig();
 }
 
+void KMFolder::setUseDefaultIdentity( bool useDefaultIdentity )
+{
+  mUseDefaultIdentity = useDefaultIdentity;
+  if ( mUseDefaultIdentity )
+    mIdentity = kmkernel->identityManager()->defaultIdentity().uoid();
+  kmkernel->slotRequestConfigSync();
+}
+
 void KMFolder::setIdentity( uint identity )
 {
   mIdentity = identity;
@@ -618,8 +687,7 @@ void KMFolder::setIdentity( uint identity )
 uint KMFolder::identity() const
 {
   // if we don't have one set ourselves, check our account
-  kdDebug() << "FOO: " << mIdentity << " :: " << mStorage << endl;
-  if ( !mIdentity && mStorage )
+  if ( mUseDefaultIdentity && mStorage )
     if ( KMAccount *act = mStorage->account() )
       return act->identityId();
   return mIdentity;
@@ -636,12 +704,13 @@ void KMFolder::setWhoField(const QString& aWhoField )
 
 void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
 {
-  if ( mUserWhoField == whoField )
+  if ( mUserWhoField == whoField && !whoField.isEmpty() )
     return;
+
   if ( whoField.isEmpty() )
   {
     // default setting
-    const KPIM::Identity & identity =
+    const KPIMIdentities::Identity & identity =
       kmkernel->identityManager()->identityForUoidOrDefault( mIdentity );
 
     if ( isSystemFolder() && folderType() != KMFolderTypeImap ) {
@@ -651,8 +720,8 @@ void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
         mWhoField = "From";
       if ( this == kmkernel->outboxFolder() ||
            this == kmkernel->sentFolder() ||
-           this == kmkernel->draftsFolder() ||
-           this == kmkernel->templatesFolder() )
+           this == kmkernel->templatesFolder() ||
+           this == kmkernel->draftsFolder() )
         mWhoField = "To";
     } else if ( identity.drafts() == idString() ||
                 identity.templates() == idString() ||
@@ -666,8 +735,7 @@ void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
     mWhoField = whoField;
   else {
     // this should not happen...
-    kdDebug(5006) << "Illegal setting " << whoField << " for userWhoField!"
-                  << endl;
+    kDebug() <<"Illegal setting" << whoField << "for userWhoField!";
     return; // don't use the value
   }
   mUserWhoField = whoField;
@@ -693,12 +761,12 @@ QString KMFolder::idString() const
   int pathLen = myPath.length() - folderNode->path().length();
   QString relativePath = myPath.right( pathLen );
   if (!relativePath.isEmpty())
-    relativePath = relativePath.right( relativePath.length() - 1 ) + "/";
+    relativePath = relativePath.right( relativePath.length() - 1 ) + '/';
   QString escapedName = name();
   /* Escape [ and ] as they are disallowed for kconfig sections and that is
      what the idString is primarily used for. */
-  escapedName.replace( "[", "%(" );
-  escapedName.replace( "]", "%)" );
+  escapedName.replace( '[', "%(" );
+  escapedName.replace( ']', "%)" );
   return relativePath + escapedName;
 }
 
@@ -808,12 +876,12 @@ int KMFolder::writeIndex( bool createEmptyIndex )
   return mStorage->writeIndex( createEmptyIndex );
 }
 
-void KMFolder::setStatus( int idx, KMMsgStatus status, bool toggle )
+void KMFolder::setStatus( int idx, const MessageStatus& status, bool toggle )
 {
   mStorage->setStatus( idx, status, toggle );
 }
 
-void KMFolder::setStatus( QValueList<int>& ids, KMMsgStatus status,
+void KMFolder::setStatus( QList<int>& ids, const MessageStatus& status,
                           bool toggle )
 {
   mStorage->setStatus( ids, status, toggle);
@@ -858,7 +926,7 @@ void KMFolder::setShortcut( const KShortcut &sc )
 
 bool KMFolder::isMoveable() const
 {
-  return !isSystemFolder();
+  return !isSystemFolder() && mStorage->isMoveable();
 }
 
 void KMFolder::slotContentsTypeChanged( KMail::FolderContentsType type )
@@ -876,5 +944,21 @@ void KMFolder::slotFolderSizeChanged()
   }
 }
 
+void KMFolder::slotIdentitiesChanged()
+{
+  uint defaultIdentity = kmkernel->identityManager()->defaultIdentity().uoid();
+
+  // The default identity may have changed, therefore set it again
+  // if necessary
+  if ( mUseDefaultIdentity )
+    mIdentity = defaultIdentity;
+
+  // Fall back to the default identity if the one used currently is invalid
+  if ( kmkernel->identityManager()->identityForUoid( mIdentity ).isNull() ) {
+    mIdentity = defaultIdentity;
+    mUseDefaultIdentity = true;
+  }
+}
+
 
 #include "kmfolder.moc"
