@@ -1,13 +1,12 @@
 // kmmsgbase.cpp
 
+#include "kmmsgbase.h"
+
 #include <config-kmail.h>
 
-#include "kmmsgbase.h"
 #include "globalsettings.h"
-
 #include "kmfolderindex.h"
 #include "kmfolder.h"
-#include "kmheaders.h"
 #include "kmmsgdict.h"
 #include "kmmessagetag.h"
 #include "messageproperty.h"
@@ -34,6 +33,8 @@ using KMail::MessageProperty;
 #include <stdlib.h>
 #include <unistd.h>
 
+#include <kmime/kmime_dateformatter.h> // kdepimlibs
+
 #ifdef HAVE_BYTESWAP_H
 #include <byteswap.h>
 #endif
@@ -86,7 +87,8 @@ KMMsgBase::KMMsgBase(KMFolder* aParentFolder)
     mDirty( false ),
     mEnableUndo( false ),
     mStatus(),
-    mTagList( 0 )
+    mTagList( 0 ),
+    mStringPartCacheBuilt( false )
 {
 }
 
@@ -104,7 +106,8 @@ KMMsgBase::KMMsgBase(KMFolder* aParentFolder, off_t off, short len)
     mDirty( false ),
     mEnableUndo( false ),
     mStatus(),
-    mTagList( 0 )
+    mTagList( 0 ),
+    mStringPartCacheBuilt( false )
 {
 }
 
@@ -114,6 +117,7 @@ KMMsgBase::KMMsgBase( const KMMsgBase& other )
 #ifdef KMAIL_SQLITE_INDEX
  , mData( 0 )
 #endif
+ , mStringPartCacheBuilt( false )
 {
     assign( &other );
 }
@@ -239,7 +243,7 @@ void KMMsgBase::setStatus(const char* aStatusStr, const char* aXStatusStr)
 
 void KMMsgBase::setEncryptionState( const KMMsgEncryptionState /*status*/, int idx )
 {
-    //kDebug(5006) <<"***setEncryptionState1(" << status <<" )";
+    //kDebug() <<"***setEncryptionState1(" << status <<" )";
     mDirty = true;
     if (storage())
         storage()->headerOfMsgChanged(this, idx);
@@ -247,7 +251,7 @@ void KMMsgBase::setEncryptionState( const KMMsgEncryptionState /*status*/, int i
 
 void KMMsgBase::setEncryptionStateChar( QChar status, int idx )
 {
-    //kDebug(5006) <<"***setEncryptionState2(" << (status.isNull() ? '?' : status.toLatin1()) <<" )";
+    //kDebug() <<"***setEncryptionState2(" << (status.isNull() ? '?' : status.toLatin1()) <<" )";
 
     if( status.toLatin1() == (char)KMMsgEncryptionStateUnknown )
         setEncryptionState( KMMsgEncryptionStateUnknown, idx );
@@ -264,7 +268,7 @@ void KMMsgBase::setEncryptionStateChar( QChar status, int idx )
 
 void KMMsgBase::setSignatureState( const KMMsgSignatureState /*status*/, int idx )
 {
-    //kDebug(5006) <<"***setSignatureState1(" << status <<" )";
+    //kDebug() <<"***setSignatureState1(" << status <<" )";
     mDirty = true;
     if (storage())
          storage()->headerOfMsgChanged(this, idx);
@@ -278,7 +282,7 @@ void KMMsgBase::setMDNSentState( KMMsgMDNSentState, int idx ) {
 
 void KMMsgBase::setSignatureStateChar( QChar status, int idx )
 {
-    //kDebug(5006) <<"***setSignatureState2(" << (status.isNull() ? '?' : status.toLatin1()) <<" )";
+    //kDebug() <<"***setSignatureState2(" << (status.isNull() ? '?' : status.toLatin1()) <<" )";
 
     if( status.toLatin1() == (char)KMMsgSignatureStateUnknown )
         setSignatureState( KMMsgSignatureStateUnknown, idx );
@@ -308,7 +312,65 @@ const MessageStatus& KMMsgBase::messageStatus() const
 //-----------------------------------------------------------------------------
 void KMMsgBase::setDate(const QByteArray& aDateStr)
 {
-  setDate( KDateTime::fromString( aDateStr, KDateTime::RFCDate ).toTime_t() );
+  KDateTime extractedDate = KDateTime::fromString( aDateStr, KDateTime::RFCDate );
+  if ( extractedDate.isValid() || aDateStr.isEmpty() )
+  {
+    setDate( extractedDate.toTime_t() );
+    return;
+  }
+
+  // KDateTime doesn't handle slightly non-standard formats.
+  // Let's try to do better.
+
+  QString dateString = aDateStr.trimmed();
+
+  // Look for a very common non-standard RFC format
+
+  // Thu, 17 Jul 2008 14:03:39 +0200 (CEST)
+  // Wed, 9 Jul 2008 18:22:45 +0200 (MET DST)
+  // Thu, 17 Jul 2008 07:50:24 -0400 (EDT)
+  // Wed, 16 Jul 2008 16:04:37 -0700 (PDT)
+  // Thu, 17 Jul 2008 04:21:20 +0000 (UTC)
+  // Wed, 09 Jan 2008 18:50:36 +0900 (JST)
+  // Wed, 9 Jan 2008 03:17:37 +0000 (GMT)
+  // Mon, 23 Jun 2008 00:40:38 +0600 (YEKST)
+
+  if ( aDateStr[ aDateStr.length() - 1 ] == ')' )
+  {
+    // Might be RFC format followed by (TIMEZONE). Very common.
+    QRegExp regularExpression("^(.+)\\s+\\([A-Z]{3,5}(\\s[A-Z]{3})?\\)$");
+    if ( dateString.indexOf(regularExpression) == 0 )
+    {
+      QStringList parts = regularExpression.capturedTexts();
+
+      Q_ASSERT( parts.size() >= 2 ); // Must be at least 2 (as per Qt docs)
+
+      extractedDate = KDateTime::fromString( parts[1], KDateTime::RFCDate );
+      if ( extractedDate.isValid() )
+      {
+        setDate( extractedDate.toTime_t() );
+        return;
+      }
+    }
+  }
+
+  // Something that I've seen around and we don't handle yet is
+
+  // Sat, 21 Jun 2008 8:58:23 -0400
+
+  // which doesn't seem to have the leading 0 digit in front of the hour...
+  // Until now I've seen only it in spam stuff so let's ignore it (it's even good to catch spam).
+
+  // Another set of horrors I've seen is
+
+  // Sun, 13 Apr 2008 14:05:54 +0100 (GMT+01:00)
+  // Sun, 15 Feb 2009 22:48:51 -0800 (Pacific Standard Time)
+  // Fri, 13 Mar 2009 07:33:20 -0600 (Mountain Daylight Time)
+
+  // We again ignore it as it's very rare.
+
+  kDebug() << "Unrecognized date format [" << aDateStr.data() << "]: fix this function if you think it's correct :)";
+  setDate( extractedDate.toTime_t() );
 }
 
 
@@ -324,12 +386,12 @@ QString KMMsgBase::dateStr(void) const
 QString KMMsgBase::skipKeyword(const QString& aStr, QChar sepChar,
 			       bool* hasKeyword)
 {
-  unsigned int i = 0, maxChars = 3;
   QString str = aStr;
 
   while (str[0] == ' ') str.remove(0,1);
   if (hasKeyword) *hasKeyword=false;
-
+  
+  unsigned int i = 0, maxChars = 3;
   unsigned int strLength(str.length());
   for (i=0; i < strLength && i < maxChars; i++)
   {
@@ -383,13 +445,14 @@ QStringList KMMsgBase::supportedEncodings(bool usAscii)
   QStringList encodings;
   QMap<QString,bool> mimeNames;
   for (QStringList::Iterator it = encodingNames.begin();
-    it != encodingNames.end(); it++)
+    it != encodingNames.end(); ++it)
   {
-    QTextCodec *codec = KGlobal::charsets()->codecForName(*it);
-    QString mimeName = (codec) ? QString(codec->name()).toLower() : (*it);
-    if (!mimeNames.contains(mimeName) )
+    bool ok;
+    QTextCodec *codec = KGlobal::charsets()->codecForName( *it, ok );
+    QString mimeName = codec ? QString( codec->name() ).toLower() : *it;
+    if ( ok && !mimeNames.contains( mimeName ) )
     {
-      encodings.append( KGlobal::charsets()->descriptionForEncoding(*it) );
+      encodings.append( KGlobal::charsets()->descriptionForEncoding( *it ) );
       mimeNames.insert( mimeName, true );
     }
   }
@@ -404,7 +467,7 @@ QString KMMsgBase::fixEncoding( const QString &encoding )
 {
   QString returnEncoding = encoding;
   // According to http://www.iana.org/assignments/character-sets, uppercase is
-  // prefered in MIME headers
+  // preferred in MIME headers
   if ( returnEncoding.toUpper().contains( "ISO " ) ) {
     returnEncoding = returnEncoding.toUpper();
     returnEncoding.replace( "ISO ", "ISO-" );
@@ -598,7 +661,7 @@ QByteArray KMMsgBase::encodeRFC2231String( const QString& _str,
 
   QByteArray result = cset + "''";
   for ( l = latin.data(); *l; ++l ) {
-    bool needsQuoting = ( *l & 0x80 );
+    bool needsQuoting = ( *l & 0x80 ) || ( *l == '%' );
     if( !needsQuoting ) {
       int len = especials.length();
       for ( int i = 0; i < len; i++ )
@@ -738,8 +801,8 @@ QByteArray KMMsgBase::autoDetectCharset(const QByteArray &_encoding, const QStri
        charsets.prepend(currentCharset);
     }
 
-    QStringList::ConstIterator it = charsets.begin();
-    for (; it != charsets.end(); ++it)
+    QStringList::ConstIterator it = charsets.constBegin();
+    for (; it != charsets.constEnd(); ++it)
     {
        QByteArray encoding = (*it).toLatin1();
        if (encoding == "locale")
@@ -759,7 +822,7 @@ QByteArray KMMsgBase::autoDetectCharset(const QByteArray &_encoding, const QStri
        {
          const QTextCodec *codec = KMMsgBase::codecForName(encoding);
          if (!codec) {
-           kDebug(5006) <<"Auto-Charset: Something is wrong and I can not get a codec. [" << encoding <<"]";
+           kDebug() <<"Auto-Charset: Something is wrong and I can not get a codec. [" << encoding <<"]";
          } else {
            if (codec->canEncode(text))
               return encoding;
@@ -816,7 +879,7 @@ namespace {
   template < typename T > void copy_from_stream( T & x ) {
     if( g_chunk_offset + int(sizeof(T)) > g_chunk_length ) {
       g_chunk_offset = g_chunk_length;
-      kDebug( 5006 ) <<"This should never happen..";
+      kDebug() <<"This should never happen..";
       x = 0;
     } else {
       // the memcpy is optimized out by the compiler for the values
@@ -830,7 +893,14 @@ namespace {
 //-----------------------------------------------------------------------------
 QString KMMsgBase::getStringPart(MsgPartType t) const
 {
-  QString ret;
+  if ( !mStringPartCacheBuilt )
+    fillStringPartCache();
+  return mCachedStringParts[t];
+}
+
+//-----------------------------------------------------------------------------
+void KMMsgBase::fillStringPartCache() const
+{
 retry:
 #ifdef KMAIL_SQLITE_INDEX
   bool swapByteOrder = false;
@@ -846,13 +916,13 @@ retry:
     if ( mIndexOffset > (off_t)storage()->indexStreamLength() ) {
       // This message has not been indexed yet, data would lie
       // outside the index data structures so do not touch it.
-      return QString();
+      return;
     }
     g_chunk = storage()->indexStreamBasePtr() + mIndexOffset;
     g_chunk_length = mIndexLength;
   } else {
     if(!storage()->mIndexStream)
-      return ret;
+      return;
     if (g_chunk_length < mIndexLength)
       g_chunk = (uchar *)realloc(g_chunk, g_chunk_length = mIndexLength);
     off_t first_off = KDE_ftell(storage()->mIndexStream);
@@ -875,7 +945,7 @@ retry:
     }
     type = (MsgPartType) tmp;
     if( g_chunk_offset + len > mIndexLength ) {
-      kDebug() << "This should never happen..";
+      kWarning() << "This should never happen..";
 #ifndef KMAIL_SQLITE_INDEX
       if( using_mmap ) {
         g_chunk_length = 0;
@@ -883,15 +953,29 @@ retry:
       }
 #endif
       if ( !storage()->recreateIndex() )
-        return QString();
+        return;
       goto retry;
     }
-    if ( type == t ) {
+
+    // Only try to create strings if the part is really a string part, see declaration of
+    // MsgPartType
+    if ( len && ( ( type >= 1 && type <= 6 ) || type == 11 || type == 14 || type == 15 || type == 19 ) ) {
+
       // This works because the QString constructor does a memcpy.
       // Otherwise we would need to be concerned about the alignment.
-      if ( len )
-        ret = QString((QChar *)(g_chunk + g_chunk_offset), len/2);
-      break;
+      mCachedStringParts[type] = QString((QChar *)(g_chunk + g_chunk_offset), len/2);
+
+      // Normally we need to swap the byte order because the QStrings are written
+      // in the style of Qt2 (MSB -> network ordered).
+      // QStrings in Qt3 expect host ordering.
+      // On e.g. Intel host ordering is LSB, on e.g. Sparc it is MSB.
+
+#     if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
+      // Byte order is little endian (swap is true)
+      swapEndian( mCachedStringParts[type] );
+#     else
+      // Byte order is big endian (swap is false)
+#     endif
     }
   } //for
 #ifndef KMAIL_SQLITE_INDEX
@@ -900,19 +984,8 @@ retry:
       g_chunk = 0;
   }
 #endif
-  // Normally we need to swap the byte order because the QStrings are written
-  // in the style of Qt2 (MSB -> network ordered).
-  // QStrings in Qt3 expect host ordering.
-  // On e.g. Intel host ordering is LSB, on e.g. Sparc it is MSB.
 
-# if Q_BYTE_ORDER == Q_LITTLE_ENDIAN
-  // Byte order is little endian (swap is true)
-  swapEndian(ret);
-# else
-  // Byte order is big endian (swap is false)
-# endif
-
-  return ret;
+  mStringPartCacheBuilt = true;
 }
 
 //-----------------------------------------------------------------------------
@@ -963,7 +1036,7 @@ retry:
     MsgPartType type = (MsgPartType) tmp;
 
     if (g_chunk_offset + len > mIndexLength) {
-      kDebug(5006) <<"This should never happen..";
+      kDebug() <<"This should never happen..";
 #ifndef KMAIL_SQLITE_INDEX
       if(using_mmap) {
         g_chunk_length = 0;
@@ -1191,7 +1264,7 @@ QString KMMsgBase::replacePrefixes( const QString& str,
                       .arg( prefixRegExps.join(")|(?:") );
   QRegExp rx( bigRegExp, Qt::CaseInsensitive );
   if ( !rx.isValid() ) {
-    kWarning(5006) << "bigRegExp = \""
+    kWarning() << "bigRegExp = \""
                    << bigRegExp << "\"\n"
                    << "prefix regexp is invalid!";
     // try good ole Re/Fwd:
@@ -1225,7 +1298,16 @@ void KMMsgBase::setTagList( const KMMessageTagList &aTagList )
     *mTagList = aTagList;
   }
   mTagList->prioritySort();
+
   mDirty = true;
+
+  if (storage())
+  {
+    int idx = storage()->find( this );
+
+    if( mParent )
+      mParent->msgTagListChanged( idx );
+  }
 }
 
 //-----------------------------------------------------------------------------
