@@ -42,6 +42,7 @@
 #include <QPaintEvent>
 #include <QPixmap>
 #include <QPushButton>
+#include <QSpinBox>
 #include <QStringList>
 
 #include <KColorDialog>
@@ -92,18 +93,18 @@ ThemeColumnPropertiesDialog::ThemeColumnPropertiesDialog( QWidget * parent, Them
   mNameEdit->setToolTip( i18n( "The label that will be displayed in the column header." ) );
   g->addWidget( mNameEdit, 0, 1 );
 
-  l = new QLabel( i18n( "Header Click Sorts Messages" ), base );
+  l = new QLabel( i18n( "Header click sorts messages:" ), base );
   g->addWidget( l, 1, 0 );
 
   mMessageSortingCombo = new KComboBox( base );
-  mMessageSortingCombo->setToolTip( i18n( "The sorting order that cliking on this column header will switch to." ) );
+  mMessageSortingCombo->setToolTip( i18n( "The sorting order that clicking on this column header will switch to." ) );
   g->addWidget( mMessageSortingCombo, 1, 1 );
 
-  mVisibleByDefaultCheck = new QCheckBox( i18n( "Visible by Default" ), base );
+  mVisibleByDefaultCheck = new QCheckBox( i18n( "Visible by default" ), base );
   mVisibleByDefaultCheck->setToolTip( i18n( "Check this if this column should be visible when the theme is selected." ) );
   g->addWidget( mVisibleByDefaultCheck, 2, 1 );
 
-  mIsSenderOrReceiverCheck = new QCheckBox( i18n( "Contains \"Sender or Receiver\" Field" ), base );
+  mIsSenderOrReceiverCheck = new QCheckBox( i18n( "Contains \"Sender or Receiver\" field" ), base );
   mIsSenderOrReceiverCheck->setToolTip( i18n( "Check this if this column label should be updated depending on the folder \"inbound\"/\"outbound\" type." ) );
   g->addWidget( mIsSenderOrReceiverCheck, 3, 1 );
 
@@ -188,8 +189,8 @@ void ThemeContentItemSourceLabel::startDrag()
   drag->exec( Qt::CopyAction, Qt::CopyAction );
 }
 
-ThemePreviewDelegate::ThemePreviewDelegate( QAbstractItemView * parent, QPaintDevice * paintDevice )
-  : ThemeDelegate( parent, paintDevice )
+ThemePreviewDelegate::ThemePreviewDelegate( QAbstractItemView * parent )
+  : ThemeDelegate( parent )
 {
   mRowMapper = new ModelInvariantRowMapper();
 
@@ -262,7 +263,7 @@ ThemePreviewWidget::ThemePreviewWidget( QWidget * parent )
   mSelectedThemeColumn = 0;
   mFirstShow = true;
 
-  mDelegate = new ThemePreviewDelegate( this, viewport() );
+  mDelegate = new ThemePreviewDelegate( this );
   setItemDelegate( mDelegate );
   setRootIsDecorated( false );
   viewport()->setAcceptDrops( true );
@@ -402,6 +403,9 @@ void ThemePreviewWidget::setTheme( Theme * theme )
   bool themeChanged = theme != mTheme;
 
   mSelectedThemeContentItem = 0;
+  mThemeSelectedContentItemRect = QRect();
+  mDropIndicatorPoint1 = QPoint();
+  mDropIndicatorPoint2 = QPoint();
   mTheme = theme;
   mDelegate->setTheme( theme );
   mGroupHeaderSampleItem->setExpanded( true );
@@ -412,7 +416,7 @@ void ThemePreviewWidget::setTheme( Theme * theme )
 
   QStringList headerLabels;
 
-  for( QList< Theme::Column * >::ConstIterator it = columns.begin(); it != columns.end(); ++it )
+  for( QList< Theme::Column * >::ConstIterator it = columns.constBegin(); it != columns.constEnd(); ++it )
   {
     QString label = ( *it )->label();
     if ( ( *it )->visibleByDefault() )
@@ -1196,7 +1200,7 @@ void ThemePreviewWidget::slotHeaderContextMenuRequested( const QPoint &pos )
 
   QAction * act;
 
-  act = menu.addAction( i18n( "Column Properties..." ) );
+  act = menu.addAction( i18n( "Column Properties" ) );
   connect( act, SIGNAL( triggered( bool ) ),
            SLOT( slotColumnProperties() ) );
 
@@ -1458,8 +1462,24 @@ ThemeEditor::ThemeEditor( QWidget *parent )
   mViewHeaderPolicyCombo = new KComboBox( tab );
   tabg->addWidget( mViewHeaderPolicyCombo, 0, 1 );
 
+  l = new QLabel( i18n( "Icon size:" ), tab );
+  tabg->addWidget( l, 1, 0 );
+
+  mIconSizeSpinBox = new QSpinBox( tab );
+  mIconSizeSpinBox->setMinimum( 8 );
+  mIconSizeSpinBox->setMaximum( 64 );
+  mIconSizeSpinBox->setSuffix( ' ' + i18nc( "suffix in a spinbox", "Pixels" ) );
+
+  QObject::connect(
+      mIconSizeSpinBox, SIGNAL( valueChanged( int ) ),
+      this, SLOT( slotIconSizeSpinBoxValueChanged( int ) )
+    );
+
+  tabg->addWidget( mIconSizeSpinBox, 1, 1 );
+
+
   tabg->setColumnStretch( 1, 1 );
-  tabg->setRowStretch( 1, 1 );
+  tabg->setRowStretch( 2, 1 );
 
 }
 
@@ -1487,6 +1507,7 @@ void ThemeEditor::editTheme( Theme *set )
   fillViewHeaderPolicyCombo();
   ComboBoxUtils::setIntegerOptionComboValue( mViewHeaderPolicyCombo, (int)mCurrentTheme->viewHeaderPolicy() );
 
+  mIconSizeSpinBox->setValue( set->iconSize() );
 }
 
 void ThemeEditor::commit()
@@ -1500,6 +1521,7 @@ void ThemeEditor::commit()
   mCurrentTheme->setViewHeaderPolicy(
       (Theme::ViewHeaderPolicy)ComboBoxUtils::getIntegerOptionComboValue( mViewHeaderPolicyCombo, 0 )
     );
+  mCurrentTheme->setIconSize( mIconSizeSpinBox->value() );
   // other settings are already committed to this theme
 }
 
@@ -1519,6 +1541,15 @@ void ThemeEditor::slotNameEditTextEdited( const QString &newName )
   emit themeNameChanged();
 }
 
+void ThemeEditor::slotIconSizeSpinBoxValueChanged( int val )
+{
+  if( !mCurrentTheme )
+    return;
+  mCurrentTheme->setIconSize( val );
+
+  mPreviewWidget->setTheme( mCurrentTheme ); // will trigger a cache reset and a view update
+}
+
 } // namespace Core
 
 } // namespace MessageListView
