@@ -1,8 +1,6 @@
 /*
-    main.cpp
-
     KNode, the KDE newsreader
-    Copyright (c) 1999-2001 the KNode authors.
+    Copyright (c) 1999-2005 the KNode authors.
     See file AUTHORS for details
 
     This program is free software; you can redistribute it and/or modify
@@ -11,11 +9,12 @@
     (at your option) any later version.
     You should have received a copy of the GNU General Public License
     along with this program; if not, write to the Free Software Foundation,
-    Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, US
+    Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 
 #include <kaboutdata.h>
 #include <kcmdlineargs.h>
+#include <kglobal.h>
 #include <klocale.h>
 
 #include "knapplication.h"
@@ -30,14 +29,14 @@ int main(int argc, char* argv[])
   AboutData aboutData;
 
   KCmdLineArgs::init( argc, argv, &aboutData );
-  KCmdLineArgs::addCmdLineOptions( knode_options );
+  KCmdLineArgs::addCmdLineOptions( knode_options() );
   KUniqueApplication::addCmdLineOptions();
 
   if (!KNApplication::start())
     return 0;
 
   KNApplication app;
-  KGlobal::locale()->insertCatalogue("libkdenetwork");
+  KGlobal::locale()->insertCatalog("libkdepim");
+  KGlobal::locale()->insertCatalog("libkpgp");
   return app.exec();
 }
-
