@@ -162,7 +162,7 @@ bool SDSummaryWidget::initHolidays()
     if ( mHolidays ) {
       delete mHolidays;
     }
-    mHolidays = new LibKHolidays::KHolidays( location );
+    mHolidays = new KHolidays::HolidayRegion( location );
     return true;
   }
   return false;
@@ -274,12 +274,12 @@ void SDSummaryWidget::updateView()
                                                   KCal::SortDirectionAscending );
     KCal::Event *ev;
     KCal::Event::List::ConstIterator it;
-    for ( it = events.begin(); it != events.end(); ++it ) {
+    for ( it = events.constBegin(); it != events.constEnd(); ++it ) {
       ev = *it;
       if ( !ev->categoriesStr().isEmpty() ) {
         QStringList::ConstIterator it2;
         QStringList c = ev->categories();
-        for ( it2 = c.begin(); it2 != c.end(); ++it2 ) {
+        for ( it2 = c.constBegin(); it2 != c.constEnd(); ++it2 ) {
 
           // Append Birthday Event?
           if ( mShowBirthdaysFromCal && ( ( *it2 ).toUpper() == i18n( "BIRTHDAY" ) ) ) {
@@ -359,15 +359,15 @@ void SDSummaryWidget::updateView()
       for ( dt=QDate::currentDate();
             dt<=QDate::currentDate().addDays( mDaysAhead - 1 );
             dt=dt.addDays(1) ) {
-        QList<LibKHolidays::KHoliday> holidays = mHolidays->getHolidays( dt );
-        QList<LibKHolidays::KHoliday>::ConstIterator it = holidays.begin();
-        for ( ; it != holidays.end(); ++it ) {
+        QList<KHolidays::Holiday> holidays = mHolidays->holidays( dt );
+        QList<KHolidays::Holiday>::ConstIterator it = holidays.constBegin();
+        for ( ; it != holidays.constEnd(); ++it ) {
           SDEntry entry;
           entry.type = IncidenceTypeEvent;
-          entry.category = ( (*it).Category == LibKHolidays::KHolidays::HOLIDAY )?
+          entry.category = ( (*it).dayType() == KHolidays::Holiday::NonWorkday )?
                            CategoryHoliday : CategoryOther;
           entry.date = dt;
-          entry.summary = (*it).text;
+          entry.summary = (*it).text();
           dateDiff( dt, entry.daysTo, entry.yearsOld );
           entry.yearsOld = -1; //ignore age of holidays
           entry.span = 1;
@@ -442,9 +442,9 @@ void SDSummaryWidget::updateView()
       QDate sD = QDate( year, (*addrIt).date.month(), (*addrIt).date.day() );
 
       if ( (*addrIt).daysTo == 0 ) {
-        datestr = i18n( "Today" );
+        datestr = i18nc( "the special day is today", "Today" );
       } else if ( (*addrIt).daysTo == 1 ) {
-        datestr = i18n( "Tomorrow" );
+        datestr = i18nc( "the special day is tomorrow", "Tomorrow" );
       } else {
         datestr = KGlobal::locale()->formatDate( sD, KLocale::FancyLongDate );
       }
@@ -507,6 +507,7 @@ void SDSummaryWidget::updateView()
         urlLabel->setUrl( (*addrIt).addressee.uid() );
         urlLabel->setText( (*addrIt).addressee.realName() );
         urlLabel->setTextFormat( Qt::RichText );
+        urlLabel->setWordWrap( true );
         mLayout->addWidget( urlLabel, counter, 4 );
         mLabels.append( urlLabel );
 
@@ -547,12 +548,12 @@ void SDSummaryWidget::updateView()
                "No special dates pending within the next %1 days",
                mDaysAhead ), this );
     label->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
-    mLayout->addWidget( label, 0, 2 );
+    mLayout->addWidget( label, 0, 0 );
     mLabels.append( label );
   }
 
-  QList<QLabel*>::iterator lit;
-  for ( lit = mLabels.begin(); lit != mLabels.end(); ++lit ) {
+  QList<QLabel*>::ConstIterator lit;
+  for ( lit = mLabels.constBegin(); lit != mLabels.constEnd(); ++lit ) {
     (*lit)->show();
   }
 
@@ -668,7 +669,7 @@ bool SDSummaryWidget::check( KCal::ResourceCalendar *cal, const QDate &date,
 
   KCal::Event::List events = cal->rawEventsForDate( date );
   KCal::Event::List::ConstIterator it;
-  for ( it = events.begin(); it != events.end(); ++it ) {
+  for ( it = events.constBegin(); it != events.constEnd(); ++it ) {
     if ( (*it)->summary() == summary ) {
       return true;
     }
