@@ -30,85 +30,65 @@
 */
 
 #include "calendarview.h"
-
 #ifndef KORG_NOPRINTER
 #include "calprinter.h"
 #endif
-#include "koeventeditor.h"
-#include "kotodoeditor.h"
-#include "kojournaleditor.h"
-#include "koprefs.h"
-#include "koeventviewerdialog.h"
-#include "publishdialog.h"
-#include "koglobals.h"
-#include "koviewmanager.h"
-#include "kodialogmanager.h"
-#include "statusdialog.h"
-#include "datenavigatorcontainer.h"
-#include "datenavigator.h"
-#include "resourceview.h"
-#include "navigatorbar.h"
-#include "history.h"
-#include "kogroupware.h"
-#include "freebusymanager.h"
 #include "datechecker.h"
-#include "komessagebox.h"
+#include "datenavigator.h"
+#include "datenavigatorcontainer.h"
 #include "exportwebdialog.h"
+#include "freebusymanager.h"
+#include "history.h"
 #include "kocorehelper.h"
-#include "incidencechanger.h"
-#include "kholidays.h"
-#include "mailscheduler.h"
+#include "kodialogmanager.h"
+#include "koeventeditor.h"
+#include "koeventviewer.h"
+#include "koeventviewerdialog.h"
+#include "koglobals.h"
+#include "kogroupware.h"
+#include "koincidenceeditor.h"
+#include "kojournaleditor.h"
 #include "komailclient.h"
+#include "komessagebox.h"
+#include "koprefs.h"
+#include "kotodoeditor.h"
+#include "koviewmanager.h"
+#include "incidencechanger.h"
+#include "mailscheduler.h"
+#include "navigatorbar.h"
+#include "publishdialog.h"
 #include "views/agendaview/koagendaview.h"
-#include "views/todoview/kotodoview.h"
 #include "views/monthview/monthview.h"
 #include "views/multiagendaview/multiagendaview.h"
+#include "views/todoview/kotodoview.h"
 
-#include <kcal/vcaldrag.h>
-#include <kcal/icaldrag.h>
-#include <kcal/icalformat.h>
-#include <kcal/vcalformat.h>
-#include <kcal/scheduler.h>
-#include <kcal/calendarlocal.h>
-#include <kcal/journal.h>
-#include <kcal/calfilter.h>
-#include <kcal/attendee.h>
-#include <kcal/dndfactory.h>
-#include <kcal/freebusy.h>
-#include <kcal/filestorage.h>
-#include <kcal/calendarresources.h>
-#include <kcal/calendarnull.h>
-#include <kcal/htmlexportsettings.h>
+#include <KCal/CalendarLocal>
+#include <KCal/CalendarNull>
+#include <KCal/CalendarResources>
+#include <KCal/CalFilter>
+#include <KCal/DndFactory>
+#include <KCal/FileStorage>
+#include <KCal/FreeBusy>
+#include <KCal/ICalDrag>
+#include <KCal/ICalFormat>
+#include <KCal/VCalFormat>
+#include <kcal/htmlexportsettings.h> //krazy:exclude=camelcase. it's generated
 
-#include <kglobal.h>
-#include <kdebug.h>
-#include <kstandarddirs.h>
-#include <kfiledialog.h>
-#include <kmessagebox.h>
-#include <knotification.h>
-#include <kconfig.h>
-#include <krun.h>
-#include <kdirwatch.h>
-#include <kvbox.h>
+#include <KHolidays/Holidays>
+
+#include <KFileDialog>
+#include <KNotification>
+#include <KRun>
+#include <KVBox>
 
 #include <QApplication>
 #include <QClipboard>
-#include <QCursor>
-#include <QTimer>
-#include <QStackedWidget>
-#include <QList>
-#include <QFile>
-#include <QLayout>
 #include <QSplitter>
-#include <QByteArray>
-#include <QBoxLayout>
+#include <QStackedWidget>
 #include <QVBoxLayout>
 
-#include <stdlib.h>
-#include <assert.h>
-
 using namespace KOrg;
-using namespace LibKHolidays;
+using namespace KHolidays;
 
 CalendarView::CalendarView( QWidget *parent )
   : CalendarViewBase( parent ),
@@ -129,8 +109,8 @@ CalendarView::CalendarView( QWidget *parent )
   mDateNavigator = new DateNavigator( this );
   mDateChecker = new DateChecker( this );
 
-  QBoxLayout *topLayout = new QVBoxLayout( this );
-  topLayout->setMargin(0);
+  QVBoxLayout *topLayout = new QVBoxLayout( this );
+  topLayout->setMargin( 0 );
 
   // create the main layout frames.
   mPanner = new QSplitter( Qt::Horizontal, this );
@@ -139,12 +119,10 @@ CalendarView::CalendarView( QWidget *parent )
 
   mLeftSplitter = new QSplitter( Qt::Vertical, mPanner );
   mLeftSplitter->setObjectName( "CalendarView::LeftFrame" );
-//  mPanner->setResizeMode( mLeftSplitter, QSplitter::Stretch );
 
   mDateNavigatorContainer = new DateNavigatorContainer( mLeftSplitter );
   mDateNavigatorContainer->setObjectName( "CalendarView::DateNavigator" );
 
-//  mLeftSplitter->setResizeMode( mDateNavigatorContainer, QSplitter::Stretch );
   mTodoList = new KOTodoView( CalendarNull::self(), mLeftSplitter );
   mTodoList->setObjectName( "todolist" );
 
@@ -166,6 +144,13 @@ CalendarView::CalendarView( QWidget *parent )
   connect( mDateNavigator, SIGNAL(datesSelected(const KCal::DateList &)),
            mDateNavigatorContainer, SLOT(selectDates(const KCal::DateList &)) );
 
+  connect( mDateNavigatorContainer, SIGNAL(newEventSignal(const QDate &)),
+           SLOT(newEvent(const QDate &)) );
+  connect( mDateNavigatorContainer, SIGNAL(newTodoSignal(const QDate &)),
+           SLOT(newTodo(const QDate &)) );
+  connect( mDateNavigatorContainer, SIGNAL(newJournalSignal(const QDate &)),
+           SLOT(newJournal(const QDate &)) );
+
   connect( mNavigatorBar, SIGNAL(goPrevYear()),
            mDateNavigator, SLOT(selectPreviousYear()) );
   connect( mNavigatorBar, SIGNAL(goNextYear()),
@@ -240,7 +225,7 @@ CalendarView::CalendarView( QWidget *parent )
   mViewManager->connectView( mTodoList );
 
   KOGlobals::self()->
-      setHolidays( new KHolidays( KOPrefs::instance()->mHolidays ) );
+      setHolidays( new HolidayRegion( KOPrefs::instance()->mHolidays ) );
 
   connect( QApplication::clipboard(), SIGNAL(dataChanged()),
            SLOT(checkClipboard()) );
@@ -260,6 +245,7 @@ CalendarView::~CalendarView()
   delete mDialogManager;
   delete mViewManager;
   delete mEventViewer;
+  delete mHistory;
 }
 
 void CalendarView::setCalendar( Calendar *cal )
@@ -271,9 +257,6 @@ void CalendarView::setCalendar( Calendar *cal )
   connect( mHistory, SIGNAL(undone()), SLOT(updateView()) );
   connect( mHistory, SIGNAL(redone()), SLOT(updateView()) );
 
-  if ( mChanger ) {
-    delete mChanger;
-  }
   setIncidenceChanger( new IncidenceChanger( mCalendar, this ) );
 
   mCalendar->registerObserver( this );
@@ -285,6 +268,7 @@ void CalendarView::setCalendar( Calendar *cal )
 
 void CalendarView::setIncidenceChanger( IncidenceChangerBase *changer )
 {
+  delete mChanger;
   mChanger = changer;
   emit newIncidenceChanger( mChanger );
   connect( mChanger, SIGNAL(incidenceAdded(Incidence *)),
@@ -327,13 +311,10 @@ QDate CalendarView::activeDate( bool fallbackToToday )
 {
   KOrg::BaseView *curView = mViewManager->currentView();
   if ( curView ) {
-    // Only the agenda view has a selectionStart (at this time)
-    KOAgendaView *aView = mViewManager->agendaView();
-    if ( curView == aView && aView->selectionStart().isValid() ) {
-      if ( aView->selectionStart().isValid() ) {
-        return aView->selectionStart().date();
-      }
+    if ( curView->selectionStart().isValid() ) {
+      return curView->selectionStart().date();
     }
+
     // Try the view's selectedDates()
     if ( !curView->selectedDates().isEmpty() ) {
       if ( curView->selectedDates().first().isValid() ) {
@@ -345,10 +326,9 @@ QDate CalendarView::activeDate( bool fallbackToToday )
   // When all else fails, use the navigator start date, or today.
   if ( fallbackToToday ) {
     return QDate::currentDate();
-  } else {  
+  } else {
     return mDateNavigator->selectedDates().first();
   }
-
 }
 
 QDate CalendarView::startDate()
@@ -391,8 +371,8 @@ bool CalendarView::openCalendar( const QString &filename, bool merge )
     if ( cl ) {
       loadedSuccesfully = cl->load( filename );
     } else {
-      CalendarResources *cr = dynamic_cast<CalendarResources*>( mCalendar );
-      assert( cr ); // otherwise something is majorly wrong
+      // otherwise something is majorly wrong
+      Q_ASSERT( dynamic_cast<CalendarResources*>( mCalendar ) );
       // openCalendar called without merge and a filename, what should we do?
       return false;
     }
@@ -477,7 +457,7 @@ void CalendarView::readSettings()
 
   mEventViewer->readSettings( config );
   mViewManager->readSettings( config );
-  mTodoList->restoreLayout( config, QString( "Sidebar Todo View" ) );
+  mTodoList->restoreLayout( config, QString( "Sidebar Todo View" ), true );
 
   readFilterSettings( config );
 
@@ -620,7 +600,7 @@ void CalendarView::updateConfig( const QByteArray &receiver )
     return;
   }
 
-  KOGlobals::self()->setHolidays( new KHolidays( KOPrefs::instance()->mHolidays ) );
+  KOGlobals::self()->setHolidays( new HolidayRegion( KOPrefs::instance()->mHolidays ) );
 
   // Only set a new time zone if it changed. This prevents the window
   // from being modified on start
@@ -647,6 +627,11 @@ void CalendarView::updateConfig( const QByteArray &receiver )
       mCalendar->shiftTimes( mCalendar->viewTimeSpec(), newTimeSpec );
     }
   }
+
+  // config changed lets tell the date navigator the new modes
+  // if there weren't changed they are ignored
+  updateHighlightModes();
+
   emit configChanged();
 
   // force reload and handle agenda view type switch
@@ -809,8 +794,9 @@ void CalendarView::changeIncidenceDisplay( Incidence *incidence, int action )
 
 void CalendarView::updateView( const QDate &start, const QDate &end, const bool updateTodos = true )
 {
-  if ( updateTodos )
+  if ( updateTodos ) {
     mTodoList->updateView();
+  }
 
   mViewManager->updateView( start, end );
   mDateNavigatorContainer->updateView();
@@ -926,7 +912,8 @@ void CalendarView::edit_paste()
     if ( aView && endDT.isValid() && useEndTime ) {
       if ( ( pastedEvent->allDay() && aView->selectedIsAllDay() ) ||
            ( !pastedEvent->allDay() && !aView->selectedIsAllDay() ) ) {
-        pastedEvent->setDtEnd( KDateTime( endDT, KOPrefs::instance()->timeSpec() ) );
+        KDateTime kdt( endDT, KOPrefs::instance()->timeSpec() );
+        pastedEvent->setDtEnd( kdt.toTimeSpec( pastedIncidence->dtEnd().timeSpec() ) );
       }
     }
     mChanger->addIncidence( pastedEvent, this );
@@ -981,7 +968,7 @@ KOEventEditor *CalendarView::newEventEditor( const QDateTime &startDtParam,
   connectIncidenceEditor( eventEditor );
   eventEditor->setDates( startDt, endDt, allDay );
   mDialogManager->connectTypeAhead(
-    eventEditor, dynamic_cast<KOrg::AgendaView*>( viewManager()->currentView() ) );
+    eventEditor, dynamic_cast<KOEventView*>( viewManager()->currentView() ) );
   return eventEditor;
 }
 
@@ -1079,12 +1066,7 @@ void CalendarView::newJournal( const QString &text, const QDate &date )
     journalDate = activeDate();
   }
   journalEditor->setDate( journalDate );
-  if ( !text.isEmpty() ) {
-    journalEditor->setTexts( text );
-  } else {
-    journalEditor->setTexts( i18nc( "Journal for DATE", "Journal for %1",
-                                    journalDate.toString( Qt::LocaleDate ) ) );
-  }
+  journalEditor->setTexts( text );
   journalEditor->show();
 }
 
@@ -1264,12 +1246,85 @@ void CalendarView::toggleAlarm( Incidence *incidence )
   delete oldincidence;
 }
 
-void CalendarView::dissociateOccurrence( Incidence *incidence, const QDate &date )
+void CalendarView::toggleTodoCompleted( Incidence *incidence )
 {
   if ( !incidence || !mChanger ) {
-    kDebug() << "toggleAlarm() called without having a clicked item";
+    kDebug() << "called without having a clicked item";
     return;
   }
+  if ( incidence->type() != "Todo" ) {
+    kDebug() << "called for a non-Todo incidence";
+    return;
+  }
+
+  Todo *todo = static_cast<Todo *>( incidence );
+  Todo *oldtodo = todo->clone();
+  if ( !mChanger->beginChange( todo ) ) {
+    kDebug() << "Unable to lock todo";
+    delete oldtodo;
+    return;
+  }
+
+  if ( todo->isCompleted() ) {
+    todo->setPercentComplete( 0 );
+  } else {
+    todo->setCompleted( KDateTime::currentDateTime( KOPrefs::instance()->timeSpec() ) );
+  }
+
+  mChanger->changeIncidence( oldtodo, todo, KOGlobals::COMPLETION_MODIFIED );
+  mChanger->endChange( todo );
+  delete oldtodo;
+}
+
+void CalendarView::dissociateOccurrences( Incidence *incidence, const QDate &date )
+{
+
+  if ( !incidence || !mChanger ) {
+    kError() << "Called without having a clicked item";
+    return;
+  }
+
+  KDateTime thisDateTime( date, KOPrefs::instance()->timeSpec() );
+  bool isFirstOccurrence = !incidence->recurrence()->getPreviousDateTime( thisDateTime ).isValid();
+
+  int answer;
+  bool doOnlyThis = false;
+  bool doFuture   = false;
+
+  if ( isFirstOccurrence ) {
+    answer = KMessageBox::questionYesNo( this,
+                                         i18n( "Do you want to dissociate "
+                                               "the occurrence at %1 "
+                                               "from the recurrence?",
+                                                KGlobal::locale()->formatDate( date ) ),
+                                                i18n( "KOrganizer Confirmation" ),
+                                                KGuiItem( i18n( "&Dissociate" ) ),
+                                                KGuiItem( i18n( "&Cancel" ) ) );
+
+    doOnlyThis = ( answer == KMessageBox::Yes );
+  } else {
+    answer = KMessageBox::questionYesNoCancel( this,
+                                               i18n( "Do you want to dissociate "
+                                                     "the occurrence at %1 "
+                                                     "from the recurrence or also "
+                                                     "dissociate future ones?",
+                                                     KGlobal::locale()->formatDate( date ) ),
+                                               i18n( "KOrganizer Confirmation" ),
+                                               KGuiItem( i18n( "&Only Dissociate This One" ) ),
+                                               KGuiItem( i18n( "&Also Dissociate Future Ones" ) ) );
+
+    doOnlyThis = ( answer == KMessageBox::Yes );
+    doFuture   = ( answer == KMessageBox::No );
+  }
+
+  if ( doOnlyThis ) {
+    dissociateOccurrence( incidence, date );
+  } else if ( doFuture ) {
+    dissociateFutureOccurrence( incidence, date );
+  }
+}
+void CalendarView::dissociateOccurrence( Incidence *incidence, const QDate &date )
+{
   if ( !mChanger->beginChange( incidence ) ) {
     kDebug() << "Unable to lock incidence";
     return;
@@ -1295,16 +1350,13 @@ void CalendarView::dissociateOccurrence( Incidence *incidence, const QDate &date
 
 void CalendarView::dissociateFutureOccurrence( Incidence *incidence, const QDate &date )
 {
-  if ( !incidence || !mChanger ) {
-    kDebug() << "toggleAlarm() called without having a clicked item";
-    return;
-  }
   if ( !mChanger->beginChange( incidence ) ) {
     kDebug() << "Unable to lock incidence";
     return;
   }
   startMultiModify( i18n( "Dissociate future occurrences" ) );
   Incidence *oldincidence = incidence->clone();
+
   Incidence *newInc =
     mCalendar->dissociateOccurrence( incidence, date,
                                      KOPrefs::instance()->timeSpec(), false );
@@ -1334,7 +1386,7 @@ void CalendarView::schedule_publish( Incidence *incidence )
     return;
   }
 
-  PublishDialog *publishdlg = new PublishDialog();
+  QPointer<PublishDialog> publishdlg = new PublishDialog();
   if ( incidence->attendeeCount() > 0 ) {
     Attendee::List attendees = incidence->attendees();
     Attendee::List::ConstIterator it;
@@ -1348,7 +1400,7 @@ void CalendarView::schedule_publish( Incidence *incidence )
     inc->clearAttendees();
 
     // Send the mail
-    KCal::MailScheduler scheduler( mCalendar );
+    MailScheduler scheduler( mCalendar );
     if ( scheduler.publish( incidence, publishdlg->addresses() ) ) {
       KMessageBox::information( this, i18n( "The item information was successfully sent." ),
                                 i18n( "Publishing" ), "IncidencePublishSuccess" );
@@ -1405,9 +1457,9 @@ void CalendarView::schedule_forward( Incidence *incidence )
     return;
   }
 
-  PublishDialog publishdlg;
-  if ( publishdlg.exec() == QDialog::Accepted ) {
-    QString recipients = publishdlg.addresses();
+  QPointer<PublishDialog> publishdlg = new PublishDialog;
+  if ( publishdlg->exec() == QDialog::Accepted ) {
+    QString recipients = publishdlg->addresses();
     if ( incidence->organizer().isEmpty() ) {
       incidence->setOrganizer( Person( KOPrefs::instance()->fullName(),
                                        KOPrefs::instance()->email() ) );
@@ -1423,6 +1475,7 @@ void CalendarView::schedule_forward( Incidence *incidence )
       KMessageBox::error( this, i18n( "Unable to forward the item '%1'", incidence->summary() ) );
     }
   }
+  delete publishdlg;
 }
 
 void CalendarView::mailFreeBusy( int daysToPublish )
@@ -1434,10 +1487,10 @@ void CalendarView::mailFreeBusy( int daysToPublish )
   freebusy->setOrganizer( Person( KOPrefs::instance()->fullName(),
                                   KOPrefs::instance()->email() ) );
 
-  PublishDialog *publishdlg = new PublishDialog();
+  QPointer<PublishDialog> publishdlg = new PublishDialog();
   if ( publishdlg->exec() == QDialog::Accepted ) {
     // Send the mail
-    KCal::MailScheduler scheduler( mCalendar );
+    MailScheduler scheduler( mCalendar );
     if ( scheduler.publish( freebusy, publishdlg->addresses() ) ) {
       KMessageBox::information(
         this,
@@ -1478,7 +1531,7 @@ void CalendarView::schedule( iTIPMethod method, Incidence *incidence )
   inc->clearAttendees();
 
   // Send the mail
-  KCal::MailScheduler scheduler( mCalendar );
+  MailScheduler scheduler( mCalendar );
   if ( scheduler.performTransaction( incidence, method ) ) {
     KMessageBox::information( this,
                               i18n( "The groupware message for item '%1' "
@@ -1546,11 +1599,30 @@ void CalendarView::print()
 #endif
 }
 
+void CalendarView::printPreview()
+{
+#ifndef KORG_NOPRINTER
+  createPrinter();
+
+  KOrg::BaseView *currentView = mViewManager->currentView();
+
+  CalPrinter::PrintType printType = CalPrinter::Month;
+
+  if ( currentView ) {
+    printType = currentView->printType();
+  }
+
+  DateList tmpDateList = mDateNavigator->selectedDates();
+  mCalPrinter->print( printType, tmpDateList.first(), tmpDateList.last(),
+                      Incidence::List(), true );
+#endif
+}
+
 void CalendarView::exportWeb()
 {
   // FIXME: Get rid of the settings object. When can I delete it???
   HTMLExportSettings *settings = new HTMLExportSettings( "KOrganizer" );
-  // Manually read in the config, because parametrized kconfigxt objects don't
+  // Manually read in the config, because parameterized kconfigxt objects don't
   // seem to load the config theirselves
   if ( settings ) {
     settings->readConfig();
@@ -1564,23 +1636,42 @@ void CalendarView::exportWeb()
 void CalendarView::exportICalendar()
 {
   QString filename = KFileDialog::getSaveFileName( KUrl( "icalout.ics" ),
-                                                   i18n( "*.ics|ICalendars" ), this );
+                                                   i18n( "*.ics|iCalendars" ), this );
 
-  if ( !filename.isEmpty() )
-  {
+  if ( !filename.isEmpty() ) {
     // Force correct extension
     if ( filename.right( 4 ) != ".ics" ) {
       filename += ".ics";
     }
 
-    FileStorage storage( mCalendar, filename, new ICalFormat );
-    storage.save();
+    if ( QFile( filename ).exists() ) {
+      if ( KMessageBox::No == KMessageBox::warningYesNo(
+             this,
+             i18n( "Do you want to overwrite %1?", filename ) ) ) {
+        return;
+      }
+    }
+    ICalFormat *format = new ICalFormat;
+    FileStorage storage( mCalendar, filename, format );
+    if ( !storage.save() ) {
+      QString errmess;
+      if ( format->exception() ) {
+        errmess = format->exception()->message();
+      } else {
+        errmess = i18nc( "save failure cause unknown", "Reason unknown" );
+      }
+      KMessageBox::error( this,
+                          i18nc( "@info",
+                                 "Cannot write iCalendar file %1. %2",
+                                 filename, errmess ) );
+
+    }
   }
 }
 
 void CalendarView::exportVCalendar()
 {
-  if ( mCalendar->journals().count() > 0 ) {
+  if ( !mCalendar->journals().isEmpty() ) {
     int result = KMessageBox::warningContinueCancel(
       this,
       i18n( "The journal entries can not be exported to a vCalendar file." ),
@@ -1596,17 +1687,32 @@ void CalendarView::exportVCalendar()
 
   QString filename = KFileDialog::getSaveFileName( KUrl( "vcalout.vcs" ),
                                                    i18n( "*.vcs|vCalendars" ), this );
-  if ( !filename.isEmpty() )
-  {
-
-    // TODO: I don't like forcing extensions:
+  if ( !filename.isEmpty() ) {
     // Force correct extension
     if ( filename.right( 4 ) != ".vcs" ) {
       filename += ".vcs";
     }
-
-    FileStorage storage( mCalendar, filename, new VCalFormat );
-    storage.save();
+    if ( QFile( filename ).exists() ) {
+      if ( KMessageBox::No == KMessageBox::warningYesNo(
+             this,
+             i18n( "Do you want to overwrite %1?", filename ) ) ) {
+        return;
+      }
+    }
+    VCalFormat *format = new VCalFormat;
+    FileStorage storage( mCalendar, filename, format );
+    if ( !storage.save() ) {
+      QString errmess;
+      if ( format->exception() ) {
+        errmess = format->exception()->message();
+      } else {
+        errmess = i18nc( "save failure cause unknown", "Reason unknown" );
+      }
+      KMessageBox::error( this,
+                          i18nc( "@info",
+                                 "Cannot write vCalendar file %1. %2",
+                                 filename, errmess ) );
+    }
   }
 }
 
@@ -1741,7 +1847,8 @@ void CalendarView::filterActivated( int filterNo )
   emit filterChanged();
 }
 
-bool CalendarView::isFiltered() const {
+bool CalendarView::isFiltered() const
+{
   return mCurrentFilter != 0;
 }
 
@@ -2145,20 +2252,33 @@ void CalendarView::deleteIncidence( Incidence *incidence, bool force )
         KDateTime itemDateTime( itemDate, KOPrefs::instance()->timeSpec() );
         bool isFirst = !incidence->recurrence()->getPreviousDateTime( itemDateTime ).isValid();
         bool isLast  = !incidence->recurrence()->getNextDateTime( itemDateTime ).isValid();
-        KGuiItem itemFuture( i18n( "Delete &Future" ) );
-        itemFuture.setEnabled( !isFirst && !isLast );
+
+        QString message;
+        KGuiItem itemFuture( i18n( "Also Delete &Future" ) );
+
+        if ( !isFirst && !isLast ) {
+          itemFuture.setEnabled( true );
+          message = i18n( "The calendar item \"%1\" recurs over multiple dates. "
+                          "Do you want to delete only the current one on %2, also "
+                          "future occurrences, or all its occurrences?",
+                          incidence->summary(),
+                          KGlobal::locale()->formatDate( itemDate ) );
+        } else {
+          itemFuture.setEnabled( false );
+          message = i18n( "The calendar item \"%1\" recurs over multiple dates. "
+                          "Do you want to delete only the current one on %2 "
+                          "or all its occurrences?",
+                          incidence->summary(),
+                          KGlobal::locale()->formatDate( itemDate ) );
+        }
 
         if ( !( isFirst && isLast ) ) {
           km = KOMessageBox::fourBtnMsgBox(
             this,
             QMessageBox::Warning,
-            i18n( "The calendar item \"%1\" recurs over multiple dates. "
-                  "Do you want to delete only the current one on %2, only all "
-                  "future recurrences, or all its recurrences?",
-                  incidence->summary(),
-                  KGlobal::locale()->formatDate( itemDate ) ),
+            message,
             i18n( "KOrganizer Confirmation" ),
-            KGuiItem( i18n( "Delete C&urrent" ) ),
+            KGuiItem ( i18n( "Delete C&urrent" ) ),
             itemFuture,
             KGuiItem( i18n( "Delete &All" ) ) );
         } else {
@@ -2416,4 +2536,18 @@ bool CalendarView::eventFilter( QObject *watched, QEvent *event )
   return KOrg::CalendarViewBase::eventFilter( watched, event );
 }
 
+void CalendarView::updateHighlightModes() {
+
+  KOrg::BaseView *view = mViewManager->currentView();
+  if ( view ) {
+    bool hiEvents;
+    bool hiTodos;
+    bool hiJournals;
+
+    view->getHighlightMode( hiEvents, hiTodos, hiJournals );
+    mDateNavigatorContainer->setHighlightMode( hiEvents,
+                                               hiTodos,
+                                               hiJournals );
+  }
+}
 #include "calendarview.moc"
