@@ -24,20 +24,25 @@
 #ifndef ALARMDIALOG_H
 #define ALARMDIALOG_H
 
-#include <kcal/incidence.h>
-
-#include <kdialog.h>
+#include <KDialog>
 
+#include <QPoint>
 #include <QTimer>
-#include <QDateTime>
 
-using namespace KCal;
+namespace KCal {
+  class Incidence;
+}
 
 class ReminderListItem;
 class KOEventViewer;
+
 class KComboBox;
+
+class QDateTime;
 class QTreeWidget;
+class QTreeWidgetItem;
 class QSpinBox;
+class QVBoxLayout;
 
 class AlarmDialog : public KDialog
 {
@@ -46,7 +51,8 @@ class AlarmDialog : public KDialog
     AlarmDialog( QWidget *parent = 0 );
     virtual ~AlarmDialog();
 
-    void addIncidence( Incidence *incidence, const QDateTime &reminderAt );
+    void addIncidence( KCal::Incidence *incidence, const QDateTime &reminderAt,
+                       const QString &displayText );
     void setRemindAt( const QDateTime &dt );
     void eventNotification();
 
@@ -69,6 +75,7 @@ class AlarmDialog : public KDialog
 
   private Q_SLOTS:
     void update();
+    void toggleDetails( QTreeWidgetItem *item, int column );
 
   private:
     bool startKOrganizer();
@@ -78,12 +85,15 @@ class AlarmDialog : public KDialog
     void updateButtons();
     void showDetails();
 
+    QVBoxLayout *mTopLayout;
     QTreeWidget *mIncidenceTree;
     KOEventViewer *mDetailView;
 
+    QPoint mPos;
     QSpinBox *mSuspendSpin;
     KComboBox *mSuspendUnit;
     QTimer mSuspendTimer;
+    QTreeWidgetItem *mLastItem;
 };
 
 #endif
