@@ -1,8 +1,8 @@
-#ifndef _KPILOT_FILEINSTALLWIDGET_H
-#define _KPILOT_FILEINSTALLWIDGET_H
+#ifndef KPILOT_FILEINSTALLWIDGET_H
+#define KPILOT_FILEINSTALLWIDGET_H
 /* fileInstallWidget.h			KPilot
 **
-** Copyright (C) 1998-2001 by Dan Pilone
+** Copyright (C) 1998-2001 by Dan Pilone <dan@kpilot.org>
 **
 ** This file defines the file install widget, which is the thing
 ** that accepts file drags for later installation into the Pilot.
@@ -24,67 +24,56 @@
 **
 ** You should have received a copy of the GNU General Public License
 ** along with this program in a file called COPYING; if not, write to
-** the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
-** MA 02111-1307, USA.
+** the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+** MA 02110-1301, USA.
 */
 
 /*
 ** Bug reports and questions can be sent to kde-pim@kde.org
 */
 
-#include "pilotComponent.h"
+#include "component_page_base.h"
 
-class QMultiLineEdit;
-class QPushButton;
-class QIconDragItem;
-
-class KIconView;
 class KPilotInstaller;
 class FileInstaller;
 
-class FileInstallWidget : public PilotComponent
+class FileInstallWidget : public ComponentPageBase
 {
 Q_OBJECT
 
 public:
-	FileInstallWidget(QWidget* parent, const QString& dbPath);
+	FileInstallWidget( QWidget* parent );
 	virtual ~FileInstallWidget();
 
-	// Pilot Component Methods:
-	void showComponent();
-	bool preHotSync(QString &);
-	void postHotSync();
+	// ComponentPageBase Methods:
+	void showPage();
+	void hidePage() {}
 
+private:
+	class Private;
+	Private *fP;
+	
+	bool fSaveFileList;
+	KPilotInstaller *fKPilotInstaller;
+	FileInstaller *fInstaller;
 
 signals:
 	void fileInstallWidgetDone();
 
-protected:
-	void setSaveFileList(bool saveIt) { fSaveFileList = saveIt; }
-	bool getSaveFileList() { return fSaveFileList; }
-
-	/* virtual */ void dragEnterEvent(QDragEnterEvent* event);
-	/* virtual */ void dropEvent(QDropEvent* drop);
-	/* virtual */ bool eventFilter (QObject *watched, QEvent *event );
-
-	KPilotInstaller* getPilotInstallerApp() { return fKPilotInstaller; }
-
-private:
-	KIconView  *fIconView;
-	bool        fSaveFileList;
-
-	KPilotInstaller* fKPilotInstaller;
-	FileInstaller *fInstaller;
-	QPushButton *clearButton,*addButton;
-
 protected slots:
-	void slotClearButton();
 	void slotAddFile();
-
-	void slotDropEvent(QDropEvent * drop, const QValueList<QIconDragItem> & lst);
-
-public slots:
+	void slotClearButton();
+	void slotDropEvent( QDropEvent *drop );
 	void refreshFileInstallList();
+
+protected:
+	bool eventFilter( QObject *watched, QEvent *event );
+	void contextMenu( QContextMenuEvent *event );
+	void dragEnterEvent( QDragEnterEvent* event );
+	void dropEvent( QDropEvent* drop );
+	void setSaveFileList( bool saveIt ) { fSaveFileList = saveIt; }
+	bool getSaveFileList() { return fSaveFileList; }
+	KPilotInstaller* getPilotInstallerApp() { return fKPilotInstaller; }
 };
 
 #endif
