@@ -14,23 +14,17 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
 #include "kcal_resourcexmlrpc.h"
 #include "kcal_resourcexmlrpcconfig.h"
+#include "kresources_export.h"
 
 #include <kglobal.h>
 #include <klocale.h>
 
 using namespace KCal;
 
-extern "C"
-{
-  void *init_kcal_xmlrpc()
-  {
-    KGlobal::locale()->insertCatalogue( "kres_xmlrpc" );
-    return new KRES::PluginFactory<ResourceXMLRPC, ResourceXMLRPCConfig>();
-  }
-}
+EXPORT_KRESOURCES_PLUGIN( ResourceXMLRPC, ResourceXMLRPCConfig, "kres_xmlrpc" )
