@@ -20,7 +20,7 @@
 #ifndef DELEGATESELECTOR_H
 #define DELEGATESELECTOR_H
 
-#include <kdialogbase.h>
+#include <kdialog.h>
 
 namespace KPIM {
   class AddresseeLineEdit;
@@ -31,7 +31,7 @@ class QCheckBox;
 /**
   Selection dialog for a delegate.
 */
-class DelegateSelector : public KDialogBase
+class DelegateSelector : public KDialog
 {
   Q_OBJECT
 
